#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "zpu3image4xi.py">
#<abstract4xi abstract = "ZPU 3D image management"/>
#<refer4xi reference = "image3d4xi.py"/>
#<verify4xi verification = "10" target = "zpu3image4xi5test.py"/>
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'ZPU3Image4Xi',                        # NACK 5.31  p76
            
            'copy3image4zpu',                      # NACK 5.3.3 p88
            'copy3imagetobuffer4zpu',              # NACK 5.3.4 p90
            'copy3buffertoimage4zpu',              # NACK 5.3.4 p92
            'create3image4zpu',                    # ACK 5.31  p78      

            'get3imageinfo4zpu',                   # ACK 5.3.6 p97 nb: modernized so that you can also access only selected elements, if you want           
            'get3supportedimageformats4zpu',       # ACK 5.3.2 p82 nb: a separate function to get format whereas using get3imageinfo4zpu you can access any element by key such as 'FORMAT' 
                              
            'manoeuvre3copyimage4zpu',             # ACK modernized 
            'manoeuvre3copyimagetobuffer4zpu',     # ACK modernized        
            'manoeuvre3copybuffertoimage4zpu',     # ACK modernized
            'manoeuvre3mapimage4zpu',              # NACK modernized                                           
            'manoeuvre3readimage4zpu',             # ACK modernized
            'manoeuvre3writeimage4zpu',            # ACK modernized            
            'map3image4zpu',                       # NACK 5.3.5 p94 
            
            'read3image4zpu',                      # ACK 5.3.3 p84  
            'retain3image4zpu',                    # ACK 5.4.1 p99
            'release3image4zpu',                   # ACK 5.4.1 p99

            'set3imagecallback4zpu',               # ACK modernized
            'set3image4zpu',                       # ACK modernized nb: with memorymap features you can process 3D images as files!                    

            'trace3image4zpu',                     # ACK modernized 
                                    
            'wait3image4zpu',                      # ACK modernized nb: checks Queue object and thereby causes delay doing something useful
            'write3image4zpu'                      # ACK 5.3.3 p84                 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

# -------------------------------------------------------------------------------------------------
# Buffer module                           
# -------------------------------------------------------------------------------------------------
from zpu2buffer4xi import ZPU2Buffer4Xi
from zpu2buffer4xi import create2buffer4zpu
from zpu2buffer4xi import create2subbuffer4zpu
from zpu2buffer4xi import get2bufferinfo4zpu
from zpu2buffer4xi import release2buffer4zpu
from zpu2buffer4xi import retain2buffer4zpu
from zpu2buffer4xi import set2buffer4zpu
from zpu2buffer4xi import trace2buffer4zpu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from zpu2commandqueue4xi import ZPU2CommandQueue4Xi
from zpu2commandqueue4xi import create2commandqueue4zpu 
from zpu2commandqueue4xi import flush2commandqueue4zpu
from zpu2commandqueue4xi import finish2commandqueue4zpu
from zpu2commandqueue4xi import get2commandqueueinfo4zpu
from zpu2commandqueue4xi import release2commandqueue4zpu
from zpu2commandqueue4xi import retain2commandqueue4zpu
from zpu2commandqueue4xi import set2commandqueue4zpu
from zpu2commandqueue4xi import trace2commandqueue4zpu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from zpu2context4xi import ZPU2Context4Xi 
from zpu2context4xi import create2context4zpu
from zpu2context4xi import get2context4zpu
from zpu2context4xi import release2context4zpu
from zpu2context4xi import retain2context4zpu
from zpu2context4xi import set2context4zpu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from zpu2device4xi import ZPU4Xi
from zpu2device4xi import init2zpu4xi 
from zpu2device4xi import set2zpu4xi
from zpu2device4xi import get2zpu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from zpu2event4xi import ZPU2Event4Xi
from zpu2event4xi import create2event4zpu
from zpu2event4xi import get2eventinfo4zpu
from zpu2event4xi import release2event4zpu
from zpu2event4xi import retain2event4zpu
from zpu2event4xi import set2eventcallback4zpu 
from zpu2event4xi import set2event4zpu
from zpu2event4xi import trace2event4zpu
from zpu2event4xi import wait2event4zpu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global zpu3image4xi                                                     # 3D-IMAGES FOR ZPU
global zpu3trace4xi                                                     # TRACE

global zpu3buffer4xi                                                    # BUFFER FOR ZPU

global control3readimage4zpu                                            # COMMAND AND CONTROL TO READ  3D IMAGES WITH ZPU  
global state3read4zpu                                                   # STATE FOR COMMAND AND CONTROL
global max3readstate4zpu                                                # MAX FOR STATE
global ack3readinfo4zpu                                                 # ACK FOR IMAGE INFO
global message3readget4zpu                                              # QUEUE GET
global message3readput4zpu                                              # QUEUE PUT
global message3readreceive4zpu                                          # PIPE RECEIVE
global message3readsend4zpu                                             # PIPE SEND

global control3writeimage4zpu                                           # COMMAND AND CONTROL TO WRITE 3D IMAGES WITH ZPU 
global state3write4zpu                                                  # STATE FOR COMMAND AND CONTROL
global max3writestate4zpu                                               # MAX FOR STATE
global ack3writeinfo4zpu                                                # ACK FOR IMAGE INFO
global message3writeget4zpu                                             # QUEUE GET
global message3writeput4zpu                                             # QUEUE PUT
global message3writereceive4zpu                                         # PIPE RECEIVE
global message3writesend4zpu                                            # PIPE SEND

global control3copyimage4zpu                                            # COMMAND AND CONTROL TO COPY 3D IMAGES WITH ZPU 
global state3copyimage4zpu                                              # STATE FOR COMMAND AND CONTROL
global max3copyimagestate4zpu                                           # MAX FOR STATE
global ack3copyimageinfo4zpu                                            # ACK FOR IMAGE INFO
global message3copyimageget4zpu                                         # QUEUE GET
global message3copyimageput4zpu                                         # QUEUE PUT
global message3copyimagereceive4zpu                                     # PIPE RECEIVE
global message3copyimagesend4zpu                                        # PIPE SEND

global control3copyimagetobuffer4zpu                                    # COMMAND AND CONTROL TO COPY 3D IMAGES TO BUFFER WITH ZPU 
global state3copyimagetobuffer4zpu                                      # STATE FOR COMMAND AND CONTROL
global max3copyimagetobufferstate4zpu                                   # MAX FOR STATE
global ack3copyimagetobufferinfo4zpu                                    # ACK FOR IMAGE INFO
global message3copyimagetobufferget4zpu                                 # QUEUE GET
global message3copyimagetobufferput4zpu                                 # QUEUE PUT
global message3copyimagetobufferreceive4zpu                             # PIPE RECEIVE
global message3copyimagetobuffersend4zpu                                # PIPE SEND

global control3copybuffertoimage4zpu                                    # COMMAND AND CONTROL TO COPY BUFFER TO 3D IMAGE WITH ZPU 
global state3copybuffertoimage4zpu                                      # STATE FOR COMMAND AND CONTROL
global max3copybuffertoimagestate4zpu                                   # MAX FOR STATE
global ack3copybuffertoimageinfo4zpu                                    # ACK FOR IMAGE INFO
global message3copybuffertoimageget4zpu                                 # QUEUE GET
global message3copybuffertoimageput4zpu                                 # QUEUE PUT
global message3copybuffertoimagereceive4zpu                             # PIPE RECEIVE
global message3copybuffertoimagesend4zpu                                # PIPE SEND                      

global control3mapimage4zpu                                            # COMMAND AND CONTROL TO READ  3D IMAGES WITH ZPU  
global state3map4zpu                                                   # STATE FOR COMMAND AND CONTROL
global max3mapstate4zpu                                                # MAX FOR STATE
global ack3mapinfo4zpu                                                 # ACK FOR IMAGE INFO
global message3mapget4zpu                                              # QUEUE GET
global message3mapput4zpu                                              # QUEUE PUT
global message3mapreceive4zpu                                          # PIPE RECEIVE
global message3mapsend4zpu                                             # PIPE SEND
            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace3image4zpu" usage = "
#
#   trace3zpu = None
#   try:
#       trace3zpu = trace3image4zpu() 
#       print('(Step 15B/25) OF ZPU3IMAGE4XI5TEST - TRACE OF 3D IMAGE FOR ZPU: ', trace3zpu)  
#   except:
#       print('(Step 15Z1/25) OF ZPU3IMAGE4XI5TEST *** FAILED IN TRACE OF 3D IMAGE FOR ZPU: ', trace3zpu)     
#">
# -------------------------------------------------------------------------------------------------   
def trace3image4zpu(task4trace3zpu=None):  

    '''
    creates trace of 3d image for zpu
    '''
    
    global zpu3trace4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace3image4zpu" 
    method2print = "trace3image4zpu"

    message2print = "{(Step 1A/33) started" + "; trace3image4zpu-started: " + str(trace3image4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # <function trace3image4zpu at 0x00000000031B1848> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace3zpu-to-trace3image4zpu: " + str(task4trace3zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # None 

    code3ack  = ' '
    code3nack = ' '    
    
    zpu3trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace3zpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY ZPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace3zpu = {}                                                                                            
            
        task4trace3zpu['CLASS']   = "trace3image4zpu"                                                               # ZPUIZE ZPU TRACE!
        task4trace3zpu['METHOD']  = "trace3image4zpu" 
        task4trace3zpu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace3image4zpu-failed " + str(zpu3trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace3zpu-set-by-trace3image4zpu to: " + str(task4trace3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace3image4zpu-failed {}; time: 06:24:23 10/10/11 Pacific Daylight Time}',
        #  'CLASS': 'trace3image4zpu',
        #  'METHOD': 'trace3image4zpu'}; time: 06:24:23 10/10/11 Pacific Daylight Time}
    
        code3ack = code3ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY ZPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace3zpu-try-by-trace3image4zpu: " + str(task4trace3zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    # 

    try:
        zpu3trace4xi ['CLASS']  = task4trace3zpu['CLASS']                                                      # ZPUIZE ZPU TRACE!
        zpu3trace4xi ['METHOD']  = task4trace3zpu['METHOD']
        zpu3trace4xi ['MESSAGE'] = task4trace3zpu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; zpu3trace4xi-set-by-trace3image4zpu to: " + str(zpu3trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  {'MESSAGE': '{(Step 2Z0/33) failed; trace3image4zpu-failed {}; time: 06:24:23 10/10/11 Pacific Daylight Time}',
        #  'CLASS': 'trace3image4zpu',
        #  'METHOD': 'trace3image4zpu'}; time: 06:24:23 10/10/11 Pacific Daylight Time}


        code3ack = code3ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; zpu-trace-failed-by-trace3image4zpu: " + str(task4trace3zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code3nack = code3nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE ZPU TRACE JATKA
        # =================================
        class2print   = task4trace3zpu['CLASS']                                                                # ZPUIZE ZPU TRACE!
        method2print  = task4trace3zpu['METHOD']
        
        message2print = task4trace3zpu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # ZPUIZED TRACE FOR OPENCL CALLBACK!
        # { trace3image4zpu : trace3image4zpu ; (Step 2Z0/33) failed; trace3image4zpu-failed {}; time: 06:24:23 10/10/11 Pacific Daylight Time}
        
        code3ack = code3ack + '3C-'         
    except:
        class2print = "trace3image4zpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace3image4zpu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-zpu-trace-failed-by-trace3image4zpu: " + str(task4trace3zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code3nack = code3nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace3image4zpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace3image4zpu"                
        message2print = "{(Step 3D/33) performed" + "; zpu-trace-performed-by-trace3image4zpu: " + str(task4trace3zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace3image4zpu-failed {}; time: 06:24:23 10/10/11 Pacific Daylight Time}', 
        #   'CLASS': 'trace3image4zpu', 
        #   'METHOD': 'trace3image4zpu'}; time: 06:24:23 10/10/11 Pacific Daylight Time}
        
        code3ack = code3ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; zpu-trace-failed-by-trace3image4zpu: " + str(task4trace3zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code3nack = code3nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; zpu3trace4xi-returning-by-trace3image4zpu: " + str(zpu3trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace3image4zpu-failed {}; time: 06:24:23 10/10/11 Pacific Daylight Time}',
    #  'CLASS': 'trace3image4zpu',
    #  'METHOD': 'trace3image4zpu'}; time: 06:24:23 10/10/11 Pacific Daylight Time}

    
    return zpu3trace4xi
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set3imagecallback4zpu" usage = "
#
#   print('(Step 14A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SET 3D IMAGE CALLBACK OF ZPU DEVICE ', time4xi())  
#   try:                               
#       response4setcallback3image = set3imagecallback4zpu()                                                                                                 
#       print('(Step 14B/700) OF ZPU3IMAGE4XI5TEST - SET 3D IMAGE CALLBACK OF ZPU DEVICE TO: ', response4setcallback3image)   
#       # 
#   except:
#       print('(Step 14Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SET 3D IMAGE CALLBACK OF ZPU DEVICE ', time4xi())  
# 
#">
# ------------------------------------------------------------------------------------------------- 
 
def set3imagecallback4zpu(task4set3imagecallback=None):

    '''
    registers callback function to zpu3image4xi
    '''
    global zpu3image4xi
    
    class2print = "set3imagecallback4zpu" 
    method2print = "set3imagecallback4zpu"
    
    message2print = "{(Step 1A/70) started" + "; set3imagecallback4zpu-started: " + str(set3imagecallback4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function trace3image4zpu at 0x00000000031B1848>
    
    try:
    
        zpu3image4xi ['CALLBACK'] = trace3image4zpu
        
        message2print = "{(Step 1B/70) set callback3zpu" + "; imagecallback4zpu-set-by-set3imagecallback4zpu: " + str(zpu3image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-zpu3image4xi-returning-by-set3imagecallback4zpu: " + str(zpu3image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace3image4zpu at 0x00000000031B1848> 
        
        return zpu3image4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback3image-failed-by-set3imagecallback4zpu: " + str(callback3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace3image4zpu-returning-by-set3imagecallback4zpu: " + str(trace3image4zpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace3image4zpu
            
#</function4xi> 




# -------------------------------------------------------------------------------------------------
#<function4xi function = "create3image4zpu" usage = "
#     
#   print('=' *33)          
#   print('(Step 2A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO CREATE 3D IMAGE4ZPU ', time4xi())      
#   print('=' *33)        
#
#   try:
#       response4create3Dimage = create3image4zpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES   
#       print('(Step 2B/700) OF ZPU3IMAGE4XI5TEST - CREATED 3D IMAGE4ZPU ', time4xi())
#       keys3response = response4create3Dimage.keys()
#       for key3response in sorted(keys3response):
#           print(key3response, ' : ', response4create3Dimage[key3response])
            # 
#   except:
#       print('(Step 2Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO CREATE 3D IMAGE4ZPU *** ', time4xi())
#
#">
# -------------------------------------------------------------------------------------------------   
def create3image4zpu(task4imagecreate3zpu=None):

    '''
    creates 3D image for ZPU
    '''
    
    global zpu3image4xi                                                                                              # 3D image of ZPU  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create3image4zpu" 
    method2print = "create3image4zpu"

    message2print = "{(Step 1A/70) started" + "; create3image4zpu-started: " + str(create3image4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create3image4zpu at 0x0000000004874648> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4imagecreate3zpu-to-create3image4zpu: " + str(task4imagecreate3zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 

    code3ack  = ' '
    code3nack = ' '    
    
    zpu3image4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4imagecreate3zpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO ZPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecreate3zpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4imagecreate3zpu-created-by-create3image4zpu to: " + str(task4imagecreate3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function3callback4zpu = trace3image4zpu                                                                 # CALLBACK FUNCTION FOR ZPU
            
            message2print = "{(Step 1D/70) initialized" + "; function3callback4zpu-created-by-create3image4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace3image4zpu at 0x000000000486C848>
            
            code3ack  = code3ack + '1D-'
           
            args3callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "create3image4zpu"                                                           # MODULE OF ZPUIZED OPENCL MODULE
            args3callback['METHOD']  = "create3image4zpu"                                                           # FUNCTION OF ZPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create3image4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args3callback-created-by-create3image4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create3image4zpu-failed ; time: 04:45:25 10/10/11 Pacific Daylight Time}',
            #  'CLASS': 'create3image4zpu',
            #  'METHOD': 'create3image4zpu'}; time: 04:45:25 10/10/11 Pacific Daylight Time}

            code3ack  = code3ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecreate3zpu['DATA3CALLBACK'] = args3callback                                                   # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecreate3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code3ack  = code3ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4imagecreate3zpu-created-by-create3image4zpu to: " + str(task4imagecreate3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'DATA3CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4zpu-failed ; time: 04:45:25 10/10/11 Pacific Daylight Time}',
            #  'CLASS': 'create3image4zpu',
            #  'METHOD': 'create3image4zpu'},
            #  'FUNCTION3CALLBACK': <function trace3image4zpu at 0x000000000486C848>}; time: 04:45:25 10/10/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-create3image4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-create3image4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 04:45:25 10/10/11 Pacific Daylight Time}
                 
                code3ack  = code3ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-create3image4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '2Z-'
                 
            task4imagecreate3zpu['PLATFORM'] = platform3xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context3zpu = {}
            message2print = "{(Step 3A/70) trying" + "; context3zpu-trying-by-create3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3zpu = get2context4zpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context3zpu-got-by-create3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 04:45:25 10/10/11 Pacific Daylight Time}', ...
                
                code3ack  = code3ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context3zpu-failed-by-create3image4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '3Z-'
                
            task4imagecreate3zpu ['CONTEXT'] = context3zpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device3zpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device3zpu-trying-by-create3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3zpu = get2zpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device3zpu-got-by-create3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR' ...
                
                code3ack  = code3ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device3zpu-failed-by-create3image4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '4Z-'
                
            task4imagecreate3zpu ['DEVICE'] = device3zpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4imagecreate3zpu ['CODE3ERROR'] = 0             
            code3ack  = code3ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue3zpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue3zpu-trying-by-create3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3zpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue3zpu-got-by-create3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x000000000482FD30> 
                
                code3ack  = code3ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue3zpu-failed-by-create3image4zpu: " + str(queue3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '6Z-'
                
            task4imagecreate3zpu ['QUEUE'] = queue3zpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe3zpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe3zpu-trying-by-create3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3zpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe3zpu-got-by-create3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>) 
                
                code3ack  = code3ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe3zpu-failed-by-create3image4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '7Z-'
                
            task4imagecreate3zpu ['PIPE'] = pipe3zpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3zpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo3zpu-trying-by-create3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo3zpu-got-by-create3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4zpu-failed ; time: 04:45:25 10/10/11 Pa ...
                
                code3ack  = code3ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo3zpu-failed-by-create3image4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '8Z-'
        
            task4imagecreate3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3zpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo3zpu-trying-by-create3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3zpu = get2bufferinfo4zpu()                                                                   # SYSTEM CREATES BUFFER FOR ZPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo3zpu-got-by-create3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4zpu-failed ; time: 04:45:25 10/10/11 Pacific  ...
                
                code3ack  = code3ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo3zpu-failed-by-create3image4zpu: " + str(bufferinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '9Z-'
        
            task4imagecreate3zpu ['BUFFER'] = bufferinfo3zpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo3zpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo3zpu-trying-by-create3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3zpu = get2eventinfo4zpu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo3zpu-got-by-create3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4zpu-failed ; time: 04:45:25 10/10/11 Pacific Daylight Time}', 'CLASS ...
                
                code3ack  = code3ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo3zpu-failed-by-create3image4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '10Z-'
        
            task4imagecreate3zpu ['EVENT'] = eventinfo3zpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) FLAGS - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            flags3zpu = 0
           
            message2print = "{(Step 11A/70) trying" + "; flags3zpu-trying-by-create3image4zpu: " + str(flags3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags3zpu = 0                                                              
                message2print = "{(Step 11B/70) set" + "; flags3zpu-set-by-create3image4zpu to: " + str(flags3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                code3ack  = code3ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; flags3zpu-failed-by-create3image4zpu: " + str(flags3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '11Z-'
        
            task4imagecreate3zpu ['FLAGS'] = flags3zpu             
            
            # ---------------------------------------------------------------------------------------------
            # (12) DEPTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            depth3zpu = 0

            quantity3core = 1                                                                      # ORIGIN 
            quantity3skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity3layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity3frame = 1                                                                     # FRAME
            quantity3extra = 4                                                                     # CONTROLS FOR GUI ETC...
            depth3layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 12A/70) trying" + "; depth3zpu-trying-by-create3image4zpu: " + str(depth3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                depth3zpu = int(depth3layer * (quantity3core + quantity3skeleton + quantity3layers + quantity3frame + quantity3extra))
                depth3zpu = 2 * depth3zpu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 12B/70) set" + "; depth3zpu-set-by-create3image4zpu to: " + str(depth3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800
                 
                code3ack  = code3ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; depth3zpu-failed-by-create3image4zpu: " + str(depth3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '12Z-'
        
            task4imagecreate3zpu ['DEPTH'] = depth3zpu
                        
            # ---------------------------------------------------------------------------------------------
            # (14) WIDTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            width3zpu = 0

            quantity3core = 1                                                                      # ORIGIN 
            quantity3skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity3layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity3frame = 1                                                                     # FRAME
            quantity3extra = 4                                                                     # CONTROLS FOR GUI ETC...
            width3layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 14A/70) trying" + "; width3zpu-trying-by-create3image4zpu: " + str(width3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                width3zpu = int(width3layer * (quantity3core + quantity3skeleton + quantity3layers + quantity3frame + quantity3extra))
                width3zpu = 2 * width3zpu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 14B/70) set" + "; width3zpu-set-by-create3image4zpu to: " + str(width3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800
                code3ack  = code3ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; width3zpu-failed-by-create3image4zpu: " + str(width3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '14Z-'
        
            task4imagecreate3zpu ['WIDTH'] = width3zpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) HEIGHT - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            height3zpu = 0
            quantity3core = 1                                                                      # ORIGIN 
            quantity3skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity3layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity3frame = 1                                                                     # FRAME
            quantity3extra = 4                                                                     # CONTROLS FOR GUI
            
            quantity3total = quantity3core + quantity3skeleton + quantity3layers + quantity3frame + quantity3extra
            height3layer = 10                                                                      # SCALED TO 10 
                       
            message2print = "{(Step 15A/70) trying" + "; height3zpu-trying-by-create3image4zpu: " + str(height3zpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
                        
            try:
                height3zpu = height3layer * quantity3total
                
                height3zpu = 2 * height3zpu                                                        # POS and NEG Y   2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 15B2/70) set" + "; height3zpu-set-by-create3image4zpu to: " + str(height3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800
                
                code3ack  = code3ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; height3zpu-failed-by-create3image4zpu: " + str(height3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '15Z-'
             
                        
            task4imagecreate3zpu ['HEIGHT'] = height3zpu 
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch3zpu = 0
            size3element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 1                                                                                  # ONE DIRECTIONS IN 3D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch3zpu-trying-by-create3image4zpu: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3zpu = int(width3zpu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch3zpu-set-by-create3image4zpu to: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                             
                 
                code3ack  = code3ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch3zpu-failed-by-create3image4zpu: " + str(rowpitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '16Z-'
        
            task4imagecreate3zpu ['ROWPITCH'] = rowpitch3zpu     
            
            # ---------------------------------------------------------------------------------------------
            # (17) SLICEPITCH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # the size in bytes of each 2D slice in the 3D image

            slicepitch3zpu = 0
            size3element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 1                                                                                  # ONE DIRECTIONS IN 3D
            
            message2print = "{(Step 17A/70) trying" + "; slicepitch3zpu-trying-by-create3image4zpu: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                slicepitch3zpu = int(width3zpu * size3element * voxelquantity3layer / quantity3direction)           # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 17B/70) set" + "; slicepitch3zpu-set-by-create3image4zpu to: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code3ack  = code3ack + '17B-' 
            except:
                message2print = "{(Step 17Z/70) failed" + "; slicepitch3zpu-failed-by-create3image4zpu: " + str(slicepitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '17Z-'
        
            task4imagecreate3zpu ['SLICEPITCH'] = slicepitch3zpu     
                        
            # ---------------------------------------------------------------------------------------------
            # (19) POINTER - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            pointer3zpu = None
            size3zpu = int(width3zpu * height3zpu)                                                                  # CALCULATED MEMORY
            if size3zpu == 0:
                size3zpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 19A/70) trying" + "; pointer3zpu-trying-by-create3image4zpu: " + str(pointer3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena3zpu = Arena4Xi(size3zpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 19B1/70) got" + "; arena3zpu-got-by-create3image4zpu: " + str(arena3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000042A2E48> 
                
                size3arena = arena3zpu.size                                                                         # 
                              
                message2print = "{(Step 19B2/70) got" + "; size3arena-got-by-create2buffer4zpu: " + str(size3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer3arena = arena3zpu.buffer
                
                message2print = "{(Step 19B3/70) got" + "; buffer3arena-got-by-create2buffer4zpu: " + str(buffer3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x0000000004295348>
                
                code3ack  = code3ack + '19B-' 
                
                if size3arena < size3zpu:
                    size3zpu = size3arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 19C/70) set" + "; size3zpu-set-by-create2buffer4zpu to: " + str(size3zpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code3ack  = code3ack + '19C-'
                else:
                    pass
                    
                pointer3zpu = buffer3arena
                message2print = "{(Step 19D/70) set" + "; pointer3zpu-set-by-create2buffer4zpu to: " + str(pointer3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x0000000004295348>
                
                code3ack  = code3ack + '19D-'       
            except:
                message2print = "{(Step 19Z/70) failed" + "; pointer3zpu-failed-by-create3image4zpu: " + str(pointer3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '19Z-'
        
            task4imagecreate3zpu ['POINTER'] = pointer3zpu                            
            task4imagecreate3zpu ['SIZE']    = size3zpu                                                             # EXTRA
            
            
            # ---------------------------------------------------------------------------------------------
            # (20) FORMAT - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            format3zpu = {}
           
            message2print = "{(Step 20A/70) trying" + "; format3zpu-trying-by-create3image4zpu: " + str(format3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                format3zpu['ORDER'] = 'CL_RGBA'                                                              
                message2print = "{(Step 20B/70) set" + "; format3zpu-set-by-create3image4zpu to: " + str(format3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'ORDER': 'CL_RGBA'}
                 
                code3ack  = code3ack + '20B-'
                
                format3zpu['TYPE'] = 'CL_UNORM_INT8'                                                              
                message2print = "{(Step 20C/70) set" + "; format3zpu-set-by-create3image4zpu to: " + str(format3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
                
                code3ack  = code3ack + '20C-'                
                 
            except:
                message2print = "{(Step 20Z/70) failed" + "; format3zpu-failed-by-create3image4zpu: " + str(format3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '20Z-'
        
            task4imagecreate3zpu ['FORMAT'] = format3zpu                  

                                                                                                                                       
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4imagecreate3zpu-yielded-by-create3image4zpu: " + str(task4imagecreate3zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys3task = task4imagecreate3zpu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys3task-using-by-create3image4zpu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE3ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA3CALLBACK',
    #  'DEPTH',
    #  'DEVICE',
    #  'EVENT',
    #  'FLAGS',
    #  'FORMAT',
    #  'FUNCTION3CALLBACK',
    #  'HEIGHT',
    #  'PIPE',
    #  'PLATFORM',
    #  'POINTER',
    #  'QUEUE',
    #  'ROWPITCH',
    #  'SIZE',
    #  'SLICEPITCH',
    #  'WIDTH']; time: 05:07:09 10/10/11 Pacific Daylight Time}

  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE 3D IMAGE FOR ZPU! 
    # ---------------------------------------------------------------------------------------------    
    zpu3image4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; zpu3image4xi-initialized-by-create3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    
    message2print = "{(Step 21A0/70) try" + "; data3callback-try-by-create3image4zpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA3CALLBACK - EXTRA 
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['DATA3CALLBACK'] = task4imagecreate3zpu['DATA3CALLBACK']
        
        message2print = "{(Step 21A1/70) initialized" + "; data3callback-created-by-create3image4zpu to: " + str(zpu3image4xi['DATA3CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 1Z1/33) failed; create3image4zpu-failed ; time: 05:07:09 10/10/11 Pacific Daylight Time}',
        #  'CLASS': 'create3image4zpu',
        #  'METHOD': 'create3image4zpu'}; time: 05:07:09 10/10/11 Pacific Daylight Time}

        code3ack  = code3ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data3callback-zpu3image4xi-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function3callback-try-by-create3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION3CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['FUNCTION3CALLBACK'] = task4imagecreate3zpu['FUNCTION3CALLBACK']
        
        message2print = "{(Step 22B1/70) initialized" + "; function3callback-created-by-create3image4zpu to: " + str(zpu3image4xi['FUNCTION3CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <function trace3image4zpu at 0x0000000004876848>  
        code3ack  = code3ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function3callback-zpu3image4xi-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform-try-by-create3image4zpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['PLATFORM'] = task4imagecreate3zpu['PLATFORM']
        
        message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create3image4zpu to: " + str(zpu3image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 05:07:09 10/10/11 Pacific Daylight Time}
  
        code3ack  = code3ack + '22C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-zpu3image4xi-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context3zpu-try-by-create3image4zpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['CONTEXT'] = task4imagecreate3zpu['CONTEXT']
        
        message2print = "{(Step 23B/70) initialized" + "; context-created-by-create3image4zpu to: " + str(zpu3image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 05:07:09 10/10/11 Pacific Daylight Time}', ' ...
        
        
        code3ack  = code3ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-zpu3image4xi-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code3nack  = code3nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-create3image4zpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['DEVICE'] = task4imagecreate3zpu['DEVICE']
        
        message2print = "{(Step 24B/70) initialized" + "; device-created-by-create3image4zpu to: " + str(zpu3image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR' ...
        
        code3ack  = code3ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-zpu3image4xi-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code3error-try-by-create3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['CODE3ERROR'] = task4imagecreate3zpu['CODE3ERROR']
        
        message2print = "{(Step 25B/70) initialized" + "; code3error-created-by-create3image4zpu to: " + str(zpu3image4xi['CODE3ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        
        code3ack  = code3ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code3error-zpu3image4xi-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-create3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['QUEUE'] = task4imagecreate3zpu['QUEUE']
        
        message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create3image4zpu to: " + str(zpu3image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x00000000048330B8>
         
        code3ack  = code3ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-zpu3image4xi-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-create3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['PIPE'] = task4imagecreate3zpu['PIPE']
        
        message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create3image4zpu to: " + str(zpu3image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 384>, <read-write PipeConnection, handle 388>) 
        
        code3ack  = code3ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-zpu3image4xi-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create3image4zpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['COMMANDQUEUE']  = task4imagecreate3zpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create3image4zpu to: " + str(zpu3image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4zpu-failed ; time: 05:07:09 10/10/11 Pa ...
        
        code3ack  = code3ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
              
        code3nack  = code3nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create3image4zpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['BUFFER']  = task4imagecreate3zpu['BUFFER']  
 
        message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create3image4zpu to: " + str(zpu3image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4zpu-failed ; time: 05:07:09 10/10/11 Pacific  ...
        
        code3ack  = code3ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create3image4zpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['EVENT']  = task4imagecreate3zpu['EVENT']  
 
        message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create3image4zpu to: " + str(zpu3image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4zpu-failed ; time: 05:07:09 10/10/11 Pacific Daylight Time}', 'CLASS ...
        
        code3ack  = code3ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-create3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['FLAGS']  = task4imagecreate3zpu['FLAGS']  
 
        message2print = "{(Step 31B/70) initialized" + "; flags-created-by-create3image4zpu to: " + str(zpu3image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0 
        
        code3ack  = code3ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '31Z1-'     

    message2print = "{(Step 32A/70) try" + "; depth-try-by-create3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (32) DEPTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        zpu3image4xi['DEPTH']  = task4imagecreate3zpu['DEPTH']  
 
        message2print = "{(Step 32H/70) initialized" + "; depth-created-by-create3image4zpu to: " + str(zpu3image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        
        code3ack  = code3ack + '32H-'
    except: 
        message2print = "{(Step 32Z1/70) yielded" + "; depth-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '32Z1-'   
        
    message2print = "{(Step 34A/70) try" + "; width-try-by-create3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        zpu3image4xi['WIDTH']  = task4imagecreate3zpu['WIDTH']  
 
        message2print = "{(Step 34H/70) initialized" + "; width-created-by-create3image4zpu to: " + str(zpu3image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        
        code3ack  = code3ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-create3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        zpu3image4xi['HEIGHT']  = task4imagecreate3zpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) initialized" + "; height-created-by-create3image4zpu to: " + str(zpu3image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        
        code3ack  = code3ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-create3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['ROWPITCH']  = task4imagecreate3zpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) initialized" + "; rowpitch-created-by-create3image4zpu to: " + str(zpu3image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        
        code3ack  = code3ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; slicepitch-try-by-create3image4zpu: None" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) SLICEPITCH - REQUIRED JATKA                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['SLICEPITCH']  = task4imagecreate3zpu['SLICEPITCH']  
 
        message2print = "{(Step 37B/70) initialized" + "; slicepitch-created-by-create3image4zpu to: " + str(zpu3image4xi['SLICEPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800
        
        code3ack  = code3ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; slicepitch-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '37Z1-'
        
    message2print = "{(Step 37A/70) try" + "; pointer-try-by-create3image4zpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['POINTER']  = task4imagecreate3zpu['POINTER']  
 
        message2print = "{(Step 38B/70) initialized" + "; pointer-created-by-create3image4zpu to: " + str(zpu3image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <mmap.mmap object at 0x000000000429E348> 
        
         
        code3ack  = code3ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; pointer-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; format-try-by-create3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['FORMAT']  = task4imagecreate3zpu['FORMAT']  
 
        message2print = "{(Step 39B/70) initialized" + "; format-created-by-create3image4zpu to: " + str(zpu3image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        
        code3ack  = code3ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; format-yielded-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '39Z1-'


                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    zpu3image4xi ['ACK2RETURN']  = code3ack
    zpu3image4xi ['NACK2RETURN'] = code3nack  
    
    message2print = "{(Step 70/70) returning" + "; zpu3image4xi-returning-by-create3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # {'PIPE': (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>),
    #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-17B-19B-19D-20B-20C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-32H-34H-35H-36B-37B-38B-39B-',
    #  'ROWPITCH': 800,
    #  'CODE3ERROR': 0,
    #  'WIDTH': 800,
    #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4zpu-failed ; time: 05:18:11 10/10/ ...
    
    
    return zpu3image4xi
            
#</function4xi> 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "set3image4zpu" usage = "
#    
#   print('(Step 4B/33) OF ZPU3IMAGE4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#   platform3xi = {}
#   try:
#       platform3xi = get2platformids4xi()
#       print('(Step 4B/33) of CONTEXT4ZPU5TEST - GOT PLATFORMIDS4XI', platform3xi) 
#   except:
#       print('(Step 4Z/33) of CONTEXT4ZPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#    
#   print('(Step 2A/33) OF DEVICE4ZPU5TEST - TRYING TO INITIALIZE DEVICE4ZPU ', time4xi())   
#   print('=' *33)        
#   try:
#       device3zpu = init2zpu4xi()                                                                                
#       print('(Step 2B/33) OF DEVICE4ZPU5TEST - INITIALIZED DEVICE4ZPU TO: ', device3zpu)        
#   except:
#       print('(Step 2Z/33) OF DEVICE4ZPU5TEST *** FAILED TO INITIALIZE DEVICE4ZPU', time4xi()) 
#
#   print('(Step 3A/33) OF ZPU3IMAGE4XI5TEST - TRYING TO CREATE CONTEXT4ZPU ', time4xi())    
#   print('=' *33)        
#   try:
#       context3zpu = create2context4zpu()                                                                               
#       print('(Step 3B/33) OF ZPU3IMAGE4XI5TEST - CREATED CONTEXT4ZPU: ', context3zpu)                                  
#   except:
#       print('(Step 3Z/33) OF ZPU3IMAGE4XI5TEST *** FAILED TO CREATE CONTEXT4ZPU', time4xi()) 
#          
#   task4set3image = {}
#   task4set3image  ['CONTEXT'] = context3zpu  
# 
#   ... other data for set operation ...
#    
#   
# ---------------------------------------------------------------------------------------------
# (11) SET 3D IMAGE                         
# ---------------------------------------------------------------------------------------------          
#   response4set3Dimage = {} 
#   print('(Step 11A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SET 3D IMAGE4ZPU ', time4xi())   
#   try:                               
#       response4set3Dimage = set3image4zpu(task4set3image)                                                                                                         
#                                                                                     
#       print('(Step 11B/700) OF ZPU3IMAGE4XI5TEST - SET 3D IMAGE4ZPU ', time4xi())
#       keys3response = response4set3Dimage.keys()
#       for key3response in sorted(keys3response):
#           print(key3response, ' : ', response4set3Dimage[key3response])
#           # 
#   except:
#       print('(Step 11Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SET 3D IMAGE4ZPU *** ', time4xi()) 
#        
#
#">
# -------------------------------------------------------------------------------------------------   
def set3image4zpu(task4imageset3zpu=None):

    '''
    sets 3D images for ZPU
    '''
    global zpu3image4xi  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set3image4zpu"
    method2print = "set3image4zpu"

    message2print = "{(Step 1A/33) started" + "; set3image4zpu-started: " + str(set3image4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set3image4zpu at 0x00000000032196C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageset3zpu-to-set3image4zpu: " + str(task4imageset3zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    #1
    # {'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>),
    #  'CODE3ERROR': 0,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004431B38>,
    #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 05:35:43 10/10/11 Pacific Dayl ...
    
    #2
    # {'HEIGHT': 1000,
    #  'WIDTH': 1100,
    #  'DEPTH': 900,
    #  'ROWPITCH': 901,
    #  'SLICEPITCH': 902,
    #  'DIMENSION': 3}; time: 06:03:06 10/10/11 Pacific Daylight Time}
    
            
    code3ack  = ' '
    code3nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if zpu3image4xi is None:
            zpu3image4xi = create3image4zpu()                                                                              # INITIALIZE IMAGE FOR ZPU 
            message2print = "{(Step 1C/70) set" + "; zpu3image4xi-set-by-set3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create3image4zpu-skipped-by-set3image4zpu: " + str(create3image4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function create3image4zpu at 0x0000000003219648>                                
    except:
        try:
            zpu3image4xi = create3image4zpu()
            message2print = "{(Step 1E/70) set" + "; zpu3image4xi-set-by-set3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            zpu3image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; zpu3image4xi-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data3callback-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA3CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['DATA3CALLBACK'] = task4imageset3zpu['DATA3CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data3callback-set-by-set3image4zpu to: " + str(zpu3image4xi['DATA3CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code3ack  = code3ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data3callback-zpu3image4xi-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code3nack  = code3nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function3callback-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION3CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['FUNCTION3CALLBACK'] = task4imageset3zpu['FUNCTION3CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function3callback-set-by-set3image4zpu to: " + str(zpu3image4xi['FUNCTION3CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code3ack  = code3ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function3callback-zpu3image4xi-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code3nack  = code3nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['PLATFORM'] = task4imageset3zpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set3image4zpu to: " + str(zpu3image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #   
        code3ack  = code3ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-zpu3image4xi-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code3nack  = code3nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context3zpu-try-by-set3image4zpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['CONTEXT'] = task4imageset3zpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set3image4zpu to: " + str(zpu3image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 05:35:43 10/10/11 Pacific Daylight Time}', ...
        
        code3ack  = code3ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-zpu3image4xi-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code3nack  = code3nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set3image4zpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['DEVICE'] = task4imageset3zpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set3image4zpu to: " + str(zpu3image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR' ...
        
        code3ack  = code3ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-zpu3image4xi-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code3error-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['CODE3ERROR'] = task4imageset3zpu['CODE3ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code3error-set-by-set3image4zpu to: " + str(zpu3image4xi['CODE3ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
         
        code3ack  = code3ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code3error-zpu3image4xi-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['QUEUE'] = task4imageset3zpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set3image4zpu to: " + str(zpu3image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000004431B38>
        code3ack  = code3ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-zpu3image4xi-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['PIPE'] = task4imageset3zpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set3image4zpu to: " + str(zpu3image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>)
        
        code3ack  = code3ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-zpu3image4xi-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set3image4zpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['COMMANDQUEUE']  = task4imageset3zpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set3image4zpu to: " + str(zpu3image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4zpu-failed ; time: 05:35:43 10/10/11 Pa ...
        
        code3ack  = code3ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set3image4zpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['BUFFER']  = task4imageset3zpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set3image4zpu to: " + str(zpu3image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
         
        code3ack  = code3ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code3nack  = code3nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set3image4zpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['EVENT']  = task4imageset3zpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set3image4zpu to: " + str(zpu3image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # yielded 
        code3ack  = code3ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['FLAGS']  = task4imageset3zpu['FLAGS']  
 
        message2print = "{(Step 31B/70) set" + "; flags-set-by-set3image4zpu to: " + str(zpu3image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # yielded
        code3ack  = code3ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '31Z1-'     

    message2print = "{(Step 32A/70) try" + "; depth-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (32) DEPTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        zpu3image4xi['DEPTH']  = task4imageset3zpu['DEPTH']  
 
        message2print = "{(Step 32H/70) set" + "; depth-set-by-set3image4zpu to: " + str(zpu3image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # yielded
        
        code3ack  = code3ack + '32H-'
    except: 
        message2print = "{(Step 32Z1/70) yielded" + "; depth-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '32Z1-'   

    message2print = "{(Step 34A/70) try" + "; width-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        zpu3image4xi['WIDTH']  = task4imageset3zpu['WIDTH']  
 
        message2print = "{(Step 34H/70) set" + "; width-set-by-set3image4zpu to: " + str(zpu3image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # yielded 
        
        code3ack  = code3ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-set3image4zpu: None"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        zpu3image4xi['HEIGHT']  = task4imageset3zpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) set" + "; height-set-by-set3image4zpu to: " + str(zpu3image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # yielded  
        code3ack  = code3ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['ROWPITCH']  = task4imageset3zpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) set" + "; rowpitch-set-by-set3image4zpu to: " + str(zpu3image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code3ack  = code3ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code3nack  = code3nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; slicepitch-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) SLICEPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['SLICEPITCH']  = task4imageset3zpu['SLICEPITCH']  
 
        message2print = "{(Step 37B/70) set" + "; slicepitch-set-by-set3image4zpu to: " + str(zpu3image4xi['SLICEPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code3ack  = code3ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; slicepitch-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code3nack  = code3nack + '37Z1-'
        
    message2print = "{(Step 38A/70) try" + "; pointer-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['POINTER']  = task4imageset3zpu['POINTER']  
 
        message2print = "{(Step 38B/70) set" + "; pointer-set-by-set3image4zpu to: " + str(zpu3image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code3ack  = code3ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; pointer-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code3nack  = code3nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; format-try-by-set3image4zpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (39) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        zpu3image4xi['FORMAT']  = task4imageset3zpu['FORMAT']  
 
        message2print = "{(Step 39B/70) set" + "; format-set-by-set3image4zpu to: " + str(zpu3image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # yielded
        
        code3ack  = code3ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; format-yielded-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code3nack  = code3nack + '39Z1-'
        
         
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    zpu3image4xi ['ACK2RETURN']  = code3ack
    zpu3image4xi ['NACK2RETURN'] = code3nack 
    
    message2print = "{(Step 70/70) returning" + "; zpu3image4xi-returning-by-set3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1:
    # ACK2RETURN  :   23B-24B-25B-26B-27B-28B-
    # BUFFER  :  {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4zpu-failed ; time: 05:35:43 10/10/11 ...
    # CODE3ERROR  :  0
    # COMMANDQUEUE  :  {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4zpu-failed ; time: 05: ...
    # CONTEXT  :  {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 05:35:43 10/10/11 Pacific Dayl ...
    # DATA3CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create3image4zpu-failed ; time: 05:35:43 10/10/11 Pacific Daylight Time}', 'CLASS': 'create3image4 ...
    # DEPTH  :  800
    # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHL ...
    # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4zpu-failed ; time: 05:35:43 10/10/11 Pacific Daylight Time ...
    # FLAGS  :  0
    # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # FUNCTION3CALLBACK  :  <function trace3image4zpu at 0x0000000003213848>
    # HEIGHT  :  800
    # NACK2RETURN  :   21Z1-22Z2-22Z3-29Z1-30Z1-31Z1-32Z1-34Z1-35Z1-36Z1-37Z1-38Z1-
    # PIPE  :  (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>)
    # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
    # POINTER  :  <mmap.mmap object at 0x000000000427B3F8>
    # QUEUE  :  <queue4xi.Queue4Xi object at 0x0000000004431B38>
    # ROWPITCH  :  800
    # SLICEPITCH  :  800
    # WIDTH  :  800
    
    #2:
    # {'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 'ACK2RETURN': ' 32H-34H-35H-36B-37B-', 'ROWPITCH': 901, 'CODE3ERROR': 0, 'WIDTH': 1100,
    # 'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4zpu-failed ; time: 06:24:23 10/10/1 ...
    #  'DEPTH': 900, 'FLAGS': 0, 
    # 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 06:24:23 10/10/11 Pacific Dayli ...
    
    
    return zpu3image4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy3image4zpu" usage = "
#                                                                                                              
#   # -----------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH COPY3IMAGE4ZPU                      
#   # -----------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY3IMAGE4ZPU ', time4xi())    
#   response4copy3image = {} 
#   task4copy3image = None
#   try:                               
#       response4copy3image = copy3image4zpu(task4copy3image)                                                                                                          
#       print('(Step 16B/700) OF ZPU3IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY3IMAGE4ZPU : ', response4copy3image)
#       queue3copy4zpu      = response4copy3image['QUEUE']
#       channel3copypublish4zpu = response4copy3image['PUBLISHERCHANNEL']
#       channel3copyconsume4zpu = response4copy3image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY3IMAGE4ZPU ', time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE3IMAGE4ZPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGE4ZPU ', time4xi())     
#   try:                               
#       message3copyimagesend4zpu = "<copy4xi>copy3copy4zpu</copy4xi>"
#       channel3copypublish4zpu = response4copy3image['PUBLISHERCHANNEL']
#       channel3copypublish4zpu.send(message3copyimagesend4zpu)
#       print("(Step 17B/700) OF ZPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4ZPU: ", channel3copypublish4zpu, " MESSAGE3SEND4ZPU: " , message3copyimagesend4zpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGE4ZPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE3IMAGE4ZPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvre4zpu = manoeuvre3copyimage4zpu(queue3copy4zpu, channel3copyconsume4zpu, channel3copypublish4zpu)
#       print("(Step 18D/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", manoeuvre3copyimage4zpu)
#   except:
#       print("(Step 18Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # -----------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY3IMAGE4ZPU ', time4xi())     
#   try:
#       channel3copyconsume4zpu = response4copy3image['CONSUMERCHANNEL']                              
#       message3copyimagereceive4zpu = channel3copyconsume4zpu.recv()
#       print("(Step 19B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3copyconsume4zpu, " CONTROL & COMMAND: ", message3copyimagereceive4zpu)                                                                                                  
#       # <roger4xi>roger3copy4zpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY3IMAGE4ZPU ', time4xi())         
#   # -----------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE3IMAGE4ZPU TO RECEIVE                
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGE4ZPU ', time4xi())     
#   try:                               
#       message3copyimagesend4zpu = "<copy4xi>copy3copy4zpu</copy4xi>"
#       channel3copypublish4zpu = response4copy3image['PUBLISHERCHANNEL']
#       channel3copypublish4zpu.send(message3copyimagesend4zpu)
#       print("(Step 20B/700) OF ZPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4ZPU: ", channel3copypublish4zpu, " MESSAGE3SEND4ZPU: " , message3copyimagesend4zpu)                                                                                                
#       # <copy4xi>copy3copy4zpu</copy4xi> 
#   except:
#        print('(Step 20Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGE4ZPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE3IMAGE4ZPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvre4zpu = manoeuvre3copyimage4zpu(queue3copy4zpu, channel3copyconsume4zpu, channel3copypublish4zpu)
#       print("(Step 21B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", manoeuvre3copyimage4zpu)
#   except:
#       print("(Step 21Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY3IMAGE4ZPU ', time4xi())     
#   try:
#       channel3copyconsume4zpu = response4copy3image['CONSUMERCHANNEL']                              
#       message3copyimagereceive4zpu = channel3copyconsume4zpu.recv()
#       print("(Step 22B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3copyconsume4zpu, " CONTROL & COMMAND: ", message3copyimagereceive4zpu)                                                                                                  
#       # <copy4xi>copy3copy4zpu</copy4xi> 
#   except:
#       print('(Step 22Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY3IMAGE4ZPU ', time4xi())          
#   # ----------------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # ----------------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue3copy4zpu.put(path3zpu)             
#       print("(Step 24B/700) OF ZPU3IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path3zpu) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE3IMAGE4ZPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvre4zpu = manoeuvre3copyimage4zpu(queue3copy4zpu, channel3copyconsume4zpu, channel3copypublish4zpu)
#       print("(Step 25B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", manoeuvre3copyimage4zpu)
#   except:
#       print("(Step 25Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message3copyimagereceive4zpu = channel3copyconsume4zpu.recv()
#       print("(Step 26D/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3copyconsume4zpu, " CONTROL & COMMAND: ", message3copyimagereceive4zpu)                                                                                                   
#       # <ack4xi>ack3imageinfo4zpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -----------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE3IMAGE4ZPU GO TO STATE 0               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvre4zpu = manoeuvre3copyimage4zpu(queue3copy4zpu, channel3copyconsume4zpu, channel3copypublish4zpu)
#       print("(Step 27B/700) OF ZPU3IMAGE4XI5TEST - GOT RESPONSE3MANOEUVRE4ZPU ", response3manoeuvre4zpu)
#   except:
#       print("(Step 27Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi())   

#">
# -------------------------------------------------------------------------------------------------   
def copy3image4zpu(task4imagecopy3zpu=None):

    '''
    copies 3D images for ZPU
    '''
    global zpu3image4xi                                                                                             # 3D-IMAGES FOR ZPU  
    global control3copyimage4zpu                                                                                    # COMMAND AND CONTROL FOR ZPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy3image4zpu"
    method2print = "copy3image4zpu"

    message2print = "{(Step 0A/33) started" + "; copy3image4zpu-started: " + str(copy3image4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function copy3image4zpu at 0x000000000321A748>
    
    message2print = "{(Step 0B/33) passed" + "; task4imagecopy3zpu-to-copy3image4zpu: " + str(task4imagecopy3zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE ZPU3IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if zpu3image4xi is None:
            zpu3image4xi = create3image4zpu()                                                                       # INITIALIZE IMAGE FOR ZPU 
            message2print = "{(Step 0C/70) created" + "; zpu3image4xi-created-by-copy3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        else:
            message2print = "{(Step 0D/70) skipped" + "; create3image4zpu-skipped-by-copy3image4zpu: " + str(copy3image4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  <function copy3image4zpu at 0x000000000321A748>                               
    except:
        try:
            zpu3image4xi = create3image4zpu()
            message2print = "{(Step 0E/70) set" + "; zpu3image4xi-set-by-copy3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            zpu3image4xi = {}
            message2print = "{(Step 0Z/70) yielded" + "; zpu3image4xi-yielded-by-copy3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '0Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagecopy3zpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecopy3zpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagecopy3zpu-created-by-copy3image4zpu to: " + str(task4imagecopy3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {} 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4zpu = trace3image4zpu                                                                 # CALLBACK FUNCTION FOR ZPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function3callback4zpu-created-by-copy3image4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace3image4zpu at 0x0000000003214848> 
            
            code3ack  = code3ack + '1B1-'
           
            args3callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "copy3image4zpu"                                                            # MODULE OF ZPUIZED OPENCL MODULE
            args3callback['METHOD']  = "copy3image4zpu"                                                            # FUNCTION OF ZPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy3image4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args3callback-created-by-copy3image4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; copy3image4zpu-failed ; time: 07:14:55 10/10/11 Pacific Daylight Time}', 
            #   'CLASS': 'copy3image4zpu', 
            #   'METHOD': 'copy3image4zpu'}; time: 07:14:55 10/10/11 Pacific Daylight Time}
            

            code3ack  = code3ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy3zpu['DATA3CALLBACK'] = args3callback                                                    # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagecopy3zpu-created-by-copy3image4zpu to: " + str(task4imagecopy3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-copy3image4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-copy3image4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code3ack  = code3ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-copy3image4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '2Z-'
                 
            task4imagecopy3zpu['PLATFORM'] = platform3xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3zpu = {}
            message2print = "{(Step 3A/70) trying" + "; context3zpu-trying-by-copy3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3zpu = get2context4zpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context3zpu-got-by-copy3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context3zpu-failed-by-copy3image4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '3Z-'
                
            task4imagecopy3zpu ['CONTEXT'] = context3zpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3zpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device3zpu-trying-by-copy3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3zpu = get2zpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device3zpu-got-by-copy3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device3zpu-failed-by-copy3image4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '4Z-'
            
            task4imagecopy3zpu ['DEVICE'] = device3zpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE SOURCE OBJECT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------         
            imagesourceinfo3zpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imagesourceinfo3zpu-trying-by-copy3image4zpu: " + str(imagesourceinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagesourceinfo3zpu = get3imageinfo4zpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imagesourceinfo3zpu-got-by-copy3image4zpu: " + str(imagesourceinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo3zpu-failed-by-copy3image4zpu: " + str(imagesourceinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '5Z-'          
                      
            task4imagecopy3zpu ['IMAGESOURCE'] = imagesourceinfo3zpu         
                   
            # ---------------------------------------------------------------------------------------------
            # (6) IMAGE DESTINATION OBJECT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------                         
            imagedestinationinfo3zpu = {}
             
            message2print = "{(Step 6A/70) trying" + "; imagedestinationinfo3zpu-trying-by-copy3image4zpu: " + str(imagedestinationinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagedestinationinfo3zpu = get3imageinfo4zpu()                                                                         
                
                message2print = "{(Step 6B/70) got" + "; imagedestinationinfo3zpu-got-by-copy3image4zpu: " + str(imagedestinationinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                # note: source and destination will be the same !             
                
                code3ack  = code3ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; imagedestinationinfo3zpu-failed-by-copy3image4zpu: " + str(imagedestinationinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '6Z-'          
                      
            task4imagecopy3zpu ['IMAGEDESTINATION'] = imagedestinationinfo3zpu     
            
            # ---------------------------------------------------------------------------------------------
            # (7) QUEUE - REQUIRED BY ZPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3zpu = None
            
            message2print = "{(Step 7A/70) trying" + "; queue3zpu-trying-by-copy3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3zpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 7B/70) got" + "; queue3zpu-got-by-copy3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; queue3zpu-failed-by-copy3image4zpu: " + str(queue3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '7Z-'
                
            task4imagecopy3zpu ['QUEUE'] = queue3zpu

            
            # ---------------------------------------------------------------------------------------------
            # (8) PIPE - REQUIRED BY ZPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3zpu = None
           
            message2print = "{(Step 8A/70) trying" + "; pipe3zpu-trying-by-copy3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3zpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 8B/70) got" + "; pipe3zpu-got-by-copy3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; pipe3zpu-failed-by-copy3image4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '8Z-'
                
            task4imagecopy3zpu ['PIPE'] = pipe3zpu 
            
            # ---------------------------------------------------------------------------------------------
            # (9) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3zpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; commandqueueinfo3zpu-trying-by-copy3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 9B/70) got" + "; commandqueueinfo3zpu-got-by-copy3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '9B-'  
            except:
                message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo3zpu-failed-by-copy3image4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '9Z-'
        
            task4imagecopy3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (10) BUFFER - AUTOMATIC BY ZPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3zpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; bufferinfo3zpu-trying-by-copy3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3zpu = get2bufferinfo4zpu()                                                               # SYSTEM CREATES BUFFER FOR ZPU, IF NECESSARY
                message2print = "{(Step 10B/70) got" + "; bufferinfo3zpu-got-by-copy3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; bufferinfo3zpu-failed-by-copy3image4zpu: " + str(bufferinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '10Z-'
        
            task4imagecopy3zpu ['BUFFER'] = bufferinfo3zpu 
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3zpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventinfo3zpu-trying-by-copy3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3zpu = get2eventinfo4zpu()                                                                    
                message2print = "{(Step 11B/70) got" + "; eventinfo3zpu-got-by-copy3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventinfo3zpu-failed-by-copy3image4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '11Z-'
        
            task4imagecopy3zpu ['EVENT'] = eventinfo3zpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3zpu = {}
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlist3zpu-trying-by-copy3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3zpu = {}                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlist3zpu-set-by-copy3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlist3zpu-failed-by-copy3image4zpu: " + str(eventwaitlist3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '12Z-'
        
            task4imagecopy3zpu ['EVENTWAITLIST'] = eventwaitlist3zpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (13) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3zpu = 0
            
            message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity3zpu-trying-by-copy3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3zpu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3zpu = eventwaitlistquantity3zpu + 1    
                
                message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity3zpu-set-by-copy3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  
                
                code3ack  = code3ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity3zpu-failed-by-copy3image4zpu: " + str(eventwaitlistquantity3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '13Z-'
        
            task4imagecopy3zpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3zpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN FOR SOURCE - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin4source3zpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin4source3zpu-trying-by-copy3image4zpu: " + str(origin4source3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4source3zpu['X'] = 0 
                origin4source3zpu['Y'] = 0  
                origin4source3zpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin4source3zpu-set-by-copy3image4zpu to: " + str(origin4source3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin4source3zpu-failed-by-copy3image4zpu: " + str(origin4source3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '14Z-'
        
            task4imagecopy3zpu ['SOURCEORIGIN'] = origin4source3zpu
            
            # ---------------------------------------------------------------------------------------------
            # (15) ORIGIN FOR DESTINATION - REQUIRED                             
            # ---------------------------------------------------------------------------------------------              
            origin4destination3zpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; origin4destination3zpu-trying-by-copy3image4zpu: " + str(origin4destination3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4destination3zpu['X'] = 100 
                origin4destination3zpu['Y'] = 100  
                origin4destination3zpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; origin4destination3zpu-set-by-copy3image4zpu to: " + str(origin4destination3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; origin4destination3zpu-failed-by-copy3image4zpu: " + str(origin4destination3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '15Z-'
        
            task4imagecopy3zpu ['DESTINATIONORIGIN'] = origin4destination3zpu
                        
            
            
            # --------------------------------------------------------------------------------------------------------
            # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
            # --------------------------------------------------------------------------------------------------------             
            region4copy3zpu = {}                                                                                                         
           
            message2print = "{(Step 16A/70) trying" + "; region4copy3zpu-trying-by-copy3image4zpu: " + str(region4copy3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                region4copy3zpu['X'] = 10 
                region4copy3zpu['Y'] = 10  
                region4copy3zpu['Z'] = 10                                                                           
                message2print = "{(Step 16B/70) set" + "; region4copy3zpu-set-by-copy3image4zpu to: " + str(region4copy3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code3ack  = code3ack + '16B-'                
                 
            except:
                message2print = "{(Step 16Z/70) failed" + "; region4copy3zpu-failed-by-copy3image4zpu: " + str(region4copy3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '16Z-'
            
            task4imagecopy3zpu ['REGION'] = region4copy3zpu              
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imagecopy3zpu.keys()
        message2print = "{(Step 20/70) using" + "; keys3task-using-by-copy3image4zpu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
 
        if (('DATA3CALLBACK' in keys3task) and ('FUNCTION3CALLBACK' in keys3task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4zpu = trace3image4zpu                                                            # CALLBACK FUNCTION FOR ZPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function3callback4zpu-created-by-copy3image4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code3ack  = code3ack + '21B1-'
           
            args3callback = {}                                                                                 # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "copy3image4zpu"                                                        # MODULE OF ZPUIZED OPENCL MODULE
            args3callback['METHOD']  = "copy3image4zpu"                                                        # FUNCTION OF ZPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy3image4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    
            
            message2print = "{(Step 21B2/70) initialized" + "; args3callback-created-by-copy3image4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code3ack  = code3ack + '21B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy3zpu['DATA3CALLBACK'] = args3callback                                                # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
                
            code3ack  = code3ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagecopy3zpu-created-by-copy3image4zpu to: " + str(task4imagecopy3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                 
        if 'PLATFORM' in keys3task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform3xi-created-by-copy3image4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform3xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform3xi-got-by-copy3image4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code3ack  = code3ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform3xi-failed-by-copy3image4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '22Z-'
                 
            task4imagecopy3zpu['PLATFORM'] = platform3xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3zpu = {}
            message2print = "{(Step 23A/70) trying" + "; context3zpu-trying-by-copy3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3zpu = get2context4zpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context3zpu-got-by-copy3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context3zpu-failed-by-copy3image4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '23Z-'
                
            task4imagecopy3zpu ['CONTEXT'] = context3zpu           
        
        if 'DEVICE' in keys3task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3zpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device3zpu-trying-by-copy3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3zpu = get2zpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device3zpu-got-by-copy3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device3zpu-failed-by-copy3image4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '24Z-'
            
            task4imagecopy3zpu ['DEVICE'] = device3zpu
        
        if 'IMAGESOURCE' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) SOURCE IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourceimageinfo3zpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; sourceimageinfo3zpu-trying-by-copy3image4zpu: " + str(sourceimageinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourceimageinfo3zpu = get3imageinfo4zpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; sourceimageinfo3zpu-got-by-copy3image4zpu: " + str(sourceimageinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo3zpu-failed-by-copy3image4zpu: " + str(sourceimageinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '25Z-'          
                      
            task4imagecopy3zpu ['IMAGESOURCE'] = sourceimageinfo3zpu         
        
        if 'IMAGEDESTINATION' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo3zpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; destinationimageinfo3zpu-trying-by-copy3image4zpu: " + str(destinationimageinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo3zpu = get3imageinfo4zpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; destinationimageinfo3zpu-got-by-copy3image4zpu: " + str(destinationimageinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; destinationimageinfo3zpu-failed-by-copy3image4zpu: " + str(destinationimageinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '26Z-'          
                      
            task4imagecopy3zpu ['IMAGEDESTINATION'] = destinationimageinfo3zpu    
                                    
        if 'QUEUE' in keys3task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY ZPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3zpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue3zpu-trying-by-copy3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3zpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue3zpu-got-by-copy3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue3zpu-failed-by-copy3image4zpu: " + str(queue3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '27Z-'
                
            task4imagecopy3zpu ['QUEUE'] = queue3zpu

        if 'PIPE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY ZPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3zpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe3zpu-trying-by-copy3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3zpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe3zpu-got-by-copy3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe3zpu-failed-by-copy3image4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '28Z-'
                
            task4imagecopy3zpu ['PIPE'] = pipe3zpu 
            
        if 'COMMANDQUEUE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3zpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo3zpu-trying-by-copy3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo3zpu-got-by-copy3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo3zpu-failed-by-copy3image4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '29Z-'
        
            task4imagecopy3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
        
        
        if 'BUFFER' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) BUFFER - AUTOMATIC BY ZPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3zpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; bufferinfo3zpu-trying-by-copy3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3zpu = get2bufferinfo4zpu()                                                           # SYSTEM CREATES BUFFER FOR ZPU, IF NECESSARY
                message2print = "{(Step 30B/70) got" + "; bufferinfo3zpu-got-by-copy3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; bufferinfo3zpu-failed-by-copy3image4zpu: " + str(bufferinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '30Z-'
        
            task4imagecopy3zpu ['BUFFER'] = bufferinfo3zpu
             
        if 'EVENT' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3zpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo3zpu-trying-by-copy3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3zpu = get2eventinfo4zpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo3zpu-got-by-copy3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo3zpu-failed-by-copy3image4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '31Z-'
        
            task4imagecopy3zpu ['EVENT'] = eventinfo3zpu 
        
             
        if 'EVENTWAITLIST' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3zpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist3zpu-trying-by-copy3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3zpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist3zpu-set-by-copy3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist3zpu-failed-by-copy3image4zpu: " + str(eventwaitlist3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '32Z-'
        
            task4imagecopy3zpu ['EVENTWAITLIST'] = eventwaitlist3zpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------
            eventwaitlistquantity3zpu = 0
            
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity3zpu-trying-by-copy3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3zpu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3zpu = eventwaitlistquantity3zpu + 1    
                
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity3zpu-set-by-copy3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code3ack  = code3ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity3zpu-failed-by-copy3image4zpu: " + str(eventwaitlistquantity3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '33Z-'
                    
            task4imagecopy3zpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3zpu       
                                             
                    
        if 'SOURCEORIGIN' in keys3task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) SOURCEORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            source4origin3zpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; source4origin3zpu-trying-by-copy3image4zpu: " + str(source4origin3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                source4origin3zpu['X'] = 0 
                source4origin3zpu['Y'] = 0  
                source4origin3zpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; source4origin3zpu-set-by-copy3image4zpu to: " + str(source4origin3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; source4origin3zpu-failed-by-copy3image4zpu: " + str(source4origin3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '34Z-'
        
            task4imagecopy3zpu ['SOURCEORIGIN'] = source4origin3zpu
             
        if 'DESTINATIONORIGIN' in keys3task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (35) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin3zpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; destination4origin3zpu-trying-by-copy3image4zpu: " + str(destination4origin3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin3zpu['X'] = 100 
                destination4origin3zpu['Y'] = 100  
                destination4origin3zpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; destination4origin3zpu-set-by-copy3image4zpu to: " + str(destination4origin3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; destination4origin3zpu-failed-by-copy3image4zpu: " + str(destination4origin3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '35Z-'
        
            task4imagecopy3zpu ['DESTINATIONSOURCEORIGIN'] = destination4origin3zpu                
             
        if 'REGION' in keys3task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3zpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region3zpu-trying-by-copy3image4zpu: " + str(region3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region3zpu['X'] = 330                                                                               # WIDTH  IN PIXELS TO WRITE 
                region3zpu['Y'] = 330                                                                               # HEIGHT IN PIXELS TO WRITE
                region3zpu['Z'] = 330                                                                               # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region3zpu-set-by-copy3image4zpu to: " + str(region3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region3zpu-failed-by-copy3image4zpu: " + str(region3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '36Z-'
        
            task4imagecopy3zpu ['REGION'] = region3zpu
        
        # ---------------------------------------------------------------------------------------------
        #  EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imagecopy3zpu.keys()
        message2print = "{(Step 39/70) set" + "; keys3task-set-by-copy3image4zpu to: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagecopy3zpu-yielded-by-copy3image4zpu: " + str(task4imagecopy3zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control3copyimage4zpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels3zpu = False
    channels3zpu = ()
    message2print = "{(Step 40A/70) trying" + "channels3zpu-trying-by-copy3image4zpu: " + str(channels3zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel3zpu, publisherchannel3zpu = task4imagecopy3zpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel3zpu-got-by-copy3image4zpu: " + str(consumerchannel3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel3zpu-got-by-copy3image4zpu: " + str(publisherchannel3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                              
        ack4channels3zpu = True
        
        control3copyimage4zpu['CONSUMERCHANNEL']  = consumerchannel3zpu
        control3copyimage4zpu['PUBLISHERCHANNEL'] = publisherchannel3zpu
        
        code3ack  = code3ack + '40D-' 
        
    except:
        ack4channels3zpu = False
        code3nack  = code3nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue3zpu = False
    queue3zpu = None
    message2print = "{(Step 41A/70) trying" + "queue3zpu-trying-by-copy3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue3zpu = task4imagecopy3zpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue3zpu-got-by-copy3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue3zpu = True
        control3copyimage4zpu['QUEUE']  = queue3zpu        
        code3ack  = code3ack + '41C-'
    except:
        ack4queue3zpu = False  
        code3nack  = code3nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy3image4zpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy3image4zpu-trying-by-copy3image4zpu: " + str(dummy3image4zpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy3image4zpu = DummyProcess4Xi(target=manoeuvre3copyimage4zpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue3zpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel3zpu, publisherchannel3zpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy3image4zpu-created-by-copy3image4zpu: " + str(dummy3image4zpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code3ack  = code3ack + '69B-'
        
    # except:
        # code3nack  = code3nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code3ack-set-by-copy3image4zpu: " + str(code3ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    
    message2print = "{(Step 70B/70) set" + "; code3nack-set-by-copy3image4zpu: " + str(code3nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control3copyimage4zpu-returning-by-copy3image4zpu: " + str(control3copyimage4zpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000430E710>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 684>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 680>}; time: 07:14:56 10/10/11 Pacific Daylight Time}


    return control3copyimage4zpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy3imagetobuffer4zpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (60) ESTABLISH CONTROL & COMMAND WITH COPY3IMAGETOBUFFER4ZPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 60A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY3IMAGETOBUFFER4ZPU ', time4xi())    
#   response4copy3imagetobuffer = {} 
#   task4copy3imagetobuffer = None
#   try:                               
#       response4copy3imagetobuffer = copy3imagetobuffer4zpu(task4copy3imagetobuffer)                                                                                                          
#       print('(Step 60B/700) OF ZPU3IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY3IMAGETOBUFFER4ZPU : ', response4copy3imagetobuffer)
#       queue3copyimagetobuffer4zpu      = response4copy3imagetobuffer['QUEUE']
#       channel3copyimagetobufferpublish4zpu = response4copy3imagetobuffer['PUBLISHERCHANNEL']
#       channel3copyimagetobufferconsume4zpu = response4copy3imagetobuffer['CONSUMERCHANNEL']
#   except:
#       print('(Step 60Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY3IMAGETOBUFFER4ZPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (61) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE3COPYIMAGETOBUFFER4ZPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 61A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGETOBUFFER4ZPU ', time4xi())     
#   try:                               
#       message3copyimagetobuffersend4zpu = "<copy4xi>copy3copyimagetobuffer4zpu</copy4xi>"
#       channel3copyimagetobufferpublish4zpu = response4copy3imagetobuffer['PUBLISHERCHANNEL']
#       channel3copyimagetobufferpublish4zpu.send(message3copyimagetobuffersend4zpu)
#       print("(Step 61B/700) OF ZPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4ZPU: ", channel3copyimagetobufferpublish4zpu, " MESSAGE3SEND4ZPU: " , message3copyimagetobuffersend4zpu)                                                                                                
#   except:
#       print('(Step 61Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGETOBUFFER4ZPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (62) LET MANOEUVRE3COPYIMAGETOBUFFER4ZPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 62A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4zpu = manoeuvre3copyimagetobuffer4zpu(queue3copyimagetobuffer4zpu, channel3copyimagetobufferconsume4zpu, channel3copyimagetobufferpublish4zpu)
#       print("(Step 62B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", manoeuvre3copyimagetobuffer4zpu)
#   except:
#       print("(Step 62Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (63) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 63A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY3IMAGETOBUFFER4ZPU ', time4xi())     
#   try:
#       channel3copyimagetobufferconsume4zpu = response4copy3imagetobuffer['CONSUMERCHANNEL']                              
#       message3copyimagereceive4zpu = channel3copyimagetobufferconsume4zpu.recv()
#       print("(Step 63B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3copyimagetobufferconsume4zpu, " CONTROL & COMMAND: ", message3copyimagereceive4zpu)                                                                                                  
#       # 
#   except:
#       print('(Step 63Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY3IMAGETOBUFFER4ZPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (64) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE3COPYIMAGETOBUFFER4ZPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 64A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGETOBUFFER4ZPU ', time4xi())     
#   try:                               
#       message3copyimagetobuffersend4zpu = "<copy4xi>copy3copyimagetobuffer4zpu</copy4xi>"
#       channel3copyimagetobufferpublish4zpu = response4copy3imagetobuffer['PUBLISHERCHANNEL']
#       channel3copyimagetobufferpublish4zpu.send(message3copyimagetobuffersend4zpu)
#       print("(Step 64B/700) OF ZPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4ZPU: ", channel3copyimagetobufferpublish4zpu, " MESSAGE3SEND4ZPU: " , message3copyimagetobuffersend4zpu)                                                                                                
#       #  
#   except:
#       print('(Step 64Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGETOBUFFER4ZPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (65) LET MANOEUVRE3COPYIMAGETOBUFFER4ZPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 65A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4zpu = manoeuvre3copyimagetobuffer4zpu(queue3copyimagetobuffer4zpu, channel3copyimagetobufferconsume4zpu, channel3copyimagetobufferpublish4zpu)
#       print("(Step 65B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", manoeuvre3copyimagetobuffer4zpu)
#   except:
#       print("(Step 65Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (66) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 66A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY3IMAGETOBUFFER4ZPU ', time4xi())     
#   try:
#       channel3copyimagetobufferconsume4zpu = response4copy3imagetobuffer['CONSUMERCHANNEL']                              
#       message3copyimagereceive4zpu = channel3copyimagetobufferconsume4zpu.recv()
#       print("(Step 66B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3copyimagetobufferconsume4zpu, " CONTROL & COMMAND: ", message3copyimagereceive4zpu)                                                                                                  
#       # 
#   except:
#       print('(Step 66Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY3IMAGETOBUFFER4ZPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (67) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 67A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue3copyimagetobuffer4zpu.put(path3zpu)             
#       print("(Step 67B/700) OF ZPU3IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path3zpu) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 67Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (68) LET MANOEUVRE3COPYIMAGETOBUFFER4ZPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 68A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4zpu = manoeuvre3copyimagetobuffer4zpu(queue3copyimagetobuffer4zpu, channel3copyimagetobufferconsume4zpu, channel3copyimagetobufferpublish4zpu)
#       print("(Step 68B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", manoeuvre3copyimagetobuffer4zpu)
#   except:
#       print("(Step 68Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (69) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 69A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message3copyimagereceive4zpu = channel3copyimagetobufferconsume4zpu.recv()
#       print("(Step 69D/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3copyimagetobufferconsume4zpu, " CONTROL & COMMAND: ", message3copyimagereceive4zpu)                                                                                                   
#   except:
#       print('(Step 69Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (70) LET MANOEUVRE3COPYIMAGETOBUFFER4ZPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 70A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4zpu = manoeuvre3copyimagetobuffer4zpu(queue3copyimagetobuffer4zpu, channel3copyimagetobufferconsume4zpu, channel3copyimagetobufferpublish4zpu)
#       print("(Step 70B/700) OF ZPU3IMAGE4XI5TEST - GOT RESPONSE3MANOEUVRE4ZPU ", response3manoeuvreimagecopytobuffer4zpu)
#   except:
#       print("(Step 70Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi())   

#">
# -------------------------------------------------------------------------------------------------   
def copy3imagetobuffer4zpu(task4copyimagetobuffer3zpu=None):

    '''
    copies 3D images to buffer for ZPU
    '''
    global zpu3image4xi                                                                                             # 3D-IMAGES FOR ZPU  
    global zpu3buffer4xi                                                                                            # BUFFER FOR ZPU
    
    global control3copyimagetobuffer4zpu                                                                            # COMMAND AND CONTROL FOR ZPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy3imagetobuffer4zpu"
    method2print = "copy3imagetobuffer4zpu"

    message2print = "{(Step 1A/33) started" + "; copy3imagetobuffer4zpu-started: " + str(copy3imagetobuffer4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                 
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4copyimagetobuffer3zpu-to-copy3imagetobuffer4zpu: " + str(task4copyimagetobuffer3zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE ZPU3IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if zpu3image4xi is None:
            zpu3image4xi = create3image4zpu()                                                                              # INITIALIZE IMAGE FOR ZPU 
            message2print = "{(Step 1C/70) created" + "; zpu3image4xi-created-by-copy3imagetobuffer4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create3image4zpu-skipped-by-copy3imagetobuffer4zpu: " + str(copy3imagetobuffer4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                          
    except:
        try:
            zpu3image4xi = create3image4zpu()
            message2print = "{(Step 1E/70) set" + "; zpu3image4xi-set-by-copy3imagetobuffer4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            zpu3image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; zpu3image4xi-yielded-by-copy3imagetobuffer4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE ZPU3BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if zpu3buffer4xi is None:
            zpu3buffer4xi = create2buffer4zpu()                                                                     # INITIALIZE BUFFER FOR ZPU 
            message2print = "{(Step 1F/70) created" + "; zpu3buffer4xi-created-by-copy3imagetobuffer4zpu to: " + str(zpu3buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4zpu-skipped-by-copy3imagetobuffer4zpu: " + str(copy3imagetobuffer4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                  
    except:
        try:
            zpu3buffer4xi = create2buffer4zpu()
            message2print = "{(Step 1H/70) set" + "; zpu3buffer4xi-set-by-copy3imagetobuffer4zpu to: " + str(zpu3buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        except:
            zpu3buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; zpu3buffer4xi-yielded-by-copy3imagetobuffer4zpu: " + str(zpu3buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASK          
    # -----------------------------------------------------------------------------------------          
              
    if task4copyimagetobuffer3zpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
            
        task4copyimagetobuffer3zpu = {}                                                                                                                                                                                       
            
        message2print = "{(Step 1A/70) initialized" + "; task4copyimagetobuffer3zpu-created-by-copy3imagetobuffer4zpu to: " + str(task4copyimagetobuffer3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
            
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function3callback4zpu = trace3image4zpu                                                                 # CALLBACK FUNCTION FOR ZPU
            
        message2print = "{(Step 1B1/70) initialized" + "; function3callback4zpu-created-by-copy3imagetobuffer4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code3ack  = code3ack + '1B1-'
           
        args3callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args3callback['CLASS']   = "copy3imagetobuffer4zpu"                                                            # MODULE OF ZPUIZED OPENCL MODULE
        args3callback['METHOD']  = "copy3imagetobuffer4zpu"                                                            # FUNCTION OF ZPUIZED OPENCL MODULE
        args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy3imagetobuffer4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args3callback-created-by-copy3imagetobuffer4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        

        code3ack  = code3ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer3zpu['DATA3CALLBACK'] = args3callback                                                    # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code3ack  = code3ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copyimagetobuffer3zpu-created-by-copy3imagetobuffer4zpu to: " + str(task4copyimagetobuffer3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform3xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-copy3imagetobuffer4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-copy3imagetobuffer4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
              
            code3ack  = code3ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-copy3imagetobuffer4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code3nack  = code3nack + '2Z-'
                 
        task4copyimagetobuffer3zpu['PLATFORM'] = platform3xi                                                           # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context3zpu = {}
        message2print = "{(Step 3A/70) trying" + "; context3zpu-trying-by-copy3imagetobuffer4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context3zpu = get2context4zpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context3zpu-got-by-copy3imagetobuffer4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
             
            code3ack  = code3ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context3zpu-failed-by-copy3imagetobuffer4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z-'
            
        task4copyimagetobuffer3zpu ['CONTEXT'] = context3zpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device3zpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device3zpu-trying-by-copy3imagetobuffer4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device3zpu = get2zpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device3zpu-got-by-copy3imagetobuffer4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device3zpu-failed-by-copy3imagetobuffer4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z-'
        
        task4copyimagetobuffer3zpu ['DEVICE'] = device3zpu
        
               
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE SOURCE OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagesourceinfo3zpu = {}
         
        message2print = "{(Step 5A/70) trying" + "; imagesourceinfo3zpu-trying-by-copy3imagetobuffer4zpu: " + str(imagesourceinfo3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagesourceinfo3zpu = get3imageinfo4zpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagesourceinfo3zpu-got-by-copy3imagetobuffer4zpu: " + str(imagesourceinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo3zpu-failed-by-copy3imagetobuffer4zpu: " + str(imagesourceinfo3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '5Z-'          
                  
        task4copyimagetobuffer3zpu ['IMAGESOURCE'] = imagesourceinfo3zpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER DESTINATION OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        bufferdestinationinfo3zpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; bufferdestinationinfo3zpu-trying-by-copy3imagetobuffer4zpu: " + str(bufferdestinationinfo3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            bufferdestinationinfo3zpu = get2bufferinfo4zpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; bufferdestinationinfo3zpu-got-by-copy3imagetobuffer4zpu: " + str(bufferdestinationinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; bufferdestinationinfo3zpu-failed-by-copy3imagetobuffer4zpu: " + str(bufferdestinationinfo3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '6Z-'          
                  
        task4copyimagetobuffer3zpu ['BUFFERDESTINATION'] = bufferdestinationinfo3zpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY ZPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue3copyimagetobuffer4zpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue3copyimagetobuffer4zpu-trying-by-copy3imagetobuffer4zpu: " + str(queue3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue3copyimagetobuffer4zpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue3copyimagetobuffer4zpu-got-by-copy3imagetobuffer4zpu: " + str(queue3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue3copyimagetobuffer4zpu-failed-by-copy3imagetobuffer4zpu: " + str(queue3copyimagetobuffer4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '7Z-'
            
        task4copyimagetobuffer3zpu ['QUEUE'] = queue3copyimagetobuffer4zpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY ZPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe3zpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe3zpu-trying-by-copy3imagetobuffer4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe3zpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe3zpu-got-by-copy3imagetobuffer4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe3zpu-failed-by-copy3imagetobuffer4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '8Z-'
            
        task4copyimagetobuffer3zpu ['PIPE'] = pipe3zpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo3zpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo3zpu-trying-by-copy3imagetobuffer4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo3zpu-got-by-copy3imagetobuffer4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo3zpu-failed-by-copy3imagetobuffer4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '9Z-'
    
        task4copyimagetobuffer3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo3zpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo3zpu-trying-by-copy3imagetobuffer4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo3zpu = get2eventinfo4zpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo3zpu-got-by-copy3imagetobuffer4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo3zpu-failed-by-copy3imagetobuffer4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '11Z-'
    
        task4copyimagetobuffer3zpu ['EVENT'] = eventinfo3zpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist3zpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist3zpu-trying-by-copy3imagetobuffer4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist3zpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist3zpu-set-by-copy3imagetobuffer4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist3zpu-failed-by-copy3imagetobuffer4zpu: " + str(eventwaitlist3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '12Z-'
    
        task4copyimagetobuffer3zpu ['EVENTWAITLIST'] = eventwaitlist3zpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity3zpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity3zpu-trying-by-copy3imagetobuffer4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys3event = eventwaitlist3zpu.keys()
            for key3event in keys3event:
                eventwaitlistquantity3zpu = eventwaitlistquantity3zpu + 1                                                                      
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity3zpu-set-by-copy3imagetobuffer4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity3zpu-failed-by-copy3imagetobuffer4zpu: " + str(eventwaitlistquantity3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '13Z-'
    
        task4copyimagetobuffer3zpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3zpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (14) ORIGIN FOR SOURCE - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        origin4source3zpu = {}
        
        message2print = "{(Step 14A/70) trying" + "; origin4source3zpu-trying-by-copy3imagetobuffer4zpu: " + str(origin4source3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4source3zpu['X'] = 0 
            origin4source3zpu['Y'] = 0  
            origin4source3zpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 14B/70) set" + "; origin4source3zpu-set-by-copy3imagetobuffer4zpu to: " + str(origin4source3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '14B-' 
        except:
            message2print = "{(Step 14Z/70) failed" + "; origin4source3zpu-failed-by-copy3imagetobuffer4zpu: " + str(origin4source3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '14Z-'
    
        task4copyimagetobuffer3zpu ['SOURCEORIGIN'] = origin4source3zpu
        
       
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy3zpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy3zpu-trying-by-copy3imagetobuffer4zpu: " + str(region4copy3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy3zpu['X'] = 10 
            region4copy3zpu['Y'] = 10  
            region4copy3zpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy3zpu-set-by-copy3imagetobuffer4zpu to: " + str(region4copy3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy3zpu-failed-by-copy3imagetobuffer4zpu: " + str(region4copy3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '16Z-'
        
        task4copyimagetobuffer3zpu ['REGION'] = region4copy3zpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER                     
        # --------------------------------------------------------------------------------------------------------             
        offset4copy3zpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy3zpu-set-by-copy3imagetobuffer4zpu to: " + str(offset4copy3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code3ack  = code3ack + '17A-'                
        task4copyimagetobuffer3zpu ['OFFSET'] = offset4copy3zpu            
                    
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
    else:
        pass
                    
    # ---------------------------------------------------------------------------------------------
    # ANALYZE TASK - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
    # ---------------------------------------------------------------------------------------------
    keys3task = task4copyimagetobuffer3zpu.keys()
    message2print = "{(Step 20/70) got" + "; keys3task-got-by-copy3imagetobuffer4zpu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)            
    # 


    if (('DATA3CALLBACK' in keys3task) and ('FUNCTION3CALLBACK' in keys3task)):
        pass
    else:                
        # ---------------------------------------------------------------------------------------------
        # (21B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function3callback4zpu = trace3image4zpu                                                             # CALLBACK FUNCTION FOR ZPU
        
        message2print = "{(Step 21B1/70) initialized" + "; function3callback4zpu-created-by-copy3imagetobuffer4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code3ack  = code3ack + '21B1-'
           
        args3callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args3callback['CLASS']   = "copy3imagetobuffer4zpu"                                                        # MODULE OF ZPUIZED OPENCL MODULE
        args3callback['METHOD']  = "copy3imagetobuffer4zpu"                                                        # FUNCTION OF ZPUIZED OPENCL MODULE
        args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy3imagetobuffer4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    

        message2print = "{(Step 21B2/70) initialized" + "; args3callback-created-by-copy3imagetobuffer4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 

        code3ack  = code3ack + '21B2-'
        
        # ---------------------------------------------------------------------------------------------
        # (21C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer3zpu['DATA3CALLBACK'] = args3callback                                                # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code3ack  = code3ack + '21C-'
        message2print = "{(Step 21C/70) initialized" + "; task4copyimagetobuffer3zpu-created-by-copy3imagetobuffer4zpu to: " + str(task4copyimagetobuffer3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
             
    if 'PLATFORM' in keys3task:
        pass
    else:  
        # ---------------------------------------------------------------------------------------------
        # (22) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform3xi = {}
        message2print = "{(Step 22A/70) initialized" + "; platform3xi-created-by-copy3imagetobuffer4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform3xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 22B/70) got" + "; platform3xi-got-by-copy3imagetobuffer4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code3ack  = code3ack + '22B-'
        except:
            message2print = "{(Step 22Z/70) failed" + "; platform3xi-failed-by-copy3imagetobuffer4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code3nack  = code3nack + '22Z-'
                 
        task4copyimagetobuffer3zpu['PLATFORM'] = platform3xi                                                       # PLATFORM OF SYSTEM 
          
    if 'CONTEXT' in keys3task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (23) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context3zpu = {}
        message2print = "{(Step 23A/70) trying" + "; context3zpu-trying-by-copy3imagetobuffer4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context3zpu = get2context4zpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
            
            message2print = "{(Step 23B/70) got" + "; context3zpu-got-by-copy3imagetobuffer4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '23B-'         
        except:
            message2print = "{(Step 23Z/70) failed" + "; context3zpu-failed-by-copy3imagetobuffer4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '23Z-'
            
        task4copyimagetobuffer3zpu ['CONTEXT'] = context3zpu           
    
    if 'DEVICE' in keys3task:
        pass
    else:                   
        # ---------------------------------------------------------------------------------------------
        # (24) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device3zpu = {}
        
        message2print = "{(Step 24A/70) trying" + "; device3zpu-trying-by-copy3imagetobuffer4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device3zpu = get2zpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 24B/70) got" + "; device3zpu-got-by-copy3imagetobuffer4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
                
            code3ack  = code3ack + '24B-'  
        except:
            message2print = "{(Step 24Z/70) failed" + "; device3zpu-failed-by-copy3imagetobuffer4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '24Z-'
        
        task4copyimagetobuffer3zpu ['DEVICE'] = device3zpu
    
    if 'IMAGESOURCE' in keys3task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (25) SOURCE IMAGE - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        sourceimageinfo3zpu = {}
         
        message2print = "{(Step 25A/70) trying" + "; sourceimageinfo3zpu-trying-by-copy3imagetobuffer4zpu: " + str(sourceimageinfo3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            sourceimageinfo3zpu = get3imageinfo4zpu()                                                                         
            
            message2print = "{(Step 25B/70) got" + "; sourceimageinfo3zpu-got-by-copy3imagetobuffer4zpu: " + str(sourceimageinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code3ack  = code3ack + '25B-'  
        except:
            message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo3zpu-failed-by-copy3imagetobuffer4zpu: " + str(sourceimageinfo3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '25Z-'          
                  
        task4copyimagetobuffer3zpu ['IMAGESOURCE'] = sourceimageinfo3zpu         
    
    if 'BUFFERDESTINATION' in keys3task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (26) DESTINATION BUFFER - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        destinationbufferinfo3zpu = {}
         
        message2print = "{(Step 26A/70) trying" + "; destinationbufferinfo3zpu-trying-by-copy3imagetobuffer4zpu: " + str(destinationbufferinfo3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            destinationbufferinfo3zpu = get2bufferinfo4zpu()                                                                         
            
            message2print = "{(Step 26B/70) got" + "; destinationbufferinfo3zpu-got-by-copy3imagetobuffer4zpu: " + str(destinationbufferinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code3ack  = code3ack + '26B-'  
        except:
            message2print = "{(Step 26Z/70) failed" + "; destinationbufferinfo3zpu-failed-by-copy3imagetobuffer4zpu: " + str(destinationbufferinfo3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '26Z-'          
                  
        task4copyimagetobuffer3zpu ['BUFFERDESTINATION'] = destinationbufferinfo3zpu    
                                
    if 'QUEUE' in keys3task:
        pass
    else:   
        # ---------------------------------------------------------------------------------------------
        # (27) QUEUE - REQUIRED BY ZPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue3copyimagetobuffer4zpu = None
        
        message2print = "{(Step 27A/70) trying" + "; queue3copyimagetobuffer4zpu-trying-by-copy3imagetobuffer4zpu: " + str(queue3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue3copyimagetobuffer4zpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 27B/70) got" + "; queue3copyimagetobuffer4zpu-got-by-copy3imagetobuffer4zpu: " + str(queue3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '27B-'  
        except:
            message2print = "{(Step 27Z/70) failed" + "; queue3copyimagetobuffer4zpu-failed-by-copy3imagetobuffer4zpu: " + str(queue3copyimagetobuffer4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '27Z-'
            
        task4copyimagetobuffer3zpu ['QUEUE'] = queue3copyimagetobuffer4zpu
        
    if 'PIPE' in keys3task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) PIPE - REQUIRED BY ZPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe3zpu = None
       
        message2print = "{(Step 28A/70) trying" + "; pipe3zpu-trying-by-copy3imagetobuffer4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe3zpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 28B/70) got" + "; pipe3zpu-got-by-copy3imagetobuffer4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '28B-'  
        except:
            message2print = "{(Step 28Z/70) failed" + "; pipe3zpu-failed-by-copy3imagetobuffer4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '28Z-'
            
        task4copyimagetobuffer3zpu ['PIPE'] = pipe3zpu 
        
    if 'COMMANDQUEUE' in keys3task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo3zpu = {}
       
        message2print = "{(Step 29A/70) trying" + "; commandqueueinfo3zpu-trying-by-copy3imagetobuffer4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 29B/70) got" + "; commandqueueinfo3zpu-got-by-copy3imagetobuffer4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '29B-'  
        except:
            message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo3zpu-failed-by-copy3imagetobuffer4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '29Z-'
    
        task4copyimagetobuffer3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
    
         
    if 'EVENT' in keys3task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (31) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo3zpu = {}
       
        message2print = "{(Step 31A/70) trying" + "; eventinfo3zpu-trying-by-copy3imagetobuffer4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo3zpu = get2eventinfo4zpu()                                                                    
            message2print = "{(Step 31B/70) got" + "; eventinfo3zpu-got-by-copy3imagetobuffer4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '31B-' 
        except:
            message2print = "{(Step 31Z/70) failed" + "; eventinfo3zpu-failed-by-copy3imagetobuffer4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '31Z-'
    
        task4copyimagetobuffer3zpu ['EVENT'] = eventinfo3zpu 
    
         
    if 'EVENTWAITLIST' in keys3task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (32) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist3zpu = {}
       
        message2print = "{(Step 32A/70) trying" + "; eventwaitlist3zpu-trying-by-copy3imagetobuffer4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist3zpu = {}                                                                   
            message2print = "{(Step 32B/70) set" + "; eventwaitlist3zpu-set-by-copy3imagetobuffer4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '32B-' 
        except:
            message2print = "{(Step 32Z/70) failed" + "; eventwaitlist3zpu-failed-by-copy3imagetobuffer4zpu: " + str(eventwaitlist3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '32Z-'
    
        task4copyimagetobuffer3zpu ['EVENTWAITLIST'] = eventwaitlist3zpu                     
          
    if 'EVENTWAITLISTQUANTITY' in keys3task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity3zpu = 0
       
        message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity3zpu-trying-by-copy3imagetobuffer4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys3event = eventwaitlist3zpu.keys()
            for key3event in keys3event:
                eventwaitlistquantity3zpu = eventwaitlistquantity3zpu + 1                                                                   
            message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity3zpu-set-by-copy3imagetobuffer4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '33B-' 
        except:
            message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity3zpu-failed-by-copy3imagetobuffer4zpu: " + str(eventwaitlistquantity3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '33Z-'
    
        task4copyimagetobuffer3zpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3zpu       
                                         
                
    if 'SOURCEORIGIN' in keys3task:
        pass
    else:                    
        # ---------------------------------------------------------------------------------------------
        # (34) SOURCEORIGIN - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        source4origin3zpu = {}
        
        message2print = "{(Step 34A/70) trying" + "; source4origin3zpu-trying-by-copy3imagetobuffer4zpu: " + str(source4origin3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            source4origin3zpu['X'] = 0 
            source4origin3zpu['Y'] = 0  
            source4origin3zpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 34B/70) set" + "; source4origin3zpu-set-by-copy3imagetobuffer4zpu to: " + str(source4origin3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code3ack  = code3ack + '34B-' 
        except:
            message2print = "{(Step 34Z/70) failed" + "; source4origin3zpu-failed-by-copy3imagetobuffer4zpu: " + str(source4origin3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '34Z-'
    
        task4copyimagetobuffer3zpu ['SOURCEORIGIN'] = source4origin3zpu
             
   
             
    if 'REGION' in keys3task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (36) REGION - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        region3zpu = {}
        
        message2print = "{(Step 36A/70) trying" + "; region3zpu-trying-by-copy3imagetobuffer4zpu: " + str(region3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            region3zpu['X'] = 330                                                                               # WIDTH  IN PIXELS TO WRITE 
            region3zpu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO WRITE
            region3zpu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
            message2print = "{(Step 36B/70) set" + "; region3zpu-set-by-copy3imagetobuffer4zpu to: " + str(region3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code3ack  = code3ack + '36B-' 
        except:
            message2print = "{(Step 36Z/70) failed" + "; region3zpu-failed-by-copy3imagetobuffer4zpu: " + str(region3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '36Z-'
    
        task4copyimagetobuffer3zpu ['REGION'] = region3zpu
    
         
    if 'OFFSET' in keys3task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (37) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        offset2zpu = 0
        message2print = "{(Step 37A/70) set" + "; offset2zpu-set-by-copy3imagetobuffer4zpu to: " + str(offset2zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        task4copyimagetobuffer3zpu ['OFFSET'] = offset2zpu        
        code3ack  = code3ack + '37A-' 
        
    # ---------------------------------------------------------------------------------------------
    # EOF TASK
    # ---------------------------------------------------------------------------------------------
    keys3task = task4copyimagetobuffer3zpu.keys()
    message2print = "{(Step 39/70) set" + "; keys3task-set-by-copy3imagetobuffer4zpu to: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    # 
               


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control3copyimagetobuffer4zpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels3zpu = False
    channels3zpu = ()
    message2print = "{(Step 40A/70) trying" + "channels3zpu-trying-by-copy3imagetobuffer4zpu: " + str(channels3zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel3copyimagetobuffer4zpu, publisherchannel3copyimagetobuffer4zpu = task4copyimagetobuffer3zpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel3copyimagetobuffer4zpu-got-by-copy3imagetobuffer4zpu: " + str(consumerchannel3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel3copyimagetobuffer4zpu-got-by-copy3imagetobuffer4zpu: " + str(publisherchannel3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                   
        ack4channels3zpu = True
        
        control3copyimagetobuffer4zpu['CONSUMERCHANNEL']  = consumerchannel3copyimagetobuffer4zpu
        
        
        control3copyimagetobuffer4zpu['PUBLISHERCHANNEL'] = publisherchannel3copyimagetobuffer4zpu
        
        code3ack  = code3ack + '40D-' 
        
    except:
        ack4channels3zpu = False
        code3nack  = code3nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue3copyimagetobuffer4zpu = False
    queue3copyimagetobuffer4zpu = None
    message2print = "{(Step 41A/70) trying" + "queue3copyimagetobuffer4zpu-trying-by-copy3imagetobuffer4zpu: " + str(queue3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue3copyimagetobuffer4zpu = task4copyimagetobuffer3zpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue3copyimagetobuffer4zpu-got-by-copy3imagetobuffer4zpu: " + str(queue3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
              
        ack4queue3copyimagetobuffer4zpu = True
        control3copyimagetobuffer4zpu['QUEUE']  = queue3copyimagetobuffer4zpu        
        code3ack  = code3ack + '41C-'
    except:
        ack4queue3copyimagetobuffer4zpu = False  
        code3nack  = code3nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy3image4zpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy3image4zpu-trying-by-copy3imagetobuffer4zpu: " + str(dummy3image4zpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy3image4zpu = DummyProcess4Xi(target=manoeuvre3copyimagetobuffer4zpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue3copyimagetobuffer4zpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel3copyimagetobuffer4zpu, publisherchannel3copyimagetobuffer4zpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy3image4zpu-created-by-copy3imagetobuffer4zpu: " + str(dummy3image4zpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code3ack  = code3ack + '69B-'
        
    # except:
        # code3nack  = code3nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code3ack-set-by-copy3imagetobuffer4zpu: " + str(code3ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    
    message2print = "{(Step 70B/70) set" + "; code3nack-set-by-copy3imagetobuffer4zpu: " + str(code3nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control3copyimagetobuffer4zpu-returning-by-copy3imagetobuffer4zpu: " + str(control3copyimagetobuffer4zpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
    
        
    return control3copyimagetobuffer4zpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy3buffertoimage4zpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (80) ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4ZPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 80A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4ZPU ', time4xi())    
#   response4copy3buffertoimage = {} 
#   task4copy3buffertoimage = None
#   try:                               
#       response4copy3buffertoimage = copy3buffertoimage4zpu(task4copy3buffertoimage)                                                                                                          
#       print('(Step 80B/700) OF ZPU3IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4ZPU : ', response4copy3buffertoimage)
#       queue3copybuffertoimage4zpu      = response4copy3buffertoimage['QUEUE']
#       channel3copybuffertoimagepublish4zpu = response4copy3buffertoimage['PUBLISHERCHANNEL']
#       channel3copybuffertoimageconsume4zpu = response4copy3buffertoimage['CONSUMERCHANNEL']
#   except:
#       print('(Step 80Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4ZPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (81) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE3COPYIMAGETOBUFFER4ZPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 81A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4ZPU ', time4xi())     
#   try:                               
#       message3copybuffertoimagesend4zpu = "<copy4xi>copy3copybuffertoimage4zpu</copy4xi>"
#       channel3copybuffertoimagepublish4zpu = response4copy3buffertoimage['PUBLISHERCHANNEL']
#       channel3copybuffertoimagepublish4zpu.send(message3copybuffertoimagesend4zpu)
#       print("(Step 81B/700) OF ZPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4ZPU: ", channel3copybuffertoimagepublish4zpu, " MESSAGE3SEND4ZPU: " , message3copybuffertoimagesend4zpu)                                                                                                
#   except:
#       print('(Step 81Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4ZPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (82) LET MANOEUVRE3COPYIMAGETOBUFFER4ZPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 82C/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4zpu = manoeuvre3copybuffertoimage4zpu(queue3copybuffertoimage4zpu, channel3copybuffertoimageconsume4zpu, channel3copybuffertoimagepublish4zpu)
#       print("(Step 83D/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", manoeuvre3copybuffertoimage4zpu)
#   except:
#       print("(Step 82Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (83) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 83A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4ZPU ', time4xi())     
#   try:
#       channel3copybuffertoimageconsume4zpu = response4copy3buffertoimage['CONSUMERCHANNEL']                              
#       message3copyimagereceive4zpu = channel3copybuffertoimageconsume4zpu.recv()
#       print("(Step 83B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3copybuffertoimageconsume4zpu, " CONTROL & COMMAND: ", message3copyimagereceive4zpu)                                                                                                  
#       # 
#   except:
#       print('(Step 83Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4ZPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (84) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE3COPYIMAGETOBUFFER4ZPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 84A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4ZPU ', time4xi())     
#   try:                               
#       message3copybuffertoimagesend4zpu = "<copy4xi>copy3copybuffertoimage4zpu</copy4xi>"
#       channel3copybuffertoimagepublish4zpu = response4copy3buffertoimage['PUBLISHERCHANNEL']
#       channel3copybuffertoimagepublish4zpu.send(message3copybuffertoimagesend4zpu)
#       print("(Step 84B/700) OF ZPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4ZPU: ", channel3copybuffertoimagepublish4zpu, " MESSAGE3SEND4ZPU: " , message3copybuffertoimagesend4zpu)                                                                                                
#       #  
#   except:
#        print('(Step 84Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4ZPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (85) LET MANOEUVRE3COPYIMAGETOBUFFER4ZPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 85A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4zpu = manoeuvre3copybuffertoimage4zpu(queue3copybuffertoimage4zpu, channel3copybuffertoimageconsume4zpu, channel3copybuffertoimagepublish4zpu)
#       print("(Step 85B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", manoeuvre3copybuffertoimage4zpu)
#   except:
#       print("(Step 85Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (86) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 86A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4ZPU ', time4xi())     
#   try:
#       channel3copybuffertoimageconsume4zpu = response4copy3buffertoimage['CONSUMERCHANNEL']                              
#       message3copyimagereceive4zpu = channel3copybuffertoimageconsume4zpu.recv()
#       print("(Step 86B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3copybuffertoimageconsume4zpu, " CONTROL & COMMAND: ", message3copyimagereceive4zpu)                                                                                                  
#       # 
#   except:
#       print('(Step 86Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4ZPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (87) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 87A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue3copybuffertoimage4zpu.put(path3zpu)             
#       print("(Step 87B/700) OF ZPU3IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path3zpu) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 87Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (88) LET MANOEUVRE3COPYIMAGETOBUFFER4ZPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 88A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4zpu = manoeuvre3copybuffertoimage4zpu(queue3copybuffertoimage4zpu, channel3copybuffertoimageconsume4zpu, channel3copybuffertoimagepublish4zpu)
#       print("(Step 88B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", manoeuvre3copybuffertoimage4zpu)
#   except:
#       print("(Step 88Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (89) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 89A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message3copyimagereceive4zpu = channel3copybuffertoimageconsume4zpu.recv()
#       print("(Step 89D/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3copybuffertoimageconsume4zpu, " CONTROL & COMMAND: ", message3copyimagereceive4zpu)                                                                                                   
#   except:
#       print('(Step 89Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (90) LET MANOEUVRE3COPYIMAGETOBUFFER4ZPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 90A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4zpu = manoeuvre3copybuffertoimage4zpu(queue3copybuffertoimage4zpu, channel3copybuffertoimageconsume4zpu, channel3copybuffertoimagepublish4zpu)
#       print("(Step 90B/700) OF ZPU3IMAGE4XI5TEST - GOT RESPONSE3MANOEUVRE4ZPU ", response3manoeuvreimagecopytobuffer4zpu)
#   except:
#       print("(Step 90Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4ZPU ", time4xi())   

#">
# -------------------------------------------------------------------------------------------------   
def copy3buffertoimage4zpu(task4copybuffertoimage3zpu=None):

    '''
    copies buffer to image for ZPU
    '''
    global zpu3image4xi                                                                                             # 3D-IMAGES FOR ZPU  
    global zpu3buffer4xi                                                                                            # BUFFER FOR ZPU
    
    global control3copybuffertoimage4zpu                                                                            # COMMAND AND CONTROL FOR ZPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy3buffertoimage4zpu"
    method2print = "copy3buffertoimage4zpu"

    message2print = "{(Step 1A/33) started" + "; copy3buffertoimage4zpu-started: " + str(copy3buffertoimage4zpu)  + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                                
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4copybuffertoimage3zpu-to-copy3buffertoimage4zpu: " + str(task4copybuffertoimage3zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE ZPU3IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if zpu3image4xi is None:
            zpu3image4xi = create3image4zpu()                                                                              # INITIALIZE IMAGE FOR ZPU 
            message2print = "{(Step 1C/70) created" + "; zpu3image4xi-created-by-copy3buffertoimage4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create3image4zpu-skipped-by-copy3buffertoimage4zpu: " + str(copy3buffertoimage4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                 
    except:
        try:
            zpu3image4xi = create3image4zpu()
            message2print = "{(Step 1E/70) set" + "; zpu3image4xi-set-by-copy3buffertoimage4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            zpu3image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; zpu3image4xi-yielded-by-copy3buffertoimage4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE ZPU3BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if zpu3buffer4xi is None:
            zpu3buffer4xi = create2buffer4zpu()                                                                     # INITIALIZE BUFFER FOR ZPU 
            message2print = "{(Step 1F/70) created" + "; zpu3buffer4xi-created-by-copy3buffertoimage4zpu to: " + str(zpu3buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4zpu-skipped-by-copy3buffertoimage4zpu: " + str(copy3buffertoimage4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            zpu3buffer4xi = create2buffer4zpu()
            message2print = "{(Step 1H/70) set" + "; zpu3buffer4xi-set-by-copy3buffertoimage4zpu to: " + str(zpu3buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            zpu3buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; zpu3buffer4xi-yielded-by-copy3buffertoimage4zpu: " + str(zpu3buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASKS           
    # -----------------------------------------------------------------------------------------          
    if task4copybuffertoimage3zpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
        
        task4copybuffertoimage3zpu = {}                                                                                                                                                                                       
        
        message2print = "{(Step 1A/70) initialized" + "; task4copybuffertoimage3zpu-created-by-copy3buffertoimage4zpu to: " + str(task4copybuffertoimage3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function3callback4zpu = trace3image4zpu                                                                 # CALLBACK FUNCTION FOR ZPU
        
        message2print = "{(Step 1B1/70) initialized" + "; function3callback4zpu-created-by-copy3buffertoimage4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
        code3ack  = code3ack + '1B1-'
       
        args3callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args3callback['CLASS']   = "copy3buffertoimage4zpu"                                                     # MODULE OF ZPUIZED OPENCL MODULE
        args3callback['METHOD']  = "copy3buffertoimage4zpu"                                                     # FUNCTION OF ZPUIZED OPENCL MODULE
        args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy3buffertoimage4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args3callback-created-by-copy3buffertoimage4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        code3ack  = code3ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copybuffertoimage3zpu['DATA3CALLBACK'] = args3callback                                             # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copybuffertoimage3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                 # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
        code3ack  = code3ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copybuffertoimage3zpu-created-by-copy3buffertoimage4zpu to: " + str(task4copybuffertoimage3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform3xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-copy3buffertoimage4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-copy3buffertoimage4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code3ack  = code3ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-copy3buffertoimage4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code3nack  = code3nack + '2Z-'
                 
        task4copybuffertoimage3zpu['PLATFORM'] = platform3xi                                                    # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context3zpu = {}
        message2print = "{(Step 3A/70) trying" + "; context3zpu-trying-by-copy3buffertoimage4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context3zpu = get2context4zpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context3zpu-got-by-copy3buffertoimage4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
             
            code3ack  = code3ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context3zpu-failed-by-copy3buffertoimage4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z-'
                
        task4copybuffertoimage3zpu ['CONTEXT'] = context3zpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device3zpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device3zpu-trying-by-copy3buffertoimage4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device3zpu = get2zpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device3zpu-got-by-copy3buffertoimage4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
             
            code3ack  = code3ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device3zpu-failed-by-copy3buffertoimage4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z-'
            
        task4copybuffertoimage3zpu ['DEVICE'] = device3zpu
            
                   
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE DESTINATION OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagedestinationinfo3zpu = {}
             
        message2print = "{(Step 5A/70) trying" + "; imagedestinationinfo3zpu-trying-by-copy3buffertoimage4zpu: " + str(imagedestinationinfo3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagedestinationinfo3zpu = get3imageinfo4zpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagedestinationinfo3zpu-got-by-copy3buffertoimage4zpu: " + str(imagedestinationinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagedestinationinfo3zpu-failed-by-copy3buffertoimage4zpu: " + str(imagedestinationinfo3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '5Z-'          
                  
        task4copybuffertoimage3zpu ['IMAGEDESTINATION'] = imagedestinationinfo3zpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER SOURCE OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        buffersourceinfo3zpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; buffersourceinfo3zpu-trying-by-copy3buffertoimage4zpu: " + str(buffersourceinfo3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            buffersourceinfo3zpu = get2bufferinfo4zpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; buffersourceinfo3zpu-got-by-copy3buffertoimage4zpu: " + str(buffersourceinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; buffersourceinfo3zpu-failed-by-copy3buffertoimage4zpu: " + str(buffersourceinfo3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '6Z-'          
                  
        task4copybuffertoimage3zpu ['BUFFERSOURCE'] = buffersourceinfo3zpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY ZPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue3copybuffertoimage4zpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue3copybuffertoimage4zpu-trying-by-copy3buffertoimage4zpu: " + str(queue3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue3copybuffertoimage4zpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue3copybuffertoimage4zpu-got-by-copy3buffertoimage4zpu: " + str(queue3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue3copybuffertoimage4zpu-failed-by-copy3buffertoimage4zpu: " + str(queue3copybuffertoimage4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '7Z-'
            
        task4copybuffertoimage3zpu ['QUEUE'] = queue3copybuffertoimage4zpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY ZPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe3zpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe3zpu-trying-by-copy3buffertoimage4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe3zpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe3zpu-got-by-copy3buffertoimage4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe3zpu-failed-by-copy3buffertoimage4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '8Z-'
            
        task4copybuffertoimage3zpu ['PIPE'] = pipe3zpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo3zpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo3zpu-trying-by-copy3buffertoimage4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo3zpu-got-by-copy3buffertoimage4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo3zpu-failed-by-copy3buffertoimage4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '9Z-'
    
        task4copybuffertoimage3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo3zpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo3zpu-trying-by-copy3buffertoimage4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo3zpu = get2eventinfo4zpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo3zpu-got-by-copy3buffertoimage4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo3zpu-failed-by-copy3buffertoimage4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '11Z-'
    
        task4copybuffertoimage3zpu ['EVENT'] = eventinfo3zpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist3zpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist3zpu-trying-by-copy3buffertoimage4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist3zpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist3zpu-set-by-copy3buffertoimage4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist3zpu-failed-by-copy3buffertoimage4zpu: " + str(eventwaitlist3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '12Z-'
    
        task4copybuffertoimage3zpu ['EVENTWAITLIST'] = eventwaitlist3zpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity3zpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity3zpu-trying-by-copy3buffertoimage4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys3event = eventwaitlist3zpu.keys()
            for key3event in keys3event:
                eventwaitlistquantity3zpu = eventwaitlistquantity3zpu + 1                                                                     
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity3zpu-set-by-copy3buffertoimage4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity3zpu-failed-by-copy3buffertoimage4zpu: " + str(eventwaitlistquantity3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '13Z-'
    
        task4copybuffertoimage3zpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3zpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (15) ORIGIN FOR DESTINATION - REQUIRED                             
        # ---------------------------------------------------------------------------------------------              
        origin4destination3zpu = {}
        
        message2print = "{(Step 15A/70) trying" + "; origin4destination3zpu-trying-by-copy3buffertoimage4zpu: " + str(origin4destination3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4destination3zpu['X'] = 100 
            origin4destination3zpu['Y'] = 100  
            origin4destination3zpu['Z'] = 100                                                                                                                          
            message2print = "{(Step 15B/70) set" + "; origin4destination3zpu-set-by-copy3buffertoimage4zpu to: " + str(origin4destination3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '15B-' 
        except:
            message2print = "{(Step 15Z/70) failed" + "; origin4destination3zpu-failed-by-copy3buffertoimage4zpu: " + str(origin4destination3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '15Z-'
    
        task4copybuffertoimage3zpu ['DESTINATIONORIGIN'] = origin4destination3zpu
                    
        
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy3zpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy3zpu-trying-by-copy3buffertoimage4zpu: " + str(region4copy3zpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy3zpu['X'] = 10 
            region4copy3zpu['Y'] = 10  
            region4copy3zpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy3zpu-set-by-copy3buffertoimage4zpu to: " + str(region4copy3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy3zpu-failed-by-copy3buffertoimage4zpu: " + str(region4copy3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '16Z-'
        
        task4copybuffertoimage3zpu ['REGION'] = region4copy3zpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER                   
        # --------------------------------------------------------------------------------------------------------             
        offset4copy3zpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy3zpu-set-by-copy3buffertoimage4zpu to: " + str(offset4copy3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
                
        task4copybuffertoimage3zpu ['OFFSET'] = offset4copy3zpu            
                                
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
        
        # ---------------------------------------------------------------------------------------------
        # ANALYZE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys3task = task4copybuffertoimage3zpu.keys()
        message2print = "{(Step 20/70) got" + "; keys3task-got-by-copy3buffertoimage4zpu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
 
        if (('DATA3CALLBACK' in keys3task) and ('FUNCTION3CALLBACK' in keys3task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4zpu = trace3image4zpu                                                             # CALLBACK FUNCTION FOR ZPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function3callback4zpu-created-by-copy3buffertoimage4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code3ack  = code3ack + '21B1-'
           
            args3callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "copy3buffertoimage4zpu"                                                        # MODULE OF ZPUIZED OPENCL MODULE
            args3callback['METHOD']  = "copy3buffertoimage4zpu"                                                        # FUNCTION OF ZPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy3buffertoimage4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args3callback-created-by-copy3buffertoimage4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code3ack  = code3ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4copybuffertoimage3zpu['DATA3CALLBACK'] = args3callback                                                # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4copybuffertoimage3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4copybuffertoimage3zpu-created-by-copy3buffertoimage4zpu to: " + str(task4copybuffertoimage3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys3task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform3xi-created-by-copy3buffertoimage4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform3xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform3xi-got-by-copy3buffertoimage4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code3ack  = code3ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform3xi-failed-by-copy3buffertoimage4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '22Z-'
                 
            task4copybuffertoimage3zpu['PLATFORM'] = platform3xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3zpu = {}
            message2print = "{(Step 23A/70) trying" + "; context3zpu-trying-by-copy3buffertoimage4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3zpu = get2context4zpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context3zpu-got-by-copy3buffertoimage4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context3zpu-failed-by-copy3buffertoimage4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '23Z-'
                
            task4copybuffertoimage3zpu ['CONTEXT'] = context3zpu           
        
        if 'DEVICE' in keys3task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3zpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device3zpu-trying-by-copy3buffertoimage4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3zpu = get2zpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device3zpu-got-by-copy3buffertoimage4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device3zpu-failed-by-copy3buffertoimage4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '24Z-'
            
            task4copybuffertoimage3zpu ['DEVICE'] = device3zpu
        
        if 'IMAGEDESTINATION' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo3zpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; destinationimageinfo3zpu-trying-by-copy3buffertoimage4zpu: " + str(destinationimageinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo3zpu = get3imageinfo4zpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; destinationimageinfo3zpu-got-by-copy3buffertoimage4zpu: " + str(destinationimageinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; destinationimageinfo3zpu-failed-by-copy3buffertoimage4zpu: " + str(destinationimageinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '25Z-'          
                      
            task4copybuffertoimage3zpu ['IMAGEDESTINATION'] = destinationimageinfo3zpu         
        
        if 'BUFFERSOURCE' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) SOURCE BUFFER - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourcebufferinfo3zpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; sourcebufferinfo3zpu-trying-by-copy3buffertoimage4zpu: " + str(sourcebufferinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourcebufferinfo3zpu = get2bufferinfo4zpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; sourcebufferinfo3zpu-got-by-copy3buffertoimage4zpu: " + str(sourcebufferinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; sourcebufferinfo3zpu-failed-by-copy3buffertoimage4zpu: " + str(sourcebufferinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '26Z-'          
                      
            task4copybuffertoimage3zpu ['BUFFERSOURCE'] = sourcebufferinfo3zpu    
                                    
        if 'QUEUE' in keys3task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY ZPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3copybuffertoimage4zpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue3copybuffertoimage4zpu-trying-by-copy3buffertoimage4zpu: " + str(queue3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3copybuffertoimage4zpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue3copybuffertoimage4zpu-got-by-copy3buffertoimage4zpu: " + str(queue3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue3copybuffertoimage4zpu-failed-by-copy3buffertoimage4zpu: " + str(queue3copybuffertoimage4zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '27Z-'
                
            task4copybuffertoimage3zpu ['QUEUE'] = queue3copybuffertoimage4zpu
            
        if 'PIPE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY ZPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3zpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe3zpu-trying-by-copy3buffertoimage4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3zpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe3zpu-got-by-copy3buffertoimage4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe3zpu-failed-by-copy3buffertoimage4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '28Z-'
                
            task4copybuffertoimage3zpu ['PIPE'] = pipe3zpu 
            
        if 'COMMANDQUEUE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3zpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo3zpu-trying-by-copy3buffertoimage4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo3zpu-got-by-copy3buffertoimage4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo3zpu-failed-by-copy3buffertoimage4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '29Z-'
        
            task4copybuffertoimage3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
        
             
        if 'EVENT' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3zpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo3zpu-trying-by-copy3buffertoimage4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3zpu = get2eventinfo4zpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo3zpu-got-by-copy3buffertoimage4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo3zpu-failed-by-copy3buffertoimage4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '31Z-'
        
            task4copybuffertoimage3zpu ['EVENT'] = eventinfo3zpu 
        
             
        if 'EVENTWAITLIST' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3zpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist3zpu-trying-by-copy3buffertoimage4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3zpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist3zpu-set-by-copy3buffertoimage4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist3zpu-failed-by-copy3buffertoimage4zpu: " + str(eventwaitlist3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '32Z-'
        
            task4copybuffertoimage3zpu ['EVENTWAITLIST'] = eventwaitlist3zpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3zpu = 0
           
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity3zpu-trying-by-copy3buffertoimage4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3zpu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3zpu = eventwaitlistquantity3zpu + 1                                                                     
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity3zpu-set-by-copy3buffertoimage4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code3ack  = code3ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity3zpu-failed-by-copy3buffertoimage4zpu: " + str(eventwaitlistquantity3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '33Z-'
        
            task4copybuffertoimage3zpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3zpu       
                                             
  
             
        if 'DESTINATIONORIGIN' in keys3task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin3zpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; destination4origin3zpu-trying-by-copy3buffertoimage4zpu: " + str(destination4origin3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin3zpu['X'] = 0 
                destination4origin3zpu['Y'] = 0  
                destination4origin3zpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; destination4origin3zpu-set-by-copy3buffertoimage4zpu to: " + str(destination4origin3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; destination4origin3zpu-failed-by-copy3buffertoimage4zpu: " + str(destination4origin3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '34Z-'
        
            task4copybuffertoimage3zpu ['DESTINATIONORIGIN'] = destination4origin3zpu
             
     
             
        if 'REGION' in keys3task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3zpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region3zpu-trying-by-copy3buffertoimage4zpu: " + str(region3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region3zpu['X'] = 330                                                                               # WIDTH  IN PIXELS TO WRITE 
                region3zpu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO WRITE
                region3zpu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region3zpu-set-by-copy3buffertoimage4zpu to: " + str(region3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region3zpu-failed-by-copy3buffertoimage4zpu: " + str(region3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '36Z-'
        
            task4copybuffertoimage3zpu ['REGION'] = region3zpu
        
             
        if 'OFFSET' in keys3task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (37) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            offset2zpu = 0
            message2print = "{(Step 37A/70) set" + "; offset2zpu-set-by-copy3buffertoimage4zpu to: " + str(offset2zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            task4copybuffertoimage3zpu ['OFFSET'] = offset2zpu        
            code3ack  = code3ack + '37A-' 
                    
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys3task = task4copybuffertoimage3zpu.keys()
        message2print = "{(Step 39/70) set" + "; keys3task-set-by-copy3buffertoimage4zpu to: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control3copybuffertoimage4zpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels3zpu = False
    channels3zpu = ()
    message2print = "{(Step 40A/70) trying" + "channels3zpu-trying-by-copy3buffertoimage4zpu: " + str(channels3zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel3copybuffertoimage4zpu, publisherchannel3copybuffertoimage4zpu = task4copybuffertoimage3zpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel3copybuffertoimage4zpu-got-by-copy3buffertoimage4zpu: " + str(consumerchannel3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel3copybuffertoimage4zpu-got-by-copy3buffertoimage4zpu: " + str(publisherchannel3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                  
        ack4channels3zpu = True
        
        control3copybuffertoimage4zpu['CONSUMERCHANNEL']  = consumerchannel3copybuffertoimage4zpu
        
        
        control3copybuffertoimage4zpu['PUBLISHERCHANNEL'] = publisherchannel3copybuffertoimage4zpu
        
        code3ack  = code3ack + '40D-' 
        
    except:
        ack4channels3zpu = False
        code3nack  = code3nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue3copybuffertoimage4zpu = False
    queue3copybuffertoimage4zpu = None
    message2print = "{(Step 41A/70) trying" + "queue3copybuffertoimage4zpu-trying-by-copy3buffertoimage4zpu: " + str(queue3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue3copybuffertoimage4zpu = task4copybuffertoimage3zpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue3copybuffertoimage4zpu-got-by-copy3buffertoimage4zpu: " + str(queue3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue3copybuffertoimage4zpu = True
        control3copybuffertoimage4zpu['QUEUE']  = queue3copybuffertoimage4zpu        
        code3ack  = code3ack + '41C-'
    except:
        ack4queue3copybuffertoimage4zpu = False  
        code3nack  = code3nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy3image4zpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy3image4zpu-trying-by-copy3buffertoimage4zpu: " + str(dummy3image4zpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy3image4zpu = DummyProcess4Xi(target=manoeuvre3copybuffertoimage4zpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue3copybuffertoimage4zpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel3copybuffertoimage4zpu, publisherchannel3copybuffertoimage4zpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy3image4zpu-created-by-copy3buffertoimage4zpu: " + str(dummy3image4zpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code3ack  = code3ack + '69B-'
        
    # except:
        # code3nack  = code3nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code3ack-set-by-copy3buffertoimage4zpu: " + str(code3ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    
    message2print = "{(Step 70B/70) set" + "; code3nack-set-by-copy3buffertoimage4zpu: " + str(code3nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control3copybuffertoimage4zpu-returning-by-copy3buffertoimage4zpu: " + str(control3copybuffertoimage4zpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
    

        
    return control3copybuffertoimage4zpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "read3image4zpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ3IMAGE4ZPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ3IMAGE4ZPU ', time4xi())    
#   respon4read3image = {} 
#   task4read3image = None
#   try:                               
#       response4read3image = read3image4zpu(task4read3image)                                                                                                          
#       print('(Step 16B/700) OF ZPU3IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ3IMAGE4ZPU : ', response4read3image)
#       queue3read4zpu      = response4read3image['QUEUE']
#       channel3readpublish4zpu = response4read3image['PUBLISHERCHANNEL']
#       channel3readconsume4zpu = response4read3image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ3IMAGE4ZPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE3IMAGE4ZPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4ZPU ', time4xi())     
#   try:                               
#       message3readsend4zpu = "<copy4xi>copy3read4zpu</copy4xi>"
#       channel3readpublish4zpu = response4read3image['PUBLISHERCHANNEL']
#       channel3readpublish4zpu.send(message3readsend4zpu)
#       print("(Step 17B/700) OF ZPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4ZPU: ", channel3readpublish4zpu, " MESSAGE3SEND4ZPU: " , message3readsend4zpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4ZPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE3IMAGE4ZPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvre4zpu = manoeuvre3readimage4zpu(queue3read4zpu, channel3readconsume4zpu, channel3readpublish4zpu)
#       print("(Step 18D/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", manoeuvre3readimage4zpu)
#   except:
#       print("(Step 18Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4ZPU ', time4xi())     
#   try:
#       channel3readconsume4zpu = response4read3image['CONSUMERCHANNEL']                              
#       message3readreceive4zpu = channel3readconsume4zpu.recv()
#       print("(Step 19B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3readconsume4zpu, " CONTROL & COMMAND: ", message3readreceive4zpu)                                                                                                  
#       # <roger4xi>roger3read4zpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ3IMAGE4ZPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE3IMAGE4ZPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4ZPU ', time4xi())     
#   try:                               
#       message3readsend4zpu = "<ready4xi>ready3read4zpu</ready4xi>"
#       channel3readpublish4zpu = response4read3image['PUBLISHERCHANNEL']
#       channel3readpublish4zpu.send(message3readsend4zpu)
#       print("(Step 20B/700) OF ZPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4ZPU: ", channel3readpublish4zpu, " MESSAGE3SEND4ZPU: " , message3readsend4zpu)                                                                                                
#       # <ready4xi>ready3read4zpu</ready4xi> 
#   except:
#        print('(Step 20Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4ZPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE3IMAGE4ZPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvre4zpu = manoeuvre3readimage4zpu(queue3read4zpu, channel3readconsume4zpu, channel3readpublish4zpu)
#       print("(Step 21B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", manoeuvre3readimage4zpu)
#   except:
#       print("(Step 21Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4ZPU ', time4xi())     
#   try:
#       channel3readconsume4zpu = response4read3image['CONSUMERCHANNEL']                              
#       message3readreceive4zpu = channel3readconsume4zpu.recv()
#       print("(Step 22B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3readconsume4zpu, " CONTROL & COMMAND: ", message3readreceive4zpu)                                                                                                  
#       # <ready4xi>ready3read4zpu</ready4xi> 
#   except:
#       print('(Step 22Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ3IMAGE4ZPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue3read4zpu.put(path3zpu)             
#       print("(Step 24B/700) OF ZPU3IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path3zpu) 
#       #  
#   except:
#       print("(Step 24Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE3IMAGE4ZPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvre4zpu = manoeuvre3readimage4zpu(queue3read4zpu, channel3readconsume4zpu, channel3readpublish4zpu)
#       print("(Step 25B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", manoeuvre3readimage4zpu)
#   except:
#       print("(Step 25Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message3readreceive4zpu = channel3readconsume4zpu.recv()
#       print("(Step 26D/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3readconsume4zpu, " CONTROL & COMMAND: ", message3readreceive4zpu)                                                                                                   
#       # <ack4xi>ack3imageinfo4zpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE3IMAGE4ZPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvre4zpu = manoeuvre3readimage4zpu(queue3read4zpu, channel3readconsume4zpu, channel3readpublish4zpu)
#       print("(Step 27B/700) OF ZPU3IMAGE4XI5TEST - GOT RESPONSE3MANOEUVRE4ZPU ", response3manoeuvre4zpu)
#   except:
#       print("(Step 27Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi())   
#">
# -------------------------------------------------------------------------------------------------   
def read3image4zpu(task4imageread3zpu=None):

    '''
    reads 3D images for ZPU
    '''
    global zpu3image4xi                                                     # 3D-IMAGES FOR ZPU  
    global control3readimage4zpu                                              # COMMAND AND CONTROL FOR ZPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "read3image4zpu"
    method2print = "read3image4zpu"

    message2print = "{(Step 1A/33) started" + "; read3image4zpu-started: " + str(read3image4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function read3image4zpu at 0x00000000031B78C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageread3zpu-to-read3image4zpu: " + str(task4imageread3zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 
            
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE ZPU3IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if zpu3image4xi is None:
            zpu3image4xi = create3image4zpu()                                                                              # INITIALIZE IMAGE FOR ZPU 
            message2print = "{(Step 1C/70) created" + "; zpu3image4xi-created-by-read3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create3image4zpu-skipped-by-read3image4zpu: " + str(read3image4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function read3image4zpu at 0x00000000031B78C8>                               
    except:
        try:
            zpu3image4xi = create3image4zpu()
            message2print = "{(Step 1E/70) set" + "; zpu3image4xi-set-by-read3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            zpu3image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; zpu3image4xi-yielded-by-read3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imageread3zpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imageread3zpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imageread3zpu-created-by-read3image4zpu to: " + str(task4imageread3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4zpu = trace3image4zpu                                                              # CALLBACK FUNCTION FOR ZPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function3callback4zpu-created-by-read3image4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace3image4zpu at 0x00000000031B1848> 
            
            code3ack  = code3ack + '1B1-'
           
            args3callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "read3image4zpu"                                                          # MODULE OF ZPUIZED OPENCL MODULE
            args3callback['METHOD']  = "read3image4zpu"                                                          # FUNCTION OF ZPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read3image4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args3callback-created-by-read3image4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; read3image4zpu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 
            #   'CLASS': 'read3image4zpu', 
            #   'METHOD': 'read3image4zpu'}; time: 06:24:23 10/10/11 Pacific Daylight Time}

            
            code3ack  = code3ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread3zpu['DATA3CALLBACK'] = args3callback                                                   # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imageread3zpu-created-by-read3image4zpu to: " + str(task4imageread3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'DATA3CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; read3image4zpu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 
            #   'CLASS': 'read3image4zpu', 
            #   'METHOD': 'read3image4zpu'}, 
            #   'FUNCTION3CALLBACK': <function trace3image4zpu at 0x00000000031B1848>}; time: 06:24:23 10/10/11 Pacific Daylight Time}
            
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-read3image4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-read3image4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 06:24:23 10/10/11 Pacific Daylight Time}
                 
                code3ack  = code3ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-read3image4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '2Z-'
                 
            task4imageread3zpu['PLATFORM'] = platform3xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3zpu = {}
            message2print = "{(Step 3A/70) trying" + "; context3zpu-trying-by-read3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3zpu = get2context4zpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context3zpu-got-by-read3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 
                #   'CLASS': 'create2context4zpu', 
                #   'METHOD': 'create2context4zpu'}, 
                #   'FUNCTION2CALLBACK': <function trace2context4zpu at 0x000000000313E0C8>, 
                #   'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #   'INFO2PRIVATE': None, 
                #   'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}; time: 06:24:23 10/10/11 Pacific Daylight Time}

                code3ack  = code3ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context3zpu-failed-by-read3image4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '3Z-'
                
            task4imageread3zpu ['CONTEXT'] = context3zpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3zpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device3zpu-trying-by-read3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3zpu = get2zpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device3zpu-got-by-read3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR' ...
                
                code3ack  = code3ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device3zpu-failed-by-read3image4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '4Z-'
            
            task4imageread3zpu ['DEVICE'] = device3zpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo3zpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo3zpu-trying-by-read3image4zpu: " + str(imageinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo3zpu = get3imageinfo4zpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo3zpu-got-by-read3image4zpu: " + str(imageinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #   'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #   'ORDER': 'CL_RGBA'}, 
                #   'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041E5EB8>, 
                #   'WIDTH': 1100, 
                #   'DEPTH': 900, 
                #   'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 06:24:23 10/10/11 Pacific Day ...
                
                code3ack  = code3ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo3zpu-failed-by-read3image4zpu: " + str(imageinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '5Z-'          
                      
            task4imageread3zpu ['IMAGE'] = imageinfo3zpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY ZPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3zpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue3zpu-trying-by-read3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3zpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue3zpu-got-by-read3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004422FD0> 
                
                code3ack  = code3ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue3zpu-failed-by-read3image4zpu: " + str(queue3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '6Z-'
                
            task4imageread3zpu ['QUEUE'] = queue3zpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY ZPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3zpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe3zpu-trying-by-read3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3zpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe3zpu-got-by-read3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 588>, <read-write PipeConnection, handle 592>) 
                
                code3ack  = code3ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe3zpu-failed-by-read3image4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '7Z-'
                
            task4imageread3zpu ['PIPE'] = pipe3zpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3zpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo3zpu-trying-by-read3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo3zpu-got-by-read3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4zpu-failed ; time: 06:24:23 10/10/11 Pa ...
                
                code3ack  = code3ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo3zpu-failed-by-read3image4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '8Z-'
        
            task4imageread3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY ZPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3zpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo3zpu-trying-by-read3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3zpu = get2bufferinfo4zpu()                                                                   # SYSTEM CREATES BUFFER FOR ZPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo3zpu-got-by-read3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4zpu-failed ; time: 06:24:23 10/10/11 Pacific  ...
                
                code3ack  = code3ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo3zpu-failed-by-read3image4zpu: " + str(bufferinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '9Z-'
        
            task4imageread3zpu ['BUFFER'] = bufferinfo3zpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3zpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo3zpu-trying-by-read3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3zpu = get2eventinfo4zpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo3zpu-got-by-read3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4zpu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS ...
                
                code3ack  = code3ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo3zpu-failed-by-read3image4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '10Z-'
        
            task4imageread3zpu ['EVENT'] = eventinfo3zpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3zpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist3zpu-trying-by-read3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3zpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist3zpu-set-by-read3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                
                code3ack  = code3ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist3zpu-failed-by-read3image4zpu: " + str(eventwaitlist3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '11Z-'
        
            task4imageread3zpu ['EVENTWAITLIST'] = eventwaitlist3zpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3zpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity3zpu-trying-by-read3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3zpu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3zpu = eventwaitlistquantity3zpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity3zpu-set-by-read3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                
                code3ack  = code3ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity3zpu-failed-by-read3image4zpu: " + str(eventwaitlistquantity3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '12Z-'
        
            task4imageread3zpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3zpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking3zpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking3zpu-trying-by-read3image4zpu: " + str(blocking3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking3zpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking3zpu-set-by-read3image4zpu to: " + str(blocking3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE
                
                code3ack  = code3ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking3zpu-failed-by-read3image4zpu: " + str(blocking3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '13Z-'
        
            task4imageread3zpu ['BLOCKING'] = blocking3zpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin3zpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin3zpu-trying-by-read3image4zpu: " + str(origin3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin3zpu['X'] = 0 
                origin3zpu['Y'] = 0  
                origin3zpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin3zpu-set-by-read3image4zpu to: " + str(origin3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                
                code3ack  = code3ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin3zpu-failed-by-read3image4zpu: " + str(origin3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '14Z-'
        
            task4imageread3zpu ['ORIGIN'] = origin3zpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3zpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region3zpu-trying-by-read3image4zpu: " + str(region3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region3zpu['X'] = 330                                                                               # WIDTH  IN PIXELS TO READ 
                region3zpu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO READ 
                region3zpu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region3zpu-set-by-read3image4zpu to: " + str(region3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 330, 'X': 330, 'Z': 330} 
                
                code3ack  = code3ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region3zpu-failed-by-read3image4zpu: " + str(region3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '15Z-'
        
            task4imageread3zpu ['REGION'] = region3zpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch3zpu = 0
            size3element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 8                                                                                  # ONE DIRECTIONS IN 3D
            width3zpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch3zpu-trying-by-read3image4zpu: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3zpu = int(width3zpu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch3zpu-set-by-read3image4zpu to: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 8000                                                                                        
                 
                code3ack  = code3ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch3zpu-failed-by-read3image4zpu: " + str(rowpitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '16Z-'
        
            task4imageread3zpu ['ROWPITCH'] = rowpitch3zpu     
            
            # JATKA --------------------------------------------------------------------------------------------------------
            # (17) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 3D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch3zpu = 0                                                                                   # SLICEPITCH IS ZERO FOR 3D IMAGES                           
            height3zpu = 800
            message2print = "{(Step 17A/70) trying" + "; slicepitch3zpu-trying-by-read3image4zpu: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch3zpu = int(rowpitch3zpu * height3zpu)                                                           
                message2print = "{(Step 17B/70) set" + "; slicepitch3zpu-set-by-read3image4zpu to: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 6400000

                code3ack  = code3ack + '17B-'                
                 
            except:
                message2print = "{(Step 17Z/70) failed" + "; slicepitch3zpu-failed-by-read3image4zpu: " + str(slicepitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '17Z-'
            
            task4imageread3zpu ['SLICEPITCH'] = slicepitch3zpu           
                        
            # ----------------------------------------------------------------------------------------------------
            # (18) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer3zpu = None
            height3zpu = 800
            size3zpu = int(width3zpu * height3zpu)                                                                  # CALCULATED MEMORY
            if size3zpu == 0:
                size3zpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 18A/70) trying" + "; pointer3zpu-trying-by-read3image4zpu: " + str(pointer3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena3zpu = Arena4Xi(size3zpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 18B/70) got" + "; arena3zpu-got-by-read3image4zpu: " + str(arena3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000048859B0>
                
                size3arena = arena3zpu.size                                                                         # 
                              
                message2print = "{(Step 18C/70) got" + "; size3arena-got-by-read2buffer4zpu: " + str(size3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer3arena = arena3zpu.buffer
                
                message2print = "{(Step 18D/70) got" + "; buffer3arena-got-by-read2buffer4zpu: " + str(buffer3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x000000000435B4A8>
                
                code3ack  = code3ack + '18D-' 
                
                if size3arena < size3zpu:
                    size3zpu = size3arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 18E/70) set" + "; size3zpu-set-by-read2buffer4zpu to: " + str(size3zpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code3ack  = code3ack + '18E-'
                else:
                    pass
                    
                pointer3zpu = buffer3arena
                message2print = "{(Step 18F/70) set" + "; pointer3zpu-set-by-read2buffer4zpu to: " + str(pointer3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x000000000435B4A8>
                
                code3ack  = code3ack + '18F-'       
            except:
                message2print = "{(Step 18Z/70) failed" + "; pointer3zpu-failed-by-read3image4zpu: " + str(pointer3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '18Z-'
        
            task4imageread3zpu ['POINTER'] = pointer3zpu                            
            task4imageread3zpu ['SIZE']    = size3zpu                                                               # AUTOMATIC BY ZPUXI
            
       
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imageread3zpu.keys()
        message2print = "{(Step 20/70) got" + "; keys3task-got-by-read3image4zpu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA3CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION3CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 07:04:35 10/10/11 Pacific Daylight Time}


        if (('DATA3CALLBACK' in keys3task) and ('FUNCTION3CALLBACK' in keys3task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4zpu = trace3image4zpu                                                              # CALLBACK FUNCTION FOR ZPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function3callback4zpu-created-by-read3image4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code3ack  = code3ack + '21B1-'
           
            args3callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "read3image4zpu"                                                          # MODULE OF ZPUIZED OPENCL MODULE
            args3callback['METHOD']  = "read3image4zpu"                                                          # FUNCTION OF ZPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read3image4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args3callback-created-by-read3image4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code3ack  = code3ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread3zpu['DATA3CALLBACK'] = args3callback                                                   # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imageread3zpu-created-by-read3image4zpu to: " + str(task4imageread3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys3task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform3xi-created-by-read3image4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform3xi-got-by-read3image4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code3ack  = code3ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform3xi-failed-by-read3image4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '22Z-'
                 
            task4imageread3zpu['PLATFORM'] = platform3xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3zpu = {}
            message2print = "{(Step 23A/70) trying" + "; context3zpu-trying-by-read3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3zpu = get2context4zpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context3zpu-got-by-read3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context3zpu-failed-by-read3image4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '23Z-'
                
            task4imageread3zpu ['CONTEXT'] = context3zpu           
        
        if 'DEVICE' in keys3task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3zpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device3zpu-trying-by-read3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3zpu = get2zpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device3zpu-got-by-read3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device3zpu-failed-by-read3image4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '24Z-'
            
            task4imageread3zpu ['DEVICE'] = device3zpu
        
        if 'IMAGE' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo3zpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo3zpu-trying-by-read3image4zpu: " + str(imageinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo3zpu = get3imageinfo4zpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo3zpu-got-by-read3image4zpu: " + str(imageinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo3zpu-failed-by-read3image4zpu: " + str(imageinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '25Z-'          
                      
            task4imageread3zpu ['IMAGE'] = imageinfo3zpu         
                    
        if 'QUEUE' in keys3task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY ZPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3zpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue3zpu-trying-by-read3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3zpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue3zpu-got-by-read3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue3zpu-failed-by-read3image4zpu: " + str(queue3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '26Z-'
                
            task4imageread3zpu ['QUEUE'] = queue3zpu
        
        if 'PIPE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY ZPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3zpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe3zpu-trying-by-read3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3zpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe3zpu-got-by-read3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe3zpu-failed-by-read3image4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '27Z-'
                
            task4imageread3zpu ['PIPE'] = pipe3zpu 
            
        if 'COMMANDQUEUE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3zpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo3zpu-trying-by-read3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo3zpu-got-by-read3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo3zpu-failed-by-read3image4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '28Z-'
        
            task4imageread3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
        
        
        if 'BUFFER' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY ZPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3zpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo3zpu-trying-by-read3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3zpu = get2bufferinfo4zpu()                                                                   # SYSTEM CREATES BUFFER FOR ZPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo3zpu-got-by-read3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo3zpu-failed-by-read3image4zpu: " + str(bufferinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '29Z-'
        
            task4imageread3zpu ['BUFFER'] = bufferinfo3zpu
             
        if 'EVENT' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3zpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo3zpu-trying-by-read3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3zpu = get2eventinfo4zpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo3zpu-got-by-read3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo3zpu-failed-by-read3image4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '30Z-'
        
            task4imageread3zpu ['EVENT'] = eventinfo3zpu 
        
             
        if 'EVENTWAITLIST' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3zpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist3zpu-trying-by-read3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3zpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist3zpu-set-by-read3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist3zpu-failed-by-read3image4zpu: " + str(eventwaitlist3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '31Z-'
        
            task4imageread3zpu ['EVENTWAITLIST'] = eventwaitlist3zpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3zpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity3zpu-trying-by-read3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3zpu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3zpu = eventwaitlistquantity3zpu + 1                                                                      
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity3zpu-set-by-read3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                code3ack  = code3ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity3zpu-failed-by-read3image4zpu: " + str(eventwaitlistquantity3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '32Z-'
        
            task4imageread3zpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3zpu       
                                                 
                 
        if 'BLOCKING' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking3zpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking3zpu-trying-by-read3image4zpu: " + str(blocking3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking3zpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking3zpu-set-by-read3image4zpu to: " + str(blocking3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking3zpu-failed-by-read3image4zpu: " + str(blocking3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '33Z-'
        
            task4imageread3zpu ['BLOCKING'] = blocking3zpu             
             
        if 'ORIGIN' in keys3task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin3zpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin3zpu-trying-by-read3image4zpu: " + str(origin3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin3zpu['X'] = 0 
                origin3zpu['Y'] = 0  
                origin3zpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin3zpu-set-by-read3image4zpu to: " + str(origin3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin3zpu-failed-by-read3image4zpu: " + str(origin3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '34Z-'
        
            task4imageread3zpu ['ORIGIN'] = origin3zpu
            
             
        if 'REGION' in keys3task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3zpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region3zpu-trying-by-read3image4zpu: " + str(region3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region3zpu['X'] = 330                                                                               # WIDTH  IN PIXELS TO READ 
                region3zpu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO READ 
                region3zpu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region3zpu-set-by-read3image4zpu to: " + str(region3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region3zpu-failed-by-read3image4zpu: " + str(region3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '35Z-'
        
            task4imageread3zpu ['REGION'] = region3zpu
        
            
             
        if 'ROWPITCH' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch3zpu = 0
            size3element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 1                                                                                  # ONE DIRECTIONS IN 3D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch3zpu-trying-by-read3image4zpu: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3zpu = int(width3zpu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch3zpu-set-by-read3image4zpu to: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code3ack  = code3ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch3zpu-failed-by-read3image4zpu: " + str(rowpitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '36Z-'
        
            task4imageread3zpu ['ROWPITCH'] = rowpitch3zpu     
             
        if 'POINTER' in keys3task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer3zpu = None
            size3zpu = int(width3zpu * height3zpu)                                                                  # CALCULATED MEMORY
            if size3zpu == 0:
                size3zpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer3zpu-trying-by-read3image4zpu: " + str(pointer3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena3zpu = Arena4Xi(size3zpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena3zpu-got-by-read3image4zpu: " + str(arena3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size3arena = arena3zpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size3arena-got-by-read2buffer4zpu: " + str(size3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer3arena = arena3zpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer3arena-got-by-read2buffer4zpu: " + str(buffer3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code3ack  = code3ack + '37D-' 
                
                if size3arena < size3zpu:
                    size3zpu = size3arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size3zpu-set-by-read2buffer4zpu to: " + str(size3zpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code3ack  = code3ack + '37E-'
                else:
                    pass
                    
                pointer3zpu = buffer3arena
                message2print = "{(Step 37F/70) set" + "; pointer3zpu-set-by-read2buffer4zpu to: " + str(pointer3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code3ack  = code3ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer3zpu-failed-by-read3image4zpu: " + str(pointer3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '37Z-'
        
            task4imageread3zpu ['POINTER'] = pointer3zpu                            
            task4imageread3zpu ['SIZE']    = size3zpu                                                               # AUTOMATIC BY ZPUXI
        
             
        if 'SLICEPITCH' in keys3task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 3D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch3zpu = 0                                                                                   # SLICEPITCH IS ZERO FOR 3D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch3zpu-trying-by-read3image4zpu: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch3zpu = int(rowpitch3zpu * height3zpu)                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch3zpu-set-by-read3image4zpu to: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch3zpu-failed-by-read3image4zpu: " + str(slicepitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '38Z-'
            
            task4imageread3zpu ['SLICEPITCH'] = slicepitch3zpu
                              
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imageread3zpu.keys()
        message2print = "{(Step 39/70) set" + "; keys3task-set-by-read3image4zpu to: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #[ 'BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA3CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION3CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 07:04:35 10/10/11 Pacific Daylight Time}

                    
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imageread3zpu-yielded-by-read3image4zpu: " + str(task4imageread3zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control3readimage4zpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels3zpu = False
    channels3zpu = ()
    message2print = "{(Step 40A/70) trying" + "channels3zpu-trying-by-read3image4zpu: " + str(channels3zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        consumerchannel3zpu, publisherchannel3zpu = task4imageread3zpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel3zpu-got-by-read3image4zpu: " + str(consumerchannel3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 584>

        message2print = "{(Step 40C/70) got" + "; publisherchannel3zpu-got-by-read3image4zpu: " + str(publisherchannel3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 588> 
                             
        ack4channels3zpu = True
        
        control3readimage4zpu['CONSUMERCHANNEL']  = consumerchannel3zpu
        control3readimage4zpu['PUBLISHERCHANNEL'] = publisherchannel3zpu
        
        code3ack  = code3ack + '40D-' 
        
    except:
        ack4channels3zpu = False
        code3nack  = code3nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue3zpu = False
    queue3zpu = None
    message2print = "{(Step 41A/70) trying" + "; queue3zpu-trying-by-read3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue3zpu = task4imageread3zpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue3zpu-got-by-read3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x000000000459DFD0>
              
        ack4queue3zpu = True
        control3readimage4zpu['QUEUE']  = queue3zpu        
        code3ack  = code3ack + '41C-'
    except:
        ack4queue3zpu = False  
        code3nack  = code3nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy3image4zpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy3image4zpu-trying-by-read3image4zpu: " + str(dummy3image4zpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy3image4zpu = DummyProcess4Xi(target=manoeuvre3readimage4zpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue3zpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel3zpu, publisherchannel3zpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy3image4zpu-created-by-read3image4zpu: " + str(dummy3image4zpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code3ack  = code3ack + '69B-'
        
        # { DummyProcess4Xi : __init__ ; (Step 1/7a) started; init-of-DummyProcess4Xi-started: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7b) passed; group-passed-to-init-of-DummyProcess4Xi: None; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7c) passed; target-passed-to-init-of-DummyProcess4Xi: <function manoeuvre3readimage4zpu at 0x0000000003F01C48>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7d) passed; name-passed-to-init-of-DummyProcess4Xi: None; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7e) passed; args-passed-to-init-of-DummyProcess4Xi: (<queue4xi.Queue4Xi object at 0x0000000003FB3208>, <read-write PipeConnection, handle 412>, <read-write PipeConnection, handle 416>); time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7f) passed; kwargs-passed-to-init-of-DummyProcess4Xi: {}; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7g) borning; dummyprocess2xi-of-DummyProcess4Xi-borning: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7h) inheriting; threading-Thread-inheriting-by-dummyprocess2xi-of-DummyProcess4Xi: <class 'threading.Thread'>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7i) sharing; is-alive-sharing-by-dummyprocess2xi-of-DummyProcess4Xi: <function is_alive at 0x0000000002891AC8>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7j) sharing; get-name-sharing-by-dummyprocess2xi-of-DummyProcess4Xi: <function getName at 0x0000000002891D48>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7k) sharing; set-name-sharing-by-dummyprocess2xi-of-DummyProcess4Xi: <function setName at 0x0000000002891DC8>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7l) sharing; is-daemon-sharing-by-dummyprocess2xi-of-DummyProcess4Xi: <function isDaemon at 0x0000000002891C48>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7m) sharing; set_daemon-sharing-by-dummyprocess2xi-of-DummyProcess4Xi: <function setDaemon at 0x0000000002891CC8>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 2/7a) calling; Thread-init-calling-by-init-of-DummyProcess4Xi: <function __init__ at 0x0000000002891348>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 2/7b) performed; Thread-init-performed-by-init-of-DummyProcess4Xi: <function __init__ at 0x0000000002891348>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; set; ack2thread-set-by-init-of-DummyProcess4Xi to: True; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 3/7) set; dummyprocess2xi-pid-set-by-init-of-DummyProcess4Xi to: None; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 4/7) set; dummyprocess2xi-children-set-by-init-of-DummyProcess4Xi to: <WeakKeyDictionary at 69165912>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 5/7) set; dummyprocess2xi-start-called-set-by-init-of-DummyProcess4Xi to: False; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 6/7a) calling; current_dummy_process: <function current_thread at 0x0000000002894248>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 6/7b) set; dummyprocess2xi-parent-set-by-init-of-DummyProcess4Xi to: <_MainThread(MainThread, started 2424)>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7a) inherited; threading-Thread-inherited-by-dummyprocess2xi-of-DummyProcess4Xi: <class 'threading.Thread'>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7b) activated; dummyprocess2xi-of-DummyProcess4Xi-activated: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7c) shared; is-alive-shared-by-dummyprocess2xi-of-DummyProcess4Xi: <function is_alive at 0x0000000002891AC8>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7d) shared; get-name-shared-by-dummyprocess2xi-of-DummyProcess4Xi: <function getName at 0x0000000002891D48>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7e) shared; set-name-shared-by-dummyprocess2xi-of-DummyProcess4Xi: <function setName at 0x0000000002891DC8>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7f) shared; is-daemon-shared-by-dummyprocess2xi-of-DummyProcess4Xi: <function isDaemon at 0x0000000002891C48>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7g) shared; set-daemon-shared-by-dummyprocess2xi-of-DummyProcess4Xi: <function setDaemon at 0x0000000002891CC8>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7h) returning; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { read3image4zpu : read3image4zpu ; (Step 69B/70) createddummy3image4zpu-created-by-read3image4zpu: <DummyProcess4Xi(Thread-1, initial)>; time: 04:49:42 09/28/11 Pacific Daylight Time}
    # except:
        # code3nack  = code3nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code3ack-set-by-read3image4zpu: " + str(code3ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18D-18F-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code3nack-set-by-read3image4zpu: " + str(code3nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # ;  
        
    message2print = "{(Step 70C/70) returning" + "; control3readimage4zpu-returning-by-read3image4zpu: " + str(control3readimage4zpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000459DFD0>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 588>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 584>}; time: 07:04:35 10/10/11 Pacific Daylight Time}

        
    return control3readimage4zpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "map3image4zpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   #                     
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 92A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH MAP3IMAGE4ZPU ', time4xi())           
#                    
#   response4map3image = {} 
#   task4map3image = None                                                                                  # WE ONLY WANT TO ESTABLISH CONTROL AND CONTROL FIRST
#
#   try:                               
#       response4map3image = map3image4zpu(task4map3image)                                                                                                          
#       print('(Step 92B/700) OF ZPU3IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH MAP3IMAGE4ZPU : ', response4map3image)
#    
#       queue4map3image4zpu      = response4map3image['QUEUE']
#       channel4map3imagepublish4zpu = response4map3image['PUBLISHERCHANNEL']
#       channel4map3imageconsume4zpu = response4map3image['CONSUMERCHANNEL']
#   except:
#       print('(Step 92Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH MAP3IMAGE4ZPU ', time4xi()) 
#        
#   # -----------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE3COPYIMAGE4ZPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------------------      
#   print('(Step 93A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP3IMAGE4ZPU ', time4xi())      
#   try:                               
#       message3send4zpu = "<copy4xi>copy3mapimage4zpu</copy4xi>"
#       channel4map3imagepublish4zpu.send(message3send4zpu)
#       print("(Step 93B/700) OF ZPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4ZPU: ", channel4map3imagepublish4zpu, " MESSAGE3SEND4ZPU: " , message3send4zpu)                                                                                                
#   except:
#       print('(Step 93Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP3IMAGE4ZPU ', time4xi())   
# 
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE3COPYIMAGE4ZPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack3simulate4zpu is True:
#       print("(Step 94A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4ZPU ", time4xi())   
#       try:        
#           response3manoeuvre4copy = manoeuvre3mapimage4zpu(queue4map3image4zpu, channel4map3imageconsume4zpu, channel4map3imagepublish4zpu)
#           print("(Step 94B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED MANOEUVRE3COPYIMAGE4ZPU, RESPONSE3MANOEUVRE4COPY: ", response3manoeuvre4copy)
#        except:
#           print("(Step 94Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4ZPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#
#   # ---------------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND ROGER VIA PIPE                  
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 95A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP3IMAGE4ZPU ', time4xi())      
#   try:
#       message3receive4zpu = channel4map3imageconsume4zpu.recv()
#       print("(Step 95B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel4map3imageconsume4zpu, " CONTROL & COMMAND: ", message3receive4zpu)                                                                                                  
#   except:
#       print('(Step 95Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP3IMAGE4ZPU ', time4xi())          
#   
#   # ------------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE3COPYIMAGE4ZPU TO RECEIVE                
#   # ------------------------------------------------------------------------------------------------------------      
#   print('(Step 96A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP3IMAGE4ZPU ', time4xi())     
#   try:                               
#       message3send4zpu = "<ready4xi>ready3mapimage4zpu</ready4xi>"
#       channel4map3imagepublish4zpu.send(message3send4zpu)
#       print("(Step 96B/700) OF ZPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4ZPU: ", channel4map3imagepublish4zpu, " MESSAGE3SEND4ZPU: " , message3send4zpu)                                                                                                
#   except:
#       print('(Step 84Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP3IMAGE4ZPU ', time4xi())                                                
# 
#   # -------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE3COPYIMAGE4ZPU  - LET IT RECEIVE READY AND SEND READY                
#   # -------------------------------------------------------------------------------------------------------       
#   if ack3simulate4zpu is True:
#       print("(Step 97A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4ZPU ", time4xi())  
#       try:        
#           response3manoeuvre4copy = manoeuvre3mapimage4zpu(queue4map3image4zpu, channel4map3imageconsume4zpu, channel4map3imagepublish4zpu)
#           print("(Step 97B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGE4ZPU ", manoeuvre3copyimage4zpu)
#       except:
#           print("(Step 97Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4ZPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#            
#    
#   # ---------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND READY VIA PIPE                
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 98A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP3IMAGE4ZPU ', time4xi())     
#   try:
#       message3receive4zpu = channel4map3imageconsume4zpu.recv()
#       print("(Step 98B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel4map3imageconsume4zpu, " CONTROL & COMMAND: ", message3receive4zpu)                                                                                                  
#   except:
#       print('(Step 86Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP3IMAGE4ZPU ', time4xi())          
#                         
#   # -----------------------------------------------------------------------------------------
#   # IMAGE ZPUXI COPY OPERATIONS 
#   # -----------------------------------------------------------------------------------------         
#   print('(Step 99A/700) OF ZPU3IMAGE4XI5TEST - TRYING FILE OPERATIONS WITH IMAGE ZPUXI ', time4xi())       
#   try:        
#       path3dragon = '..//..//..//..//..//web//images//png//dragonxi.png'
#       file3zpu = open(path3zpu)
#       print("(Step 99B/700) OF ZPU3IMAGE4XI5TEST - FILENO: ", file3zpu.fileno( ))  
#       print("(Step 99C/700) OF ZPU3IMAGE4XI5TEST - MODE: ", file3zpu.mode)
#       print("(Step 99D/700) OF ZPU3IMAGE4XI5TEST - NAME: ", file3zpu.name)
#       offset3zpu = 10
#       file3zpu.seek(offset3zpu)
#       print("(Step 99E/700) OF ZPU3IMAGE4XI5TEST - PERFORMED SEEK WITH OFFSET: ", offset3zpu)
#       file3zpu.close()
#       print("(Step 99F/700) OF ZPU3IMAGE4XI5TEST - CLOSED IMAGE FILE: ", path3zpu)  
#   except:
#       print("(Step 87Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM FILE OPERATIONS WITH IMAGE: ", path3zpu)        
# 
#   # -----------------------------------------------------------------------------------------
#   # PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 100A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue4map3image4zpu.put(path3zpu)             
#       print("(Step 100B/700) OF ZPU3IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path3zpu) 
#   except:
#       print("(Step 88Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   
#   # -------------------------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE3COPYIMAGE4ZPU  - STATE III - GETS FROM QUEUE AND SENDS EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------------------------------------       
#   if ack3simulate4zpu is True:
#       print("(Step 101A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4ZPU ", time4xi()) 
#       try:        
#           response3manoeuvre4copy = manoeuvre3mapimage4zpu(queue4map3image4zpu, channel4map3imageconsume4zpu, channel4map3imagepublish4zpu)
#           print("(Step 101B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGE4ZPU ", manoeuvre3copyimage4zpu)
#       except:
#           print("(Step 101Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4ZPU ", time4xi())     
#   else:
#       pass # trust activated thread                     
#                 
#                                  
#   # ----------------------------------------------------------------------------------------------------
#   # RECEIVE ACK OR NACK ABOUT IMAGE OPERATION CONTROL & COMMAND VIA PIPE                
#   # ----------------------------------------------------------------------------------------------------          
#   print('(Step 102A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message3receive4zpu = channel4map3imageconsume4zpu.recv()
#       print("(Step 102B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel4map3imageconsume4zpu, " CONTROL & COMMAND: ", message3receive4zpu)                                                                                                   
#   except:
#       print('(Step 90Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#
#   
#   # ----------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE3COPYIMAGE4ZPU  - LET MANOEUVRE3COPYIMAGE4ZPU TO GO TO STATE 0               
#   # ----------------------------------------------------------------------------------------------------------------       
#   if ack3simulate4zpu is True:
#       print("(Step 103A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4ZPU ", time4xi())  
#       try:        
#           response3manoeuvre4copy = manoeuvre3mapimage4zpu(queue4map3image4zpu, channel4map3imageconsume4zpu, channel4map3imagepublish4zpu)
#           print("(Step 103B/700) OF ZPU3IMAGE4XI5TEST - GOT RESPONSE3MANOEUVRE4ZPU ", response3manoeuvre4copy)
#       except:
#           print("(Step 103Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4ZPU ", time4xi())     
#   else:
#       pass # trust activated thread  
#                                           
#">

# -------------------------------------------------------------------------------------------------   
def map3image4zpu(task4imagemap3zpu=None):

    '''
    maps 3D images for ZPU
    '''
    global zpu3image4xi                                                                                             # 3D-IMAGES FOR ZPU  
    global control3mapimage4zpu                                                                                     # COMMAND AND CONTROL FOR ZPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "map3image4zpu"
    method2print = "map3image4zpu"

    message2print = "{(Step 1A/33) started" + "; map3image4zpu-started: " + str(map3image4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4imagemap3zpu-to-map3image4zpu: " + str(task4imagemap3zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE ZPU3IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if zpu3image4xi is None:
            zpu3image4xi = create3image4zpu()                                                                              # INITIALIZE IMAGE FOR ZPU 
            message2print = "{(Step 1C/70) created" + "; zpu3image4xi-created-by-map3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create3image4zpu-skipped-by-map3image4zpu: " + str(map3image4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            zpu3image4xi = create3image4zpu()
            message2print = "{(Step 1E/70) set" + "; zpu3image4xi-set-by-map3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            zpu3image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; zpu3image4xi-yielded-by-map3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagemap3zpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR MAP - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagemap3zpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagemap3zpu-created-by-map3image4zpu to: " + str(task4imagemap3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4zpu = trace3image4zpu                                                              # CALLBACK FUNCTION FOR ZPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function3callback4zpu-created-by-map3image4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code3ack  = code3ack + '1B1-'
           
            args3callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "map3image4zpu"                                                          # MODULE OF ZPUIZED OPENCL MODULE
            args3callback['METHOD']  = "map3image4zpu"                                                          # FUNCTION OF ZPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map3image4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args3callback-created-by-map3image4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            code3ack  = code3ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap3zpu['DATA3CALLBACK'] = args3callback                                                   # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagemap3zpu-created-by-map3image4zpu to: " + str(task4imagemap3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-map3image4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-map3image4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                code3ack  = code3ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-map3image4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '2Z-'
                 
            task4imagemap3zpu['PLATFORM'] = platform3xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3zpu = {}
            message2print = "{(Step 3A/70) trying" + "; context3zpu-trying-by-map3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3zpu = get2context4zpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context3zpu-got-by-map3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context3zpu-failed-by-map3image4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '3Z-'
                
            task4imagemap3zpu ['CONTEXT'] = context3zpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3zpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device3zpu-trying-by-map3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3zpu = get2zpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device3zpu-got-by-map3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code3ack  = code3ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device3zpu-failed-by-map3image4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '4Z-'
            
            task4imagemap3zpu ['DEVICE'] = device3zpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo3zpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo3zpu-trying-by-map3image4zpu: " + str(imageinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo3zpu = get3imageinfo4zpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo3zpu-got-by-map3image4zpu: " + str(imageinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code3ack  = code3ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo3zpu-failed-by-map3image4zpu: " + str(imageinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '5Z-'          
                      
            task4imagemap3zpu ['IMAGE'] = imageinfo3zpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY ZPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3zpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue3zpu-trying-by-map3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3zpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue3zpu-got-by-map3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue3zpu-failed-by-map3image4zpu: " + str(queue3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '6Z-'
                
            task4imagemap3zpu ['QUEUE'] = queue3zpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY ZPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3zpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe3zpu-trying-by-map3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3zpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe3zpu-got-by-map3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe3zpu-failed-by-map3image4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '7Z-'
                
            task4imagemap3zpu ['PIPE'] = pipe3zpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3zpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo3zpu-trying-by-map3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo3zpu-got-by-map3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo3zpu-failed-by-map3image4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '8Z-'
        
            task4imagemap3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY ZPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3zpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo3zpu-trying-by-map3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3zpu = get2bufferinfo4zpu()                                                                   # SYSTEM CREATES BUFFER FOR ZPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo3zpu-got-by-map3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo3zpu-failed-by-map3image4zpu: " + str(bufferinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '9Z-'
        
            task4imagemap3zpu ['BUFFER'] = bufferinfo3zpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3zpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo3zpu-trying-by-map3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3zpu = get2eventinfo4zpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo3zpu-got-by-map3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo3zpu-failed-by-map3image4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '10Z-'
        
            task4imagemap3zpu ['EVENT'] = eventinfo3zpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3zpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist3zpu-trying-by-map3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3zpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist3zpu-set-by-map3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist3zpu-failed-by-map3image4zpu: " + str(eventwaitlist3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '11Z-'
        
            task4imagemap3zpu ['EVENTWAITLIST'] = eventwaitlist3zpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3zpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity3zpu-trying-by-map3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3zpu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3zpu = eventwaitlistquantity3zpu + 1                                                                      
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity3zpu-set-by-map3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity3zpu-failed-by-map3image4zpu: " + str(eventwaitlistquantity3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '12Z-'
        
            task4imagemap3zpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3zpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking3zpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking3zpu-trying-by-map3image4zpu: " + str(blocking3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking3zpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking3zpu-set-by-map3image4zpu to: " + str(blocking3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking3zpu-failed-by-map3image4zpu: " + str(blocking3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '13Z-'
        
            task4imagemap3zpu ['BLOCKING'] = blocking3zpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin3zpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin3zpu-trying-by-map3image4zpu: " + str(origin3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin3zpu['X'] = 0 
                origin3zpu['Y'] = 0  
                origin3zpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin3zpu-set-by-map3image4zpu to: " + str(origin3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                code3ack  = code3ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin3zpu-failed-by-map3image4zpu: " + str(origin3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '14Z-'
        
            task4imagemap3zpu ['ORIGIN'] = origin3zpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3zpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region3zpu-trying-by-map3image4zpu: " + str(region3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region3zpu['X'] = 330                                                                               # WIDTH  IN PIXELS TO MAP 
                region3zpu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO MAP 
                region3zpu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region3zpu-set-by-map3image4zpu to: " + str(region3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region3zpu-failed-by-map3image4zpu: " + str(region3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '15Z-'
        
            task4imagemap3zpu ['REGION'] = region3zpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch3zpu = 0
            size3element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 8                                                                                  # ONE DIRECTIONS IN 3D
            width3zpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch3zpu-trying-by-map3image4zpu: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3zpu = int(width3zpu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch3zpu-set-by-map3image4zpu to: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code3ack  = code3ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch3zpu-failed-by-map3image4zpu: " + str(rowpitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '16Z-'
        
            task4imagemap3zpu ['ROWPITCH'] = rowpitch3zpu     
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 3D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch3zpu = 0                                                                                   # SLICEPITCH IS ZERO FOR 3D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch3zpu-trying-by-map3image4zpu: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch3zpu = int(rowpitch3zpu * height3zpu)                                                       
                message2print = "{(Step 18B/70) set" + "; slicepitch3zpu-set-by-map3image4zpu to: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code3ack  = code3ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch3zpu-failed-by-map3image4zpu: " + str(slicepitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '18Z-'
            
            task4imagemap3zpu ['SLICEPITCH'] = slicepitch3zpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) FLAGS - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            flags3zpu = 'CL_MAP_READ'                                                                
     
            message2print = "{(Step 19A/70) trying" + "; flags3zpu-trying-by-map3image4zpu: " + str(flags3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags3zpu = 'CL_MAP_WRITE'  # THE REGION SPECIFIED BY (ORIGIN, REGION) IN THE IMAGE OBJECT IS BEING MAPPED FOR WRITING                                                                                                                     
                message2print = "{(Step 19B/70) set" + "; flags3zpu-set-by-map3image4zpu to: " + str(flags3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '19B-' 
            except:
                message2print = "{(Step 19Z/70) failed" + "; flags3zpu-failed-by-map3image4zpu: " + str(flags3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '19Z-'
        
            task4imagemap3zpu ['FLAGS'] = flags3zpu 
                                  
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imagemap3zpu.keys()
        message2print = "{(Step 20/70) got" + "; keys3task-got-by-map3image4zpu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
 

        if (('DATA3CALLBACK' in keys3task) and ('FUNCTION3CALLBACK' in keys3task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4zpu = trace3image4zpu                                                              # CALLBACK FUNCTION FOR ZPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function3callback4zpu-created-by-map3image4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code3ack  = code3ack + '21B1-'
           
            args3callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "map3image4zpu"                                                          # MODULE OF ZPUIZED OPENCL MODULE
            args3callback['METHOD']  = "map3image4zpu"                                                          # FUNCTION OF ZPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map3image4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args3callback-created-by-map3image4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code3ack  = code3ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap3zpu['DATA3CALLBACK'] = args3callback                                                   # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagemap3zpu-created-by-map3image4zpu to: " + str(task4imagemap3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys3task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform3xi-created-by-map3image4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform3xi-got-by-map3image4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code3ack  = code3ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform3xi-failed-by-map3image4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '22Z-'
                 
            task4imagemap3zpu['PLATFORM'] = platform3xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3zpu = {}
            message2print = "{(Step 23A/70) trying" + "; context3zpu-trying-by-map3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3zpu = get2context4zpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context3zpu-got-by-map3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context3zpu-failed-by-map3image4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '23Z-'
                
            task4imagemap3zpu ['CONTEXT'] = context3zpu           
        
        if 'DEVICE' in keys3task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3zpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device3zpu-trying-by-map3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3zpu = get2zpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device3zpu-got-by-map3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device3zpu-failed-by-map3image4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '24Z-'
            
            task4imagemap3zpu ['DEVICE'] = device3zpu
        
        if 'IMAGE' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo3zpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo3zpu-trying-by-map3image4zpu: " + str(imageinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo3zpu = get3imageinfo4zpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo3zpu-got-by-map3image4zpu: " + str(imageinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo3zpu-failed-by-map3image4zpu: " + str(imageinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '25Z-'          
                      
            task4imagemap3zpu ['IMAGE'] = imageinfo3zpu         
                    
        if 'QUEUE' in keys3task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY ZPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3zpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue3zpu-trying-by-map3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3zpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue3zpu-got-by-map3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue3zpu-failed-by-map3image4zpu: " + str(queue3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '26Z-'
                
            task4imagemap3zpu ['QUEUE'] = queue3zpu
        
        if 'PIPE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY ZPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3zpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe3zpu-trying-by-map3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3zpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe3zpu-got-by-map3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe3zpu-failed-by-map3image4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '27Z-'
                
            task4imagemap3zpu ['PIPE'] = pipe3zpu 
            
        if 'COMMANDQUEUE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3zpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo3zpu-trying-by-map3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo3zpu-got-by-map3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo3zpu-failed-by-map3image4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '28Z-'
        
            task4imagemap3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
        
        
        if 'BUFFER' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY ZPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3zpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo3zpu-trying-by-map3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3zpu = get2bufferinfo4zpu()                                                                   # SYSTEM CREATES BUFFER FOR ZPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo3zpu-got-by-map3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo3zpu-failed-by-map3image4zpu: " + str(bufferinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '29Z-'
        
            task4imagemap3zpu ['BUFFER'] = bufferinfo3zpu
             
        if 'EVENT' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3zpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo3zpu-trying-by-map3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3zpu = get2eventinfo4zpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo3zpu-got-by-map3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo3zpu-failed-by-map3image4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '30Z-'
        
            task4imagemap3zpu ['EVENT'] = eventinfo3zpu 
        
             
        if 'EVENTWAITLIST' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3zpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist3zpu-trying-by-map3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3zpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist3zpu-set-by-map3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist3zpu-failed-by-map3image4zpu: " + str(eventwaitlist3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '31Z-'
        
            task4imagemap3zpu ['EVENTWAITLIST'] = eventwaitlist3zpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3zpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity3zpu-trying-by-map3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3zpu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3zpu = eventwaitlistquantity3zpu + 1                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity3zpu-set-by-map3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity3zpu-failed-by-map3image4zpu: " + str(eventwaitlistquantity3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '32Z-'
        
            task4imagemap3zpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3zpu       
                                                 
                 
        if 'BLOCKING' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking3zpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking3zpu-trying-by-map3image4zpu: " + str(blocking3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking3zpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking3zpu-set-by-map3image4zpu to: " + str(blocking3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking3zpu-failed-by-map3image4zpu: " + str(blocking3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '33Z-'
        
            task4imagemap3zpu ['BLOCKING'] = blocking3zpu             
             
        if 'ORIGIN' in keys3task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin3zpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin3zpu-trying-by-map3image4zpu: " + str(origin3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin3zpu['X'] = 0 
                origin3zpu['Y'] = 0  
                origin3zpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin3zpu-set-by-map3image4zpu to: " + str(origin3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin3zpu-failed-by-map3image4zpu: " + str(origin3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '34Z-'
        
            task4imagemap3zpu ['ORIGIN'] = origin3zpu
            
             
        if 'REGION' in keys3task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3zpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region3zpu-trying-by-map3image4zpu: " + str(region3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region3zpu['X'] = 330                                                                               # WIDTH  IN PIXELS TO MAP 
                region3zpu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO MAP 
                region3zpu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region3zpu-set-by-map3image4zpu to: " + str(region3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region3zpu-failed-by-map3image4zpu: " + str(region3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '35Z-'
        
            task4imagemap3zpu ['REGION'] = region3zpu
        
            
             
        if 'ROWPITCH' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch3zpu = 0
            size3element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 1                                                                                  # ONE DIRECTIONS IN 3D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch3zpu-trying-by-map3image4zpu: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3zpu = int(width3zpu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch3zpu-set-by-map3image4zpu to: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code3ack  = code3ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch3zpu-failed-by-map3image4zpu: " + str(rowpitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '36Z-'
        
            task4imagemap3zpu ['ROWPITCH'] = rowpitch3zpu     
   
        if 'SLICEPITCH' in keys3task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 3D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch3zpu = 0                                                                                   # SLICEPITCH IS ZERO FOR 3D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch3zpu-trying-by-map3image4zpu: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch3zpu = int(rowpitch3zpu * height3zpu)                                                       
                message2print = "{(Step 38B/70) set" + "; slicepitch3zpu-set-by-map3image4zpu to: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch3zpu-failed-by-map3image4zpu: " + str(slicepitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '38Z-'
            
            task4imagemap3zpu ['SLICEPITCH'] = slicepitch3zpu
   
        if 'FLAGS' in keys3task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (39) FLAGS - REQUIRED                   
            # --------------------------------------------------------------------------------------------------------             
            flags3zpu = 'CL_MAP_READ'                                                                                                             
           
            message2print = "{(Step 39A/70) trying" + "; flags3zpu-trying-by-map3image4zpu: " + str(flags3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags3zpu = 'CL_MAP_WRITE'                                                          
                message2print = "{(Step 39B/70) set" + "; flags3zpu-set-by-map3image4zpu to: " + str(flags3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '39B-'                
                 
            except:
                message2print = "{(Step 39Z/70) failed" + "; flags3zpu-failed-by-map3image4zpu: " + str(flags3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '39Z-'
            
            task4imagemap3zpu ['FLAGS'] = flags3zpu
                                          
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imagemap3zpu.keys()
        message2print = "{(Step 40A/70) set" + "; keys3task-set-by-map3image4zpu to: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

                       
    except: 
        message2print = "{(Step 40Z/70) yielded" + "; task4imagemap3zpu-yielded-by-map3image4zpu: " + str(task4imagemap3zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '40Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control3mapimage4zpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels3zpu = False
    channels3zpu = ()
    message2print = "{(Step 40A/70) trying" + "channels3zpu-trying-by-map3image4zpu: " + str(channels3zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel3zpu, publisherchannel3zpu = task4imagemap3zpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel3zpu-got-by-map3image4zpu: " + str(consumerchannel3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel3zpu-got-by-map3image4zpu: " + str(publisherchannel3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                    
        ack4channels3zpu = True
        
        control3mapimage4zpu['CONSUMERCHANNEL']  = consumerchannel3zpu
        
        
        control3mapimage4zpu['PUBLISHERCHANNEL'] = publisherchannel3zpu
        
        code3ack  = code3ack + '40D-' 
        
    except:
        ack4channels3zpu = False
        code3nack  = code3nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue3zpu = False
    queue3zpu = None
    message2print = "{(Step 41A/70) trying" + "queue3zpu-trying-by-map3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue3zpu = task4imagemap3zpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue3zpu-got-by-map3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
              
        ack4queue3zpu = True
        control3mapimage4zpu['QUEUE']  = queue3zpu        
        code3ack  = code3ack + '41C-'
    except:
        ack4queue3zpu = False  
        code3nack  = code3nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy3image4zpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy3image4zpu-trying-by-map3image4zpu: " + str(dummy3image4zpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy3image4zpu = DummyProcess4Xi(target=manoeuvre3mapimage4zpu,                                            # in this module to handle queue and pipe channels
        #                                   args =(queue3zpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel3zpu, publisherchannel3zpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy3image4zpu-created-by-map3image4zpu: " + str(dummy3image4zpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code3ack  = code3ack + '69B-'
        
    # except:
        # code3nack  = code3nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code3ack-set-by-map3image4zpu: " + str(code3ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    message2print = "{(Step 70B/70) set" + "; code3nack-set-by-map3image4zpu: " + str(code3nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control3mapimage4zpu-returning-by-map3image4zpu: " + str(control3mapimage4zpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
    

        
    return control3mapimage4zpu
                    
#</function4xi> 

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre3copyimage4zpu" usage = "control function for copy3image4zpu function in this module">
# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre3copyimage4zpu(queue3copymanoeuvre4zpu, consumerchannel3copymanoeuvre4zpu, publisherchannel3copymanoeuvre4zpu):
    '''
    manoeuvres copy control and command
    '''
    global state3copyimage4zpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max3copyimagestate4zpu                                                                                        # MAX FOR STATE
    global ack3copyimageinfo4zpu                                                                                         # ACK FOR IMAGE INFO
    global message3copyimageget4zpu                                 
    global message3copyimageput4zpu
    global message3copyimagereceive4zpu
    global message3copyimagesend4zpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre3copyimage4zpu" 
    method2print = "manoeuvre3copyimage4zpu" 

    code3ack  = ' '
    code3nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre3copyimage4zpu-started: " + str(manoeuvre3copyimage4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue3copymanoeuvre4zpu-to-manoeuvre3copyimage4zpu: " + str(queue3copymanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel3copymanoeuvre4zpu-to-manoeuvre3copyimage4zpu: " + str(consumerchannel3copymanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel3copymanoeuvre4zpu-to-manoeuvre3copyimage4zpu: " + str(publisherchannel3copymanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code3ack  = code3ack + '0D-' 
    response3manouvre4copy = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state3copyimage4zpu-try-by-manoeuvre3copyimage4zpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state3copyimage4zpu is None:
            state3copyimage4zpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state3copyimage4zpu-initialized-by-manoeuvre3copyimage4zpu to: " + str(state3copyimage4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state3copyimage4zpu-used-by-manoeuvre3copyimage4zpu: " + str(state3copyimage4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
    except:
        state3copyimage4zpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state3copyimage4zpu-initialized-by-manoeuvre3copyimage4zpu to: " + str(state3copyimage4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3copyimageinfo4zpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack3copyimageinfo4zpu-initialized-by-manoeuvre3copyimage4zpu to: " + str(ack3copyimageinfo4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3copyimageget4zpu     = '<default4xi>default3copy4zpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message3copyimageget4zpu-initialized-by-manoeuvre3copyimage4zpu to: " + str(message3copyimageget4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3copyimageput4zpu     = '<default4xi>default3copy4zpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message3copyimageput4zpu-initialized-by-manoeuvre3copyimage4zpu to: " + str(message3copyimageput4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagereceive4zpu = '<default4xi>default3copy4zpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message3copyimagereceive4zpu-initialized-by-manoeuvre3copyimage4zpu to: " + str(message3copyimagereceive4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagesend4zpu    = '<default4xi>default3copy4zpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message3copyimagesend4zpu-initialized-by-manoeuvre3copyimage4zpu to: " + str(message3copyimagesend4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3copyimagestate4zpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max3copyimagestate4zpu-initialized-by-manoeuvre3copyimage4zpu to: " + str( max3copyimagestate4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state3copyimage4zpu = state3copyimage4zpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state3copyimage4zpu-added-by-manoeuvre3copyimage4zpu: " + str(state3copyimage4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    if state3copyimage4zpu > max3copyimagestate4zpu:
        state3copyimage4zpu = 0
        message2print = "{(Step 0I/33) set" + "; state3copyimage4zpu-set-by-manoeuvre3copyimage4zpu to: " + str(state3copyimage4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3copyimageinfo4zpu        = False
        message2print = "{(Step 0J/33) set" + "; ack3copyimageinfo4zpu-set-by-manoeuvre3copyimage4zpu to: " + str(ack3copyimageinfo4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3copyimageget4zpu     = '<default4xi>default3copy4zpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message3copyimageget4zpu-set-by-manoeuvre3copyimage4zpu to: " + str(message3copyimageget4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3copyimageput4zpu     = '<default4xi>default3copy4zpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message3copyimageput4zpu-set-by-manoeuvre3copyimage4zpu to: " + str(message3copyimageput4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagereceive4zpu = '<default4xi>default3copy4zpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message3copyimagereceive4zpu-set-by-manoeuvre3copyimage4zpu to: " + str(message3copyimagereceive4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagesend4zpu    = '<default4xi>default3copy4zpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message3copyimagesend4zpu-set-by-manoeuvre3copyimage4zpu to: " + str(message3copyimagesend4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3copyimagestate4zpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max3copyimagestate4zpu-set-by-manoeuvre3copyimage4zpu to: " + str( max3copyimagestate4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copyimage4zpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre3copyimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3copyimagereceive4zpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3copyimagereceive4zpu = consumerchannel3copymanoeuvre4zpu.recv()
            message2print = "{(Step 1B/33) received" + "; message3copyimagereceive4zpu-received-by-manoeuvre3copyimage4zpu: " + str(message3copyimagereceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3copyimagereceive4zpu == '<copy4xi>copy3copy4zpu</copy4xi>':
                message3copyimagesend4zpu = '<roger4xi>roger3copy4zpu</roger4xi>'
            else:
                message3copyimagesend4zpu = '<negative4xi><negative3copy4zpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3copymanoeuvre4zpu.send(message3copyimagesend4zpu)
            
            message2print = "{(Step 1C/33) sent" + "; message3copyimagesend4zpu-sent-by-manoeuvre3copyimage4zpu: " + str(message3copyimagesend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message3copyimagereceive4zpu-failed-by-manoeuvre3copyimage4zpu: " + str(message3copyimagereceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copyimage4zpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre3copyimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3copyimagereceive4zpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3copyimagereceive4zpu = consumerchannel3copymanoeuvre4zpu.recv()
            message2print = "{(Step 2B/33) received" + "; message3copyimagereceive4zpu-received-by-manoeuvre3copyimage4zpu: " + str(message3copyimagereceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3copyimagereceive4zpu == '<ready4xi>ready3copy4zpu</ready4xi>':
                message3copyimagesend4zpu = '<ready4xi>ready3copy4zpu</ready4xi>'
            else:
                message3copyimagesend4zpu = '<negative4xi><negative3copy4zpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3copymanoeuvre4zpu.send(message3copyimagesend4zpu)
            
            message2print = "{(Step 2C/33) sent" + "; message3copyimagesend4zpu-sent-by-manoeuvre3copyimage4zpu: " + str(message3copyimagesend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message3copyimagereceive4zpu-failed-by-manoeuvre3copyimage4zpu: " + str(message3copyimagereceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copyimage4zpu == 3:    
        info3image4zpu = None
        message2print = "{(Step 3A/70) try" + "; info3image4zpu-try-by-manoeuvre3copyimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info3image4zpu = queue3copymanoeuvre4zpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info3image4zpu-got-by-manoeuvre3copyimage4zpu: " + str(info3image4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info3image4zpu is None:
                code3nack  = code3nack + '3Z1-'
                ack3copyimageinfo4zpu = False 
            else:    
                code3ack  = code3ack + '3C-'
                ack3copyimageinfo4zpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue3copymanoeuvre4zpu-failed-by-manoeuvre3copyimage4zpu: " + str(queue3copymanoeuvre4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z1-'
            
        message3copyimagesend4zpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack3copyimageinfo4zpu is True:
                message3copyimagesend4zpu = '<ack4xi>ack3imageinfo4zpu</ack4xi>'
            else:
                message3copyimagesend4zpu = '<nack4xi>nack3imageinfo4zpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message3copyimagesend4zpu-set-by-manoeuvre3copyimage4zpu to: " + str(message3copyimagesend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #   
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3copyimagesend4zpu-failed-by-manoeuvre3copyimage4zpu: " + str(message3copyimagesend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code3nack  = code3nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel3copymanoeuvre4zpu-calling-by-manoeuvre3copyimage4zpu: " + str(publisherchannel3copymanoeuvre4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message3copyimagesend4zpu-passing-to-send-by-manoeuvre3copyimage4zpu: " + str(message3copyimagesend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3copyimagesend4zpu-failed-by-manoeuvre3copyimage4zpu: " + str(message3copyimagesend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z2-'                
        try:    
            publisherchannel3copymanoeuvre4zpu.send(message3copyimagesend4zpu)
            
            message2print = "{(Step 4E/33) sent" + "; message3copyimagesend4zpu-sent-by-manoeuvre3copyimage4zpu: " + str(message3copyimagesend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #         
            code3ack  = code3ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message3copyimagesend4zpu-failed-by-manoeuvre3copyimage4zpu: " + str(message3copyimagesend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response3manouvre4copy['ACK3INFO4ZPU']        = ack3copyimageinfo4zpu
    response3manouvre4copy['CODE3ACK4ZPU']        = code3ack
    response3manouvre4copy['CODE3NACK4ZPU']       = code3nack
    response3manouvre4copy['MESSAGE3GET4ZPU']     = message3copyimageget4zpu
    response3manouvre4copy['MESSAGE3PUT4ZPU']     = message3copyimageput4zpu
    response3manouvre4copy['MESSAGE3RECEIVE4ZPU'] = message3copyimagereceive4zpu
    response3manouvre4copy['MESSAGE3SEND4ZPU']    = message3copyimagesend4zpu
    response3manouvre4copy['STATE3MANOEUVRE4ZPU'] = state3copyimage4zpu
                          
    message2print = "{(Step 33/33) returning" + "; response3manouvre4copy-returning-by-manoeuvre3copyimage4zpu: " + str(response3manouvre4copy) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 

 
    #2 

   
    #3 

    
    #4 

    return response3manouvre4copy           
          
#</function4xi>


# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre3copyimagetobuffer4zpu" usage = "control function for copy3imagetobuffer4zpu function in this module">
# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre3copyimagetobuffer4zpu(queue3copyimagetobuffermanoeuvre4zpu, consumerchannel3copyimagetobuffermanoeuvre4zpu, publisherchannel3copyimagetobuffermanoeuvre4zpu):
    '''
    manoeuvres copy control and command for copy3imagetobuffer4zpu
    '''
    global state3copyimagetobuffer4zpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max3copyimagetobufferstate4zpu                                                                                        # MAX FOR STATE
    global ack3copyimagetobufferinfo4zpu                                                                                         # ACK FOR IMAGE INFO
    global message3copyimagetobufferget4zpu                                 
    global message3copyimagetobufferput4zpu
    global message3copyimagetobufferreceive4zpu
    global message3copyimagetobuffersend4zpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre3copyimagetobuffer4zpu" 
    method2print = "manoeuvre3copyimagetobuffer4zpu" 

    code3ack  = ' '
    code3nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre3copyimagetobuffer4zpu-started: " + str(manoeuvre3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue3copyimagetobuffermanoeuvre4zpu-to-manoeuvre3copyimagetobuffer4zpu: " + str(queue3copyimagetobuffermanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel3copyimagetobuffermanoeuvre4zpu-to-manoeuvre3copyimagetobuffer4zpu: " + str(consumerchannel3copyimagetobuffermanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel3copyimagetobuffermanoeuvre4zpu-to-manoeuvre3copyimagetobuffer4zpu: " + str(publisherchannel3copyimagetobuffermanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code3ack  = code3ack + '0D-' 
    response3manouvre4copyimagetobuffer = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state3copyimagetobuffer4zpu-try-by-manoeuvre3copyimagetobuffer4zpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state3copyimagetobuffer4zpu is None:
            state3copyimagetobuffer4zpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state3copyimagetobuffer4zpu-initialized-by-manoeuvre3copyimagetobuffer4zpu to: " + str(state3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state3copyimagetobuffer4zpu-used-by-manoeuvre3copyimagetobuffer4zpu: " + str(state3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
    except:
        state3copyimagetobuffer4zpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state3copyimagetobuffer4zpu-initialized-by-manoeuvre3copyimagetobuffer4zpu to: " + str(state3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3copyimagetobufferinfo4zpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack3copyimagetobufferinfo4zpu-initialized-by-manoeuvre3copyimagetobuffer4zpu to: " + str(ack3copyimagetobufferinfo4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3copyimagetobufferget4zpu     = '<default4xi>default3copyimagetobuffer4zpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message3copyimagetobufferget4zpu-initialized-by-manoeuvre3copyimagetobuffer4zpu to: " + str(message3copyimagetobufferget4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3copyimagetobufferput4zpu     = '<default4xi>default3copyimagetobuffer4zpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message3copyimagetobufferput4zpu-initialized-by-manoeuvre3copyimagetobuffer4zpu to: " + str(message3copyimagetobufferput4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagetobufferreceive4zpu = '<default4xi>default3copyimagetobuffer4zpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message3copyimagetobufferreceive4zpu-initialized-by-manoeuvre3copyimagetobuffer4zpu to: " + str(message3copyimagetobufferreceive4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagetobuffersend4zpu    = '<default4xi>default3copyimagetobuffer4zpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message3copyimagetobuffersend4zpu-initialized-by-manoeuvre3copyimagetobuffer4zpu to: " + str(message3copyimagetobuffersend4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3copyimagetobufferstate4zpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max3copyimagetobufferstate4zpu-initialized-by-manoeuvre3copyimagetobuffer4zpu to: " + str( max3copyimagetobufferstate4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state3copyimagetobuffer4zpu = state3copyimagetobuffer4zpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state3copyimagetobuffer4zpu-added-by-manoeuvre3copyimagetobuffer4zpu: " + str(state3copyimagetobuffer4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
     
    if state3copyimagetobuffer4zpu > max3copyimagetobufferstate4zpu:
        state3copyimagetobuffer4zpu = 0
        message2print = "{(Step 0I/33) set" + "; state3copyimagetobuffer4zpu-set-by-manoeuvre3copyimagetobuffer4zpu to: " + str(state3copyimagetobuffer4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3copyimagetobufferinfo4zpu        = False
        message2print = "{(Step 0J/33) set" + "; ack3copyimagetobufferinfo4zpu-set-by-manoeuvre3copyimagetobuffer4zpu to: " + str(ack3copyimagetobufferinfo4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3copyimagetobufferget4zpu     = '<default4xi>default3copyimagetobuffer4zpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message3copyimagetobufferget4zpu-set-by-manoeuvre3copyimagetobuffer4zpu to: " + str(message3copyimagetobufferget4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3copyimagetobufferput4zpu     = '<default4xi>default3copyimagetobuffer4zpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message3copyimagetobufferput4zpu-set-by-manoeuvre3copyimagetobuffer4zpu to: " + str(message3copyimagetobufferput4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagetobufferreceive4zpu = '<default4xi>default3copyimagetobuffer4zpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message3copyimagetobufferreceive4zpu-set-by-manoeuvre3copyimagetobuffer4zpu to: " + str(message3copyimagetobufferreceive4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagetobuffersend4zpu    = '<default4xi>default3copyimagetobuffer4zpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message3copyimagetobuffersend4zpu-set-by-manoeuvre3copyimagetobuffer4zpu to: " + str(message3copyimagetobuffersend4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3copyimagetobufferstate4zpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max3copyimagetobufferstate4zpu-set-by-manoeuvre3copyimagetobuffer4zpu to: " + str( max3copyimagetobufferstate4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copyimagetobuffer4zpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre3copyimagetobuffer4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3copyimagetobufferreceive4zpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3copyimagetobufferreceive4zpu = consumerchannel3copyimagetobuffermanoeuvre4zpu.recv()
            message2print = "{(Step 1B/33) received" + "; message3copyimagetobufferreceive4zpu-received-by-manoeuvre3copyimagetobuffer4zpu: " + str(message3copyimagetobufferreceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3copyimagetobufferreceive4zpu == '<copy4xi>copy3copyimagetobuffer4zpu</copy4xi>': 
                message3copyimagetobuffersend4zpu = '<roger4xi>roger3copyimagetobuffer4zpu</roger4xi>'
            else:
                message3copyimagetobuffersend4zpu = '<negative4xi><negative3copyimagetobuffer4zpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3copyimagetobuffermanoeuvre4zpu.send(message3copyimagetobuffersend4zpu)
            
            message2print = "{(Step 1C/33) sent" + "; message3copyimagetobuffersend4zpu-sent-by-manoeuvre3copyimagetobuffer4zpu: " + str(message3copyimagetobuffersend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message3copyimagetobufferreceive4zpu-failed-by-manoeuvre3copyimagetobuffer4zpu: " + str(message3copyimagetobufferreceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copyimagetobuffer4zpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre3copyimagetobuffer4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3copyimagetobufferreceive4zpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3copyimagetobufferreceive4zpu = consumerchannel3copyimagetobuffermanoeuvre4zpu.recv()
            message2print = "{(Step 2B/33) received" + "; message3copyimagetobufferreceive4zpu-received-by-manoeuvre3copyimagetobuffer4zpu: " + str(message3copyimagetobufferreceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3copyimagetobufferreceive4zpu == '<ready4xi>ready3copyimagetobuffer4zpu</ready4xi>':
                message3copyimagetobuffersend4zpu = '<ready4xi>ready3copyimagetobuffer4zpu</ready4xi>'
            else:
                message3copyimagetobuffersend4zpu = '<negative4xi><negative3copyimagetobuffer4zpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3copyimagetobuffermanoeuvre4zpu.send(message3copyimagetobuffersend4zpu)
            
            message2print = "{(Step 2C/33) sent" + "; message3copyimagetobuffersend4zpu-sent-by-manoeuvre3copyimagetobuffer4zpu: " + str(message3copyimagetobuffersend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message3copyimagetobufferreceive4zpu-failed-by-manoeuvre3copyimagetobuffer4zpu: " + str(message3copyimagetobufferreceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copyimagetobuffer4zpu == 3:    
        info3image4zpu = None
        message2print = "{(Step 3A/70) try" + "; info3image4zpu-try-by-manoeuvre3copyimagetobuffer4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info3image4zpu = queue3copyimagetobuffermanoeuvre4zpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info3image4zpu-got-by-manoeuvre3copyimagetobuffer4zpu: " + str(info3image4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info3image4zpu is None:
                code3nack  = code3nack + '3Z1-'
                ack3copyimagetobufferinfo4zpu = False 
            else:    
                code3ack  = code3ack + '3C-'
                ack3copyimagetobufferinfo4zpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue3copyimagetobuffermanoeuvre4zpu-failed-by-manoeuvre3copyimagetobuffer4zpu: " + str(queue3copyimagetobuffermanoeuvre4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z1-'
            
        message3copyimagetobuffersend4zpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack3copyimagetobufferinfo4zpu is True:
                message3copyimagetobuffersend4zpu = '<ack4xi>ack3imageinfo4zpu</ack4xi>'
            else:
                message3copyimagetobuffersend4zpu = '<nack4xi>nack3imageinfo4zpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message3copyimagetobuffersend4zpu-set-by-manoeuvre3copyimagetobuffer4zpu to: " + str(message3copyimagetobuffersend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3copyimagetobuffersend4zpu-failed-by-manoeuvre3copyimagetobuffer4zpu: " + str(message3copyimagetobuffersend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code3nack  = code3nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel3copyimagetobuffermanoeuvre4zpu-calling-by-manoeuvre3copyimagetobuffer4zpu: " + str(publisherchannel3copyimagetobuffermanoeuvre4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message3copyimagetobuffersend4zpu-passing-to-send-by-manoeuvre3copyimagetobuffer4zpu: " + str(message3copyimagetobuffersend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3copyimagetobuffersend4zpu-failed-by-manoeuvre3copyimagetobuffer4zpu: " + str(message3copyimagetobuffersend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z2-'                
        try:    
            publisherchannel3copyimagetobuffermanoeuvre4zpu.send(message3copyimagetobuffersend4zpu)
            
            message2print = "{(Step 4E/33) sent" + "; message3copyimagetobuffersend4zpu-sent-by-manoeuvre3copyimagetobuffer4zpu: " + str(message3copyimagetobuffersend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #       
            code3ack  = code3ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message3copyimagetobuffersend4zpu-failed-by-manoeuvre3copyimagetobuffer4zpu: " + str(message3copyimagetobuffersend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response3manouvre4copyimagetobuffer['ACK3INFO4ZPU']        = ack3copyimagetobufferinfo4zpu
    response3manouvre4copyimagetobuffer['CODE3ACK4ZPU']        = code3ack
    response3manouvre4copyimagetobuffer['CODE3NACK4ZPU']       = code3nack
    response3manouvre4copyimagetobuffer['MESSAGE3GET4ZPU']     = message3copyimagetobufferget4zpu
    response3manouvre4copyimagetobuffer['MESSAGE3PUT4ZPU']     = message3copyimagetobufferput4zpu
    response3manouvre4copyimagetobuffer['MESSAGE3RECEIVE4ZPU'] = message3copyimagetobufferreceive4zpu
    response3manouvre4copyimagetobuffer['MESSAGE3SEND4ZPU']    = message3copyimagetobuffersend4zpu
    response3manouvre4copyimagetobuffer['STATE3MANOEUVRE4ZPU'] = state3copyimagetobuffer4zpu
                          
    message2print = "{(Step 33/33) returning" + "; response3manouvre4copyimagetobuffer-returning-by-manoeuvre3copyimagetobuffer4zpu: " + str(response3manouvre4copyimagetobuffer) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 

    
    #2 

    #3 


    #4 


    return response3manouvre4copyimagetobuffer           
          
#</function4xi>

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre3copybuffertoimage4zpu" usage = "control function for copy3imagetobuffer4zpu function in this module">
# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre3copybuffertoimage4zpu(queue3copybuffertoimagemanoeuvre4zpu, consumerchannel3copybuffertoimagemanoeuvre4zpu, publisherchannel3copybuffertoimagemanoeuvre4zpu):
    '''
    manoeuvres copy control and command for copy3buffertoimage4zpu
    '''
    global state3copybuffertoimage4zpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max3copybuffertoimagestate4zpu                                                                                        # MAX FOR STATE
    global ack3copybuffertoimageinfo4zpu                                                                                         # ACK FOR IMAGE INFO
    global message3copybuffertoimageget4zpu                                 
    global message3copybuffertoimageput4zpu
    global message3copybuffertoimagereceive4zpu
    global message3copybuffertoimagesend4zpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre3copybuffertoimage4zpu" 
    method2print = "manoeuvre3copybuffertoimage4zpu" 

    code3ack  = ' '
    code3nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre3copybuffertoimage4zpu-started: " + str(manoeuvre3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue3copybuffertoimagemanoeuvre4zpu-to-manoeuvre3copybuffertoimage4zpu: " + str(queue3copybuffertoimagemanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel3copybuffertoimagemanoeuvre4zpu-to-manoeuvre3copybuffertoimage4zpu: " + str(consumerchannel3copybuffertoimagemanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel3copybuffertoimagemanoeuvre4zpu-to-manoeuvre3copybuffertoimage4zpu: " + str(publisherchannel3copybuffertoimagemanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code3ack  = code3ack + '0D-' 
    response3manouvre4copybuffertoimage = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state3copybuffertoimage4zpu-try-by-manoeuvre3copybuffertoimage4zpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state3copybuffertoimage4zpu is None:
            state3copybuffertoimage4zpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state3copybuffertoimage4zpu-initialized-by-manoeuvre3copybuffertoimage4zpu to: " + str(state3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state3copybuffertoimage4zpu-used-by-manoeuvre3copybuffertoimage4zpu: " + str(state3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
    except:
        state3copybuffertoimage4zpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state3copybuffertoimage4zpu-initialized-by-manoeuvre3copybuffertoimage4zpu to: " + str(state3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3copybuffertoimageinfo4zpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack3copybuffertoimageinfo4zpu-initialized-by-manoeuvre3copybuffertoimage4zpu to: " + str(ack3copybuffertoimageinfo4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3copybuffertoimageget4zpu     = '<default4xi>default3copybuffertoimage4zpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message3copybuffertoimageget4zpu-initialized-by-manoeuvre3copybuffertoimage4zpu to: " + str(message3copybuffertoimageget4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3copybuffertoimageput4zpu     = '<default4xi>default3copybuffertoimage4zpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message3copybuffertoimageput4zpu-initialized-by-manoeuvre3copybuffertoimage4zpu to: " + str(message3copybuffertoimageput4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copybuffertoimagereceive4zpu = '<default4xi>default3copybuffertoimage4zpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message3copybuffertoimagereceive4zpu-initialized-by-manoeuvre3copybuffertoimage4zpu to: " + str(message3copybuffertoimagereceive4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copybuffertoimagesend4zpu    = '<default4xi>default3copybuffertoimage4zpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message3copybuffertoimagesend4zpu-initialized-by-manoeuvre3copybuffertoimage4zpu to: " + str(message3copybuffertoimagesend4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3copybuffertoimagestate4zpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max3copybuffertoimagestate4zpu-initialized-by-manoeuvre3copybuffertoimage4zpu to: " + str( max3copybuffertoimagestate4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state3copybuffertoimage4zpu = state3copybuffertoimage4zpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state3copybuffertoimage4zpu-added-by-manoeuvre3copybuffertoimage4zpu: " + str(state3copybuffertoimage4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
     
    if state3copybuffertoimage4zpu > max3copybuffertoimagestate4zpu:
        state3copybuffertoimage4zpu = 0
        message2print = "{(Step 0I/33) set" + "; state3copybuffertoimage4zpu-set-by-manoeuvre3copybuffertoimage4zpu to: " + str(state3copybuffertoimage4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3copybuffertoimageinfo4zpu        = False
        message2print = "{(Step 0J/33) set" + "; ack3copybuffertoimageinfo4zpu-set-by-manoeuvre3copybuffertoimage4zpu to: " + str(ack3copybuffertoimageinfo4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3copybuffertoimageget4zpu     = '<default4xi>default3copybuffertoimage4zpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message3copybuffertoimageget4zpu-set-by-manoeuvre3copybuffertoimage4zpu to: " + str(message3copybuffertoimageget4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3copybuffertoimageput4zpu     = '<default4xi>default3copybuffertoimage4zpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message3copybuffertoimageput4zpu-set-by-manoeuvre3copybuffertoimage4zpu to: " + str(message3copybuffertoimageput4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copybuffertoimagereceive4zpu = '<default4xi>default3copybuffertoimage4zpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message3copybuffertoimagereceive4zpu-set-by-manoeuvre3copybuffertoimage4zpu to: " + str(message3copybuffertoimagereceive4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copybuffertoimagesend4zpu    = '<default4xi>default3copybuffertoimage4zpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message3copybuffertoimagesend4zpu-set-by-manoeuvre3copybuffertoimage4zpu to: " + str(message3copybuffertoimagesend4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3copybuffertoimagestate4zpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max3copybuffertoimagestate4zpu-set-by-manoeuvre3copybuffertoimage4zpu to: " + str( max3copybuffertoimagestate4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copybuffertoimage4zpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre3copybuffertoimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3copybuffertoimagereceive4zpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3copybuffertoimagereceive4zpu = consumerchannel3copybuffertoimagemanoeuvre4zpu.recv()
            message2print = "{(Step 1B/33) received" + "; message3copybuffertoimagereceive4zpu-received-by-manoeuvre3copybuffertoimage4zpu: " + str(message3copybuffertoimagereceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3copybuffertoimagereceive4zpu == '<copy4xi>copy3copybuffertoimage4zpu</copy4xi>':
                message3copybuffertoimagesend4zpu = '<roger4xi>roger3copybuffertoimage4zpu</roger4xi>'
            else:
                message3copybuffertoimagesend4zpu = '<negative4xi><negative3copybuffertoimage4zpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3copybuffertoimagemanoeuvre4zpu.send(message3copybuffertoimagesend4zpu)
            
            message2print = "{(Step 1C/33) sent" + "; message3copybuffertoimagesend4zpu-sent-by-manoeuvre3copybuffertoimage4zpu: " + str(message3copybuffertoimagesend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message3copybuffertoimagereceive4zpu-failed-by-manoeuvre3copybuffertoimage4zpu: " + str(message3copybuffertoimagereceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copybuffertoimage4zpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre3copybuffertoimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3copybuffertoimagereceive4zpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3copybuffertoimagereceive4zpu = consumerchannel3copybuffertoimagemanoeuvre4zpu.recv()
            message2print = "{(Step 2B/33) received" + "; message3copybuffertoimagereceive4zpu-received-by-manoeuvre3copybuffertoimage4zpu: " + str(message3copybuffertoimagereceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3copybuffertoimagereceive4zpu == '<ready4xi>ready3copybuffertoimage4zpu</ready4xi>':
                message3copybuffertoimagesend4zpu = '<ready4xi>ready3copybuffertoimage4zpu</ready4xi>'
            else:
                message3copybuffertoimagesend4zpu = '<negative4xi><negative3copybuffertoimage4zpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3copybuffertoimagemanoeuvre4zpu.send(message3copybuffertoimagesend4zpu)
            
            message2print = "{(Step 2C/33) sent" + "; message3copybuffertoimagesend4zpu-sent-by-manoeuvre3copybuffertoimage4zpu: " + str(message3copybuffertoimagesend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message3copybuffertoimagereceive4zpu-failed-by-manoeuvre3copybuffertoimage4zpu: " + str(message3copybuffertoimagereceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copybuffertoimage4zpu == 3:    
        info3image4zpu = None
        message2print = "{(Step 3A/70) try" + "; info3image4zpu-try-by-manoeuvre3copybuffertoimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info3image4zpu = queue3copybuffertoimagemanoeuvre4zpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info3image4zpu-got-by-manoeuvre3copybuffertoimage4zpu: " + str(info3image4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info3image4zpu is None:
                code3nack  = code3nack + '3Z1-'
                ack3copybuffertoimageinfo4zpu = False 
            else:    
                code3ack  = code3ack + '3C-'
                ack3copybuffertoimageinfo4zpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue3copybuffertoimagemanoeuvre4zpu-failed-by-manoeuvre3copybuffertoimage4zpu: " + str(queue3copybuffertoimagemanoeuvre4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z1-'
            
        message3copybuffertoimagesend4zpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack3copybuffertoimageinfo4zpu is True:
                message3copybuffertoimagesend4zpu = '<ack4xi>ack3imageinfo4zpu</ack4xi>'
            else:
                message3copybuffertoimagesend4zpu = '<nack4xi>nack3imageinfo4zpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message3copybuffertoimagesend4zpu-set-by-manoeuvre3copybuffertoimage4zpu to: " + str(message3copybuffertoimagesend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3copybuffertoimagesend4zpu-failed-by-manoeuvre3copybuffertoimage4zpu: " + str(message3copybuffertoimagesend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code3nack  = code3nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel3copybuffertoimagemanoeuvre4zpu-calling-by-manoeuvre3copybuffertoimage4zpu: " + str(publisherchannel3copybuffertoimagemanoeuvre4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message3copybuffertoimagesend4zpu-passing-to-send-by-manoeuvre3copybuffertoimage4zpu: " + str(message3copybuffertoimagesend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3copybuffertoimagesend4zpu-failed-by-manoeuvre3copybuffertoimage4zpu: " + str(message3copybuffertoimagesend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z2-'                
        try:    
            publisherchannel3copybuffertoimagemanoeuvre4zpu.send(message3copybuffertoimagesend4zpu)
            
            message2print = "{(Step 4E/33) sent" + "; message3copybuffertoimagesend4zpu-sent-by-manoeuvre3copybuffertoimage4zpu: " + str(message3copybuffertoimagesend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #      
            code3ack  = code3ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message3copybuffertoimagesend4zpu-failed-by-manoeuvre3copybuffertoimage4zpu: " + str(message3copybuffertoimagesend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response3manouvre4copybuffertoimage['ACK3INFO4ZPU']        = ack3copybuffertoimageinfo4zpu
    response3manouvre4copybuffertoimage['CODE3ACK4ZPU']        = code3ack
    response3manouvre4copybuffertoimage['CODE3NACK4ZPU']       = code3nack
    response3manouvre4copybuffertoimage['MESSAGE3GET4ZPU']     = message3copybuffertoimageget4zpu
    response3manouvre4copybuffertoimage['MESSAGE3PUT4ZPU']     = message3copybuffertoimageput4zpu
    response3manouvre4copybuffertoimage['MESSAGE3RECEIVE4ZPU'] = message3copybuffertoimagereceive4zpu
    response3manouvre4copybuffertoimage['MESSAGE3SEND4ZPU']    = message3copybuffertoimagesend4zpu
    response3manouvre4copybuffertoimage['STATE3MANOEUVRE4ZPU'] = state3copybuffertoimage4zpu
                          
    message2print = "{(Step 33/33) returning" + "; response3manouvre4copybuffertoimage-returning-by-manoeuvre3copybuffertoimage4zpu: " + str(response3manouvre4copybuffertoimage) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 

    
    #2 

    #3 
 

    #4 

    return response3manouvre4copybuffertoimage           
          
#</function4xi>



# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre3readimage4zpu" usage = "control function for read3image4zpu function in this module">
# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre3readimage4zpu(queue3readmanoeuvre4zpu, consumerchannel3readmanoeuvre4zpu, publisherchannel3readmanoeuvre4zpu):
    '''
    manoeuvres read control and command
    '''
    global state3read4zpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max3readstate4zpu                                                                                        # MAX FOR STATE
    global ack3readinfo4zpu                                                                                         # ACK FOR IMAGE INFO
    global message3readget4zpu                                 
    global message3readput4zpu
    global message3readreceive4zpu
    global message3readsend4zpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre3readimage4zpu" 
    method2print = "manoeuvre3readimage4zpu" 

    code3ack  = ' '
    code3nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre3readimage4zpu-started: " + str(manoeuvre3readimage4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue3readmanoeuvre4zpu-to-manoeuvre3readimage4zpu: " + str(queue3readmanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel3readmanoeuvre4zpu-to-manoeuvre3readimage4zpu: " + str(consumerchannel3readmanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel3readmanoeuvre4zpu-to-manoeuvre3readimage4zpu: " + str(publisherchannel3readmanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    #   
    
    code3ack  = code3ack + '0D-' 
    response3manouvre4read = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state3read4zpu-try-by-manoeuvre3readimage4zpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state3read4zpu is None:
            state3read4zpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state3read4zpu-initialized-by-manoeuvre3readimage4zpu to: " + str(state3read4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state3read4zpu-used-by-manoeuvre3readimage4zpu: " + str(state3read4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
    except:
        state3read4zpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state3read4zpu-initialized-by-manoeuvre3readimage4zpu to: " + str(state3read4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3readinfo4zpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack3readinfo4zpu-initialized-by-manoeuvre3readimage4zpu to: " + str(ack3readinfo4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3readget4zpu     = '<default4xi>default3read4zpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message3readget4zpu-initialized-by-manoeuvre3readimage4zpu to: " + str(message3readget4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3readput4zpu     = '<default4xi>default3read4zpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message3readput4zpu-initialized-by-manoeuvre3readimage4zpu to: " + str(message3readput4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3readreceive4zpu = '<default4xi>default3read4zpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message3readreceive4zpu-initialized-by-manoeuvre3readimage4zpu to: " + str(message3readreceive4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3readsend4zpu    = '<default4xi>default3read4zpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message3readsend4zpu-initialized-by-manoeuvre3readimage4zpu to: " + str(message3readsend4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3readstate4zpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max3readstate4zpu-initialized-by-manoeuvre3readimage4zpu to: " + str( max3readstate4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state3read4zpu = state3read4zpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state3read4zpu-added-by-manoeuvre3readimage4zpu: " + str(state3read4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    if state3read4zpu > max3readstate4zpu:
        state3read4zpu = 0
        message2print = "{(Step 0I/33) set" + "; state3read4zpu-set-by-manoeuvre3readimage4zpu to: " + str(state3read4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3readinfo4zpu        = False
        message2print = "{(Step 0J/33) set" + "; ack3readinfo4zpu-set-by-manoeuvre3readimage4zpu to: " + str(ack3readinfo4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3readget4zpu     = '<default4xi>default3read4zpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message3readget4zpu-set-by-manoeuvre3readimage4zpu to: " + str(message3readget4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3readput4zpu     = '<default4xi>default3read4zpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message3readput4zpu-set-by-manoeuvre3readimage4zpu to: " + str(message3readput4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3readreceive4zpu = '<default4xi>default3read4zpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message3readreceive4zpu-set-by-manoeuvre3readimage4zpu to: " + str(message3readreceive4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3readsend4zpu    = '<default4xi>default3read4zpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message3readsend4zpu-set-by-manoeuvre3readimage4zpu to: " + str(message3readsend4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3readstate4zpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max3readstate4zpu-set-by-manoeuvre3readimage4zpu to: " + str( max3readstate4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3read4zpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre3readimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3readreceive4zpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3readreceive4zpu = consumerchannel3readmanoeuvre4zpu.recv()
            message2print = "{(Step 1B/33) received" + "; message3readreceive4zpu-received-by-manoeuvre3readimage4zpu: " + str(message3readreceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3readreceive4zpu == '<copy4xi>copy3read4zpu</copy4xi>':
                message3readsend4zpu = '<roger4xi>roger3read4zpu</roger4xi>'
            else:
                message3readsend4zpu = '<negative4xi><negative3read4zpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3readmanoeuvre4zpu.send(message3readsend4zpu)
            
            message2print = "{(Step 1C/33) sent" + "; message3readsend4zpu-sent-by-manoeuvre3readimage4zpu: " + str(message3readsend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message3readreceive4zpu-failed-by-manoeuvre3readimage4zpu: " + str(message3readreceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3read4zpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre3readimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3readreceive4zpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3readreceive4zpu = consumerchannel3readmanoeuvre4zpu.recv()
            message2print = "{(Step 2B/33) received" + "; message3readreceive4zpu-received-by-manoeuvre3readimage4zpu: " + str(message3readreceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3readreceive4zpu == '<ready4xi>ready3read4zpu</ready4xi>':
                message3readsend4zpu = '<ready4xi>ready3read4zpu</ready4xi>'
            else:
                message3readsend4zpu = '<negative4xi><negative3ready4zpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3readmanoeuvre4zpu.send(message3readsend4zpu)
            
            message2print = "{(Step 2C/33) sent" + "; message3readsend4zpu-sent-by-manoeuvre3readimage4zpu: " + str(message3readsend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message3readreceive4zpu-failed-by-manoeuvre3readimage4zpu: " + str(message3readreceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state3read4zpu == 3:    
        info3image4zpu = None
        message2print = "{(Step 3A/70) try" + "; info3image4zpu-try-by-manoeuvre3readimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info3image4zpu = queue3readmanoeuvre4zpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info3image4zpu-got-by-manoeuvre3readimage4zpu: " + str(info3image4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info3image4zpu is None:
                code3nack  = code3nack + '3Z1-'
                ack3readinfo4zpu = False 
            else:    
                code3ack  = code3ack + '3C-'
                ack3readinfo4zpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue3readmanoeuvre4zpu-failed-by-manoeuvre3readimage4zpu: " + str(queue3readmanoeuvre4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z1-'
            
        message3readsend4zpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack3readinfo4zpu is True:
                message3readsend4zpu = '<ack4xi>ack3imageinfo4zpu</ack4xi>'
            else:
                message3readsend4zpu = '<nack4xi>nack3imageinfo4zpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message3readsend4zpu-set-by-manoeuvre3readimage4zpu to: " + str(message3readsend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #   
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3readsend4zpu-failed-by-manoeuvre3readimage4zpu: " + str(message3readsend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code3nack  = code3nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel3readmanoeuvre4zpu-calling-by-manoeuvre3readimage4zpu: " + str(publisherchannel3readmanoeuvre4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message3readsend4zpu-passing-to-send-by-manoeuvre3readimage4zpu: " + str(message3readsend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3readsend4zpu-failed-by-manoeuvre3readimage4zpu: " + str(message3readsend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z2-'                
        try:    
            publisherchannel3readmanoeuvre4zpu.send(message3readsend4zpu)
            
            message2print = "{(Step 4E/33) sent" + "; message3readsend4zpu-sent-by-manoeuvre3readimage4zpu: " + str(message3readsend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #         
            code3ack  = code3ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message3readsend4zpu-failed-by-manoeuvre3readimage4zpu: " + str(message3readsend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response3manouvre4read['ACK3INFO4ZPU']        = ack3readinfo4zpu
    response3manouvre4read['CODE3ACK4ZPU']        = code3ack
    response3manouvre4read['CODE3NACK4ZPU']       = code3nack
    response3manouvre4read['MESSAGE3GET4ZPU']     = message3readget4zpu
    response3manouvre4read['MESSAGE3PUT4ZPU']     = message3readput4zpu
    response3manouvre4read['MESSAGE3RECEIVE4ZPU'] = message3readreceive4zpu
    response3manouvre4read['MESSAGE3SEND4ZPU']    = message3readsend4zpu
    response3manouvre4read['STATE3MANOEUVRE4ZPU'] = state3read4zpu
                          
    message2print = "{(Step 33/33) returning" + "; response3manouvre4read-returning-by-manoeuvre3readimage4zpu: " + str(response3manouvre4read) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 
    # {'CODE3NACK4ZPU': ' ',
    #  'STATE3MANOEUVRE4ZPU': 1,
    #  'MESSAGE3PUT4ZPU': '<default4xi>default3read4zpu</default4xi>',
    #  'MESSAGE3SEND4ZPU': '<roger4xi>roger3read4zpu</roger4xi>',
    #  'MESSAGE3RECEIVE4ZPU': '<copy4xi>copy3read4zpu</copy4xi>',
    #  'ACK3INFO4ZPU': False,
    #  'MESSAGE3GET4ZPU': '<default4xi>default3read4zpu</default4xi>',
    #  'CODE3ACK4ZPU': ' 0D-1C-'}; time: 07:14:55 10/10/11 Pacific Daylight Time}
    
    #2 
    
    #3 
    
    #4 
    
    return response3manouvre4read           
          
#</function4xi>


# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre3writeimage4zpu" usage = "control function for write3image4zpu function in this module">
# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre3writeimage4zpu(queue3writemanoeuvre4zpu, consumerchannel3writemanoeuvre4zpu, publisherchannel3writemanoeuvre4zpu):
    '''
    manoeuvres control and command
    '''
    global state3write4zpu                                                       # STATE FOR COMMAND AND CONTROL
    global max3writestate4zpu                                                    # MAX FOR STATE
    global ack3writeinfo4zpu                                                     # ACK FOR IMAGE INFO
    global message3writeget4zpu                                 
    global message3writeput4zpu
    global message3writereceive4zpu
    global message3writesend4zpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre3writeimage4zpu" 
    method2print = "manoeuvre3writeimage4zpu" 

    code3ack  = ' '
    code3nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre3writeimage4zpu-started: " + str(manoeuvre3writeimage4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue3writemanoeuvre4zpu-to-manoeuvre3writeimage4zpu: " + str(queue3writemanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel3writemanoeuvre4zpu-to-manoeuvre3writeimage4zpu: " + str(consumerchannel3writemanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel3writemanoeuvre4zpu-to-manoeuvre3writeimage4zpu: " + str(publisherchannel3writemanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code3ack  = code3ack + '0D-' 
    response3manouvre4write = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state3write4zpu-try-by-manoeuvre3writeimage4zpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state3write4zpu is None:
            state3write4zpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state3write4zpu-initialized-by-manoeuvre3writeimage4zpu to: " + str(state3write4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state3write4zpu-used-by-manoeuvre3writeimage4zpu: " + str(state3write4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state3write4zpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state3write4zpu-initialized-by-manoeuvre3writeimage4zpu to: " + str(state3write4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3writeinfo4zpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack3writeinfo4zpu-initialized-by-manoeuvre3writeimage4zpu to: " + str(ack3writeinfo4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3writeget4zpu     = '<default4xi>default3write4zpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message3writeget4zpu-initialized-by-manoeuvre3writeimage4zpu to: " + str(message3writeget4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3writeput4zpu     = '<default4xi>default3write4zpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message3writeput4zpu-initialized-by-manoeuvre3writeimage4zpu to: " + str(message3writeput4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3writereceive4zpu = '<default4xi>default3write4zpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message3writereceive4zpu-initialized-by-manoeuvre3writeimage4zpu to: " + str(message3writereceive4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3writesend4zpu    = '<default4xi>default3write4zpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message3writesend4zpu-initialized-by-manoeuvre3writeimage4zpu to: " + str(message3writesend4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3writestate4zpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max3writestate4zpu-initialized-by-manoeuvre3writeimage4zpu to: " + str( max3writestate4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state3write4zpu = state3write4zpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state3write4zpu-added-by-manoeuvre3writeimage4zpu: " + str(state3write4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    if state3write4zpu > max3writestate4zpu:
        state3write4zpu = 0
        message2print = "{(Step 0I/33) set" + "; state3write4zpu-set-by-manoeuvre3writeimage4zpu to: " + str(state3write4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3writeinfo4zpu        = False
        message2print = "{(Step 0J/33) set" + "; ack3writeinfo4zpu-set-by-manoeuvre3writeimage4zpu to: " + str(ack3writeinfo4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3writeget4zpu     = '<default4xi>default3write4zpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message3writeget4zpu-set-by-manoeuvre3writeimage4zpu to: " + str(message3writeget4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3writeput4zpu     = '<default4xi>default3write4zpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message3writeput4zpu-set-by-manoeuvre3writeimage4zpu to: " + str(message3writeput4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3writereceive4zpu = '<default4xi>default3write4zpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message3writereceive4zpu-set-by-manoeuvre3writeimage4zpu to: " + str(message3writereceive4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3writesend4zpu    = '<default4xi>default3write4zpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message3writesend4zpu-set-by-manoeuvre3writeimage4zpu to: " + str(message3writesend4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3writestate4zpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max3writestate4zpu-set-by-manoeuvre3writeimage4zpu to: " + str( max3writestate4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3write4zpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre3writeimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3writereceive4zpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3writereceive4zpu = consumerchannel3writemanoeuvre4zpu.recv()
            message2print = "{(Step 1B/33) received" + "; message3writereceive4zpu-received-by-manoeuvre3writeimage4zpu: " + str(message3writereceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3writereceive4zpu == '<copy4xi>copy3write4zpu</copy4xi>':
                message3writesend4zpu = '<roger4xi>roger3write4zpu</roger4xi>'
            else:
                message3writesend4zpu = '<negative4xi><negative3write4zpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3writemanoeuvre4zpu.send(message3writesend4zpu)
            
            message2print = "{(Step 1C/33) sent" + "; message3writesend4zpu-sent-by-manoeuvre3writeimage4zpu: " + str(message3writesend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message3writereceive4zpu-failed-by-manoeuvre3writeimage4zpu: " + str(message3writereceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3write4zpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre3writeimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3writereceive4zpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3writereceive4zpu = consumerchannel3writemanoeuvre4zpu.recv()
            message2print = "{(Step 2B/33) received" + "; message3writereceive4zpu-received-by-manoeuvre3writeimage4zpu: " + str(message3writereceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3writereceive4zpu == '<ready4xi>ready3write4zpu</ready4xi>':
                message3writesend4zpu = '<ready4xi>ready3write4zpu</ready4xi>'
            else:
                message3writesend4zpu = '<negative4xi><negative3ready4zpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3writemanoeuvre4zpu.send(message3writesend4zpu)
            
            message2print = "{(Step 2C/33) sent" + "; message3writesend4zpu-sent-by-manoeuvre3writeimage4zpu: " + str(message3writesend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message3writereceive4zpu-failed-by-manoeuvre3writeimage4zpu: " + str(message3writereceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state3write4zpu == 3:    
        info3image4zpu = None
        message2print = "{(Step 3A/70) try" + "; info3image4zpu-try-by-manoeuvre3writeimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info3image4zpu = queue3writemanoeuvre4zpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info3image4zpu-got-by-manoeuvre3writeimage4zpu: " + str(info3image4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info3image4zpu is None:
                code3nack  = code3nack + '3Z1-'
                ack3writeinfo4zpu = False 
            else:    
                code3ack  = code3ack + '3C-'
                ack3writeinfo4zpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue3writemanoeuvre4zpu-failed-by-manoeuvre3writeimage4zpu: " + str(queue3writemanoeuvre4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z1-'
            
        message3writesend4zpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack3writeinfo4zpu is True:
                message3writesend4zpu = '<ack4xi>ack3imageinfo4zpu</ack4xi>'
            else:
                message3writesend4zpu = '<nack4xi>nack3imageinfo4zpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message3writesend4zpu-set-by-manoeuvre3writeimage4zpu to: " + str(message3writesend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #   
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3writesend4zpu-failed-by-manoeuvre3writeimage4zpu: " + str(message3writesend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code3nack  = code3nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel3writemanoeuvre4zpu-calling-by-manoeuvre3writeimage4zpu: " + str(publisherchannel3writemanoeuvre4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message3writesend4zpu-passing-to-send-by-manoeuvre3writeimage4zpu: " + str(message3writesend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3writesend4zpu-failed-by-manoeuvre3writeimage4zpu: " + str(message3writesend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z2-'                
        try:    
            publisherchannel3writemanoeuvre4zpu.send(message3writesend4zpu)
            
            message2print = "{(Step 4E/33) sent" + "; message3writesend4zpu-sent-by-manoeuvre3writeimage4zpu: " + str(message3writesend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #         
            code3ack  = code3ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message3writesend4zpu-failed-by-manoeuvre3writeimage4zpu: " + str(message3writesend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response3manouvre4write['ACK3INFO4ZPU']        = ack3writeinfo4zpu
    response3manouvre4write['CODE3ACK4ZPU']        = code3ack
    response3manouvre4write['CODE3NACK4ZPU']       = code3nack
    response3manouvre4write['MESSAGE3GET4ZPU']     = message3writeget4zpu
    response3manouvre4write['MESSAGE3PUT4ZPU']     = message3writeput4zpu
    response3manouvre4write['MESSAGE3RECEIVE4ZPU'] = message3writereceive4zpu
    response3manouvre4write['MESSAGE3SEND4ZPU']    = message3writesend4zpu
    response3manouvre4write['STATE3MANOEUVRE4ZPU'] = state3write4zpu
                          
    message2print = "{(Step 33/33) returning" + "; response3manouvre4write-returning-by-manoeuvre3writeimage4zpu: " + str(response3manouvre4write) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1
    # 
    
    #2
    # 
    
    #3
    # 
    

    return response3manouvre4write           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre3mapimage4zpu" usage = "control function for map3image4zpu function in this module">
# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre3mapimage4zpu(queue3mapmanoeuvre4zpu, consumerchannel3mapmanoeuvre4zpu, publisherchannel3mapmanoeuvre4zpu):
    '''
    manoeuvres control and command
    '''
    global state3map4zpu                                                       # STATE FOR COMMAND AND CONTROL
    global max3mapstate4zpu                                                    # MAX FOR STATE
    global ack3mapinfo4zpu                                                     # ACK FOR IMAGE INFO
    global message3mapget4zpu                                 
    global message3mapput4zpu
    global message3mapreceive4zpu
    global message3mapsend4zpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre3mapimage4zpu" 
    method2print = "manoeuvre3mapimage4zpu" 

    code3ack  = ' '
    code3nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre3mapimage4zpu-started: " + str(manoeuvre3mapimage4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue3mapmanoeuvre4zpu-to-manoeuvre3mapimage4zpu: " + str(queue3mapmanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel3mapmanoeuvre4zpu-to-manoeuvre3mapimage4zpu: " + str(consumerchannel3mapmanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel3mapmanoeuvre4zpu-to-manoeuvre3mapimage4zpu: " + str(publisherchannel3mapmanoeuvre4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code3ack  = code3ack + '0D-' 
    response3manouvre4map = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state3map4zpu-try-by-manoeuvre3mapimage4zpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state3map4zpu is None:
            state3map4zpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state3map4zpu-initialized-by-manoeuvre3mapimage4zpu to: " + str(state3map4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state3map4zpu-used-by-manoeuvre3mapimage4zpu: " + str(state3map4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
    except:
        state3map4zpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state3map4zpu-initialized-by-manoeuvre3mapimage4zpu to: " + str(state3map4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3mapinfo4zpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack3mapinfo4zpu-initialized-by-manoeuvre3mapimage4zpu to: " + str(ack3mapinfo4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3mapget4zpu     = '<default4xi>default3map4zpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message3mapget4zpu-initialized-by-manoeuvre3mapimage4zpu to: " + str(message3mapget4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3mapput4zpu     = '<default4xi>default3map4zpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message3mapput4zpu-initialized-by-manoeuvre3mapimage4zpu to: " + str(message3mapput4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3mapreceive4zpu = '<default4xi>default3map4zpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message3mapreceive4zpu-initialized-by-manoeuvre3mapimage4zpu to: " + str(message3mapreceive4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3mapsend4zpu    = '<default4xi>default3map4zpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message3mapsend4zpu-initialized-by-manoeuvre3mapimage4zpu to: " + str(message3mapsend4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3mapstate4zpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max3mapstate4zpu-initialized-by-manoeuvre3mapimage4zpu to: " + str( max3mapstate4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state3map4zpu = state3map4zpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state3map4zpu-added-by-manoeuvre3mapimage4zpu: " + str(state3map4zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    if state3map4zpu > max3mapstate4zpu:
        state3map4zpu = 0
        message2print = "{(Step 0I/33) set" + "; state3map4zpu-set-by-manoeuvre3mapimage4zpu to: " + str(state3map4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3mapinfo4zpu        = False
        message2print = "{(Step 0J/33) set" + "; ack3mapinfo4zpu-set-by-manoeuvre3mapimage4zpu to: " + str(ack3mapinfo4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3mapget4zpu     = '<default4xi>default3map4zpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message3mapget4zpu-set-by-manoeuvre3mapimage4zpu to: " + str(message3mapget4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3mapput4zpu     = '<default4xi>default3map4zpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message3mapput4zpu-set-by-manoeuvre3mapimage4zpu to: " + str(message3mapput4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3mapreceive4zpu = '<default4xi>default3map4zpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message3mapreceive4zpu-set-by-manoeuvre3mapimage4zpu to: " + str(message3mapreceive4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3mapsend4zpu    = '<default4xi>default3map4zpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message3mapsend4zpu-set-by-manoeuvre3mapimage4zpu to: " + str(message3mapsend4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3mapstate4zpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max3mapstate4zpu-set-by-manoeuvre3mapimage4zpu to: " + str( max3mapstate4zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3map4zpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre3mapimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3mapreceive4zpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3mapreceive4zpu = consumerchannel3mapmanoeuvre4zpu.recv()
            message2print = "{(Step 1B/33) received" + "; message3mapreceive4zpu-received-by-manoeuvre3mapimage4zpu: " + str(message3mapreceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3mapreceive4zpu == '<copy4xi>copy3mapimage4zpu</copy4xi>':
                message3mapsend4zpu = '<roger4xi>roger3mapimage4zpu</roger4xi>'
            else:
                message3mapsend4zpu = '<negative4xi><negative3mapimage4zpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3mapmanoeuvre4zpu.send(message3mapsend4zpu)
            
            message2print = "{(Step 1C/33) sent" + "; message3mapsend4zpu-sent-by-manoeuvre3mapimage4zpu: " + str(message3mapsend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message3mapreceive4zpu-failed-by-manoeuvre3mapimage4zpu: " + str(message3mapreceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3map4zpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre3mapimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3mapreceive4zpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3mapreceive4zpu = consumerchannel3mapmanoeuvre4zpu.recv()
            message2print = "{(Step 2B/33) received" + "; message3mapreceive4zpu-received-by-manoeuvre3mapimage4zpu: " + str(message3mapreceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3mapreceive4zpu == '<ready4xi>ready3mapimage4zpu</ready4xi>':
                message3mapsend4zpu = '<ready4xi>ready3mapimage4zpu</ready4xi>'
            else:
                message3mapsend4zpu = '<negative4xi><negative3ready4zpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3mapmanoeuvre4zpu.send(message3mapsend4zpu)
            
            message2print = "{(Step 2C/33) sent" + "; message3mapsend4zpu-sent-by-manoeuvre3mapimage4zpu: " + str(message3mapsend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message3mapreceive4zpu-failed-by-manoeuvre3mapimage4zpu: " + str(message3mapreceive4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state3map4zpu == 3:    
        info3image4zpu = None
        message2print = "{(Step 3A/70) try" + "; info3image4zpu-try-by-manoeuvre3mapimage4zpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info3image4zpu = queue3mapmanoeuvre4zpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info3image4zpu-got-by-manoeuvre3mapimage4zpu: " + str(info3image4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info3image4zpu is None:
                code3nack  = code3nack + '3Z1-'
                ack3mapinfo4zpu = False 
            else:    
                code3ack  = code3ack + '3C-'
                ack3mapinfo4zpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue3mapmanoeuvre4zpu-failed-by-manoeuvre3mapimage4zpu: " + str(queue3mapmanoeuvre4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z1-'
            
        message3mapsend4zpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack3mapinfo4zpu is True:
                message3mapsend4zpu = '<ack4xi>ack3imageinfo4zpu</ack4xi>'
            else:
                message3mapsend4zpu = '<nack4xi>nack3imageinfo4zpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message3mapsend4zpu-set-by-manoeuvre3mapimage4zpu to: " + str(message3mapsend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3mapsend4zpu-failed-by-manoeuvre3mapimage4zpu: " + str(message3mapsend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code3nack  = code3nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel3mapmanoeuvre4zpu-calling-by-manoeuvre3mapimage4zpu: " + str(publisherchannel3mapmanoeuvre4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message3mapsend4zpu-passing-to-send-by-manoeuvre3mapimage4zpu: " + str(message3mapsend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3mapsend4zpu-failed-by-manoeuvre3mapimage4zpu: " + str(message3mapsend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z2-'                
        try:    
            publisherchannel3mapmanoeuvre4zpu.send(message3mapsend4zpu)
            
            message2print = "{(Step 4E/33) sent" + "; message3mapsend4zpu-sent-by-manoeuvre3mapimage4zpu: " + str(message3mapsend4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #          
            code3ack  = code3ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message3mapsend4zpu-failed-by-manoeuvre3mapimage4zpu: " + str(message3mapsend4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response3manouvre4map['ACK3INFO4ZPU']        = ack3mapinfo4zpu
    response3manouvre4map['CODE3ACK4ZPU']        = code3ack
    response3manouvre4map['CODE3NACK4ZPU']       = code3nack
    response3manouvre4map['MESSAGE3GET4ZPU']     = message3mapget4zpu
    response3manouvre4map['MESSAGE3PUT4ZPU']     = message3mapput4zpu
    response3manouvre4map['MESSAGE3RECEIVE4ZPU'] = message3mapreceive4zpu
    response3manouvre4map['MESSAGE3SEND4ZPU']    = message3mapsend4zpu
    response3manouvre4map['STATE3MANOEUVRE4ZPU'] = state3map4zpu
                          
    message2print = "{(Step 33/33) returning" + "; response3manouvre4map-returning-by-manoeuvre3mapimage4zpu: " + str(response3manouvre4map) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 
    
    #2 
    
    #3 
    
    #4 
    
    return response3manouvre4map           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "write3image4zpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ3IMAGE4ZPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ3IMAGE4ZPU ', time4xi())    
#   respon4write3image = {} 
#   task4write3image = None
#   try:                               
#       response4write3image = write3image4zpu(task4write3image)                                                                                                          
#       print('(Step 16B/700) OF ZPU3IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ3IMAGE4ZPU : ', response4write3image)
#       queue3write4zpu      = response4write3image['QUEUE']
#       channel3publish4zpu = response4write3image['PUBLISHERCHANNEL']
#       channel3consume4zpu = response4write3image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ3IMAGE4ZPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE3IMAGE4ZPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4ZPU ', time4xi())     
#   try:                               
#       message3writesend4zpu = "<copy4xi>copy3write4zpu</copy4xi>"
#       channel3publish4zpu = response4write3image['PUBLISHERCHANNEL']
#       channel3publish4zpu.send(message3writesend4zpu)
#       print("(Step 17B/700) OF ZPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4ZPU: ", channel3publish4zpu, " MESSAGE3SEND4ZPU: " , message3writesend4zpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4ZPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE3IMAGE4ZPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvre4zpu = manoeuvre3readimage4zpu(queue3write4zpu, channel3consume4zpu, channel3publish4zpu)
#       print("(Step 18D/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", manoeuvre3readimage4zpu)
#   except:
#       print("(Step 18Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4ZPU ', time4xi())     
#   try:
#       channel3consume4zpu = response4write3image['CONSUMERCHANNEL']                              
#       message3writereceive4zpu = channel3consume4zpu.recv()
#       print("(Step 19B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3consume4zpu, " CONTROL & COMMAND: ", message3writereceive4zpu)                                                                                                  
#       # <roger4xi>roger3write4zpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ3IMAGE4ZPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE3IMAGE4ZPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4ZPU ', time4xi())     
#   try:                               
#       message3writesend4zpu = "<write4xi>write3write4zpu</write4xi>"
#       channel3publish4zpu = response4write3image['PUBLISHERCHANNEL']
#       channel3publish4zpu.send(message3writesend4zpu)
#       print("(Step 20B/700) OF ZPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4ZPU: ", channel3publish4zpu, " MESSAGE3SEND4ZPU: " , message3writesend4zpu)                                                                                                
#       # <write4xi>write3write4zpu</write4xi> 
#   except:
#        print('(Step 20Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4ZPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE3IMAGE4ZPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvre4zpu = manoeuvre3readimage4zpu(queue3write4zpu, channel3consume4zpu, channel3publish4zpu)
#       print("(Step 21B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", manoeuvre3readimage4zpu)
#   except:
#       print("(Step 21Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4ZPU ', time4xi())     
#   try:
#       channel3consume4zpu = response4write3image['CONSUMERCHANNEL']                              
#       message3writereceive4zpu = channel3consume4zpu.recv()
#       print("(Step 22B/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3consume4zpu, " CONTROL & COMMAND: ", message3writereceive4zpu)                                                                                                  
#       # <write4xi>write3write4zpu</write4xi> 
#   except:
#       print('(Step 22Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ3IMAGE4ZPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue3write4zpu.put(path3zpu)             
#       print("(Step 24B/700) OF ZPU3IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path3zpu) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE3IMAGE4ZPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvre4zpu = manoeuvre3readimage4zpu(queue3write4zpu, channel3consume4zpu, channel3publish4zpu)
#       print("(Step 25B/700) OF ZPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", manoeuvre3readimage4zpu)
#   except:
#       print("(Step 25Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message3writereceive4zpu = channel3consume4zpu.recv()
#       print("(Step 26D/700) OF ZPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4ZPU: ", channel3consume4zpu, " CONTROL & COMMAND: ", message3writereceive4zpu)                                                                                                   
#       # <ack4xi>ack3imageinfo4zpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE3IMAGE4ZPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi()) 
#   try:        
#       response3manoeuvre4zpu = manoeuvre3readimage4zpu(queue3write4zpu, channel3consume4zpu, channel3publish4zpu)
#       print("(Step 27B/700) OF ZPU3IMAGE4XI5TEST - GOT RESPONSE3MANOEUVRE4ZPU ", response3manoeuvre4zpu)
#   except:
#       print("(Step 27Z/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4ZPU ", time4xi())   

#">
# -------------------------------------------------------------------------------------------------   
def write3image4zpu(task4imagewrite3zpu=None):

    '''
    writes 3D images for ZPU
    '''
    global zpu3image4xi                                                     # 3D-IMAGES FOR ZPU  
    global control3writeimage4zpu                                              # COMMAND AND CONTROL FOR ZPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS        
    # -----------------------------------------------------------------------------------------          
    class2print = "write3image4zpu"
    method2print = "write3image4zpu"

    message2print = "{(Step 1A/33) started" + "; write3image4zpu-started: " + str(write3image4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function write3image4zpu at 0x000000000321ACC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4imagewrite3zpu-to-write3image4zpu: " + str(task4imagewrite3zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 
            
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE ZPU3IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if zpu3image4xi is None:
            zpu3image4xi = create3image4zpu()                                                                              # INITIALIZE IMAGE FOR ZPU 
            message2print = "{(Step 1C/70) created" + "; zpu3image4xi-created-by-write3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create3image4zpu-skipped-by-write3image4zpu: " + str(write3image4zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function write3image4zpu at 0x000000000321ACC8>                            
    except:
        try:
            zpu3image4xi = create3image4zpu()
            message2print = "{(Step 1E/70) set" + "; zpu3image4xi-set-by-write3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            zpu3image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; zpu3image4xi-yielded-by-write3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagewrite3zpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagewrite3zpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagewrite3zpu-created-by-write3image4zpu to: " + str(task4imagewrite3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4zpu = trace3image4zpu                                                                 # CALLBACK FUNCTION FOR ZPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function3callback4zpu-created-by-write3image4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            
            code3ack  = code3ack + '1B1-'
           
            args3callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "write3image4zpu"                                                            # MODULE OF ZPUIZED OPENCL MODULE
            args3callback['METHOD']  = "write3image4zpu"                                                            # FUNCTION OF ZPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write3image4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args3callback-created-by-write3image4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # <function trace3image4zpu at 0x0000000003214848> 
            
            code3ack  = code3ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite3zpu['DATA3CALLBACK'] = args3callback                                                    # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagewrite3zpu-created-by-write3image4zpu to: " + str(task4imagewrite3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'DATA3CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; write3image4zpu-failed ; time: 07:14:55 10/10/11 Pacific Daylight Time}',
            #  'CLASS': 'write3image4zpu',
            #  'METHOD': 'write3image4zpu'},
            #  'FUNCTION3CALLBACK': <function trace3image4zpu at 0x0000000003214848>}; time: 07:14:55 10/10/11 Pacific Daylight Time}

            
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-write3image4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-write3image4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 07:14:55 10/10/11 Pacific Daylight Time}

                code3ack  = code3ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-write3image4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '2Z-'
                 
            task4imagewrite3zpu['PLATFORM'] = platform3xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3zpu = {}
            message2print = "{(Step 3A/70) trying" + "; context3zpu-trying-by-write3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3zpu = get2context4zpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context3zpu-got-by-write3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 07:14:55 10/10/11 Pacific Daylight Time}', ...
                
                code3ack  = code3ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context3zpu-failed-by-write3image4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '3Z-'
                
            task4imagewrite3zpu ['CONTEXT'] = context3zpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3zpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device3zpu-trying-by-write3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3zpu = get2zpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device3zpu-got-by-write3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR' ...
                
                code3ack  = code3ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device3zpu-failed-by-write3image4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '4Z-'
            
            task4imagewrite3zpu ['DEVICE'] = device3zpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo3zpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo3zpu-trying-by-write3image4zpu: " + str(imageinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo3zpu = get3imageinfo4zpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo3zpu-got-by-write3image4zpu: " + str(imageinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #  {'SIZE2QUEUE': 2147483646, 
                #   'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #   'ORDER': 'CL_RGBA'}, 
                #   'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004406E48>, 
                #   'WIDTH': 1100, 
                #   'DEPTH': 900, 
                #   'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 07:14:55 10/10/11 Pacific Day ...
                
                code3ack  = code3ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo3zpu-failed-by-write3image4zpu: " + str(imageinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '5Z-'          
                      
            task4imagewrite3zpu ['IMAGE'] = imageinfo3zpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY ZPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3zpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue3zpu-trying-by-write3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3zpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue3zpu-got-by-write3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004777978> 
                
                code3ack  = code3ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue3zpu-failed-by-write3image4zpu: " + str(queue3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '6Z-'
                
            task4imagewrite3zpu ['QUEUE'] = queue3zpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY ZPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3zpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe3zpu-trying-by-write3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3zpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe3zpu-got-by-write3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 636>, <read-write PipeConnection, handle 640>) 
                
                code3ack  = code3ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe3zpu-failed-by-write3image4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '7Z-'
                
            task4imagewrite3zpu ['PIPE'] = pipe3zpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3zpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo3zpu-trying-by-write3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo3zpu-got-by-write3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4zpu-failed ; time: 07:14:55 10/10/11 Pa ...
                
                code3ack  = code3ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo3zpu-failed-by-write3image4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '8Z-'
        
            task4imagewrite3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY ZPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3zpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo3zpu-trying-by-write3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3zpu = get2bufferinfo4zpu()                                                               # SYSTEM CREATES BUFFER FOR ZPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo3zpu-got-by-write3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4zpu-failed ; time: 07:14:55 10/10/11 Pacific  ...
                
                code3ack  = code3ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo3zpu-failed-by-write3image4zpu: " + str(bufferinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '9Z-'
        
            task4imagewrite3zpu ['BUFFER'] = bufferinfo3zpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3zpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo3zpu-trying-by-write3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3zpu = get2eventinfo4zpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo3zpu-got-by-write3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4zpu-failed ; time: 07:14:55 10/10/11 Pacific Daylight Time}', 'CLASS ...
                
                code3ack  = code3ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo3zpu-failed-by-write3image4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '10Z-'
        
            task4imagewrite3zpu ['EVENT'] = eventinfo3zpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3zpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist3zpu-trying-by-write3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3zpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist3zpu-set-by-write3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                code3ack  = code3ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist3zpu-failed-by-write3image4zpu: " + str(eventwaitlist3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '11Z-'
        
            task4imagewrite3zpu ['EVENTWAITLIST'] = eventwaitlist3zpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3zpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity3zpu-trying-by-write3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3zpu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3zpu = eventwaitlistquantity3zpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity3zpu-set-by-write3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code3ack  = code3ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity3zpu-failed-by-write3image4zpu: " + str(eventwaitlistquantity3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '12Z-'
        
            task4imagewrite3zpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3zpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking3zpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking3zpu-trying-by-write3image4zpu: " + str(blocking3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking3zpu = 'CL_FALSE'                                                                           # the write command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking3zpu-set-by-write3image4zpu to: " + str(blocking3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE 
                
                code3ack  = code3ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking3zpu-failed-by-write3image4zpu: " + str(blocking3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '13Z-'
        
            task4imagewrite3zpu ['BLOCKING'] = blocking3zpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin3zpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin3zpu-trying-by-write3image4zpu: " + str(origin3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin3zpu['X'] = 0 
                origin3zpu['Y'] = 0  
                origin3zpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin3zpu-set-by-write3image4zpu to: " + str(origin3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0}
                
                code3ack  = code3ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin3zpu-failed-by-write3image4zpu: " + str(origin3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '14Z-'
        
            task4imagewrite3zpu ['ORIGIN'] = origin3zpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3zpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region3zpu-trying-by-write3image4zpu: " + str(region3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region3zpu['X'] = 330                                                                               # WIDTH  IN PIXELS TO READ 
                region3zpu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO READ 
                region3zpu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region3zpu-set-by-write3image4zpu to: " + str(region3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 330, 'X': 330, 'Z': 330}
                
                code3ack  = code3ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region3zpu-failed-by-write3image4zpu: " + str(region3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '15Z-'
        
            task4imagewrite3zpu ['REGION'] = region3zpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.

            height3zpu = 800
            width3zpu = 800
            rowpitch3zpu = 0
            size3element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 1                                                                                  # ONE DIRECTIONS IN 3D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch3zpu-trying-by-write3image4zpu: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #  
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3zpu = int(width3zpu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch3zpu-set-by-write3image4zpu to: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                           
                 
                code3ack  = code3ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch3zpu-failed-by-write3image4zpu: " + str(rowpitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '16Z-'
        
            task4imagewrite3zpu ['ROWPITCH'] = rowpitch3zpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer3zpu = None
            size3zpu = int(width3zpu * height3zpu)                                                                  # CALCULATED MEMORY
            if size3zpu == 0:
                size3zpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer3zpu-trying-by-write3image4zpu: " + str(pointer3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena3zpu = Arena4Xi(size3zpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena3zpu-got-by-write3image4zpu: " + str(arena3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041F4C50>
                
                size3arena = arena3zpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size3arena-got-by-write3buffer4zpu: " + str(size3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer3arena = arena3zpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer3arena-got-by-write3buffer4zpu: " + str(buffer3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x00000000042554A8> 
                
                code3ack  = code3ack + '17D-' 
                
                if size3arena < size3zpu:
                    size3zpu = size3arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size3zpu-set-by-write3buffer4zpu to: " + str(size3zpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code3ack  = code3ack + '17E-'
                else:
                    pass
                    
                pointer3zpu = buffer3arena
                message2print = "{(Step 17F/70) set" + "; pointer3zpu-set-by-write3buffer4zpu to: " + str(pointer3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x00000000042554A8> 
                
                code3ack  = code3ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer3zpu-failed-by-write3image4zpu: " + str(pointer3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '17Z-'
        
            task4imagewrite3zpu ['POINTER'] = pointer3zpu                            
            task4imagewrite3zpu ['SIZE']    = size3zpu                                                               # AUTOMATIC BY ZPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 3D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch3zpu = 0                                                                                   # SLICEPITCH IS ZERO FOR 3D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch3zpu-trying-by-write3image4zpu: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch3zpu = int(rowpitch3zpu * height3zpu)                                                       
                message2print = "{(Step 18B/70) set" + "; slicepitch3zpu-set-by-write3image4zpu to: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 640000 

                code3ack  = code3ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch3zpu-failed-by-write3image4zpu: " + str(slicepitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '18Z-'
            
            task4imagewrite3zpu ['SLICEPITCH'] = slicepitch3zpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imagewrite3zpu.keys()
        message2print = "{(Step 20/70) using" + "; keys3task-using-by-write3image4zpu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA3CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION3CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 07:14:55 10/10/11 Pacific Daylight Time}

        if (('DATA3CALLBACK' in keys3task) and ('FUNCTION3CALLBACK' in keys3task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4zpu = trace3image4zpu                                                             # CALLBACK FUNCTION FOR ZPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function3callback4zpu-created-by-write3image4zpu to: " + str(function3callback4zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code3ack  = code3ack + '21B1-'
           
            args3callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "write3image4zpu"                                                        # MODULE OF ZPUIZED OPENCL MODULE
            args3callback['METHOD']  = "write3image4zpu"                                                        # FUNCTION OF ZPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write3image4zpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR ZPUIZED OPENCL TO RETURN    
            message2print = "{(Step 21B2/70) initialized" + "; args3callback-created-by-write3image4zpu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code3ack  = code3ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite3zpu['DATA3CALLBACK'] = args3callback                                                # USER DATA PASSED TO ZPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite3zpu['FUNCTION3CALLBACK'] = function3callback4zpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagewrite3zpu-created-by-write3image4zpu to: " + str(task4imagewrite3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys3task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform3xi-created-by-write3image4zpu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform3xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform3xi-got-by-write3image4zpu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                     
                code3ack  = code3ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform3xi-failed-by-write3image4zpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '22Z-'
                     
            task4imagewrite3zpu['PLATFORM'] = platform3xi                                                       # PLATFORM OF SYSTEM 
              
        if 'CONTEXT' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3zpu = {}
            message2print = "{(Step 23A/70) trying" + "; context3zpu-trying-by-write3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3zpu = get2context4zpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context3zpu-got-by-write3image4zpu: " + str(context3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context3zpu-failed-by-write3image4zpu: " + str(context3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '23Z-'
                
            task4imagewrite3zpu ['CONTEXT'] = context3zpu           
        
        if 'DEVICE' in keys3task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3zpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device3zpu-trying-by-write3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3zpu = get2zpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device3zpu-got-by-write3image4zpu: " + str(device3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device3zpu-failed-by-write3image4zpu: " + str(device3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '24Z-'
            
            task4imagewrite3zpu ['DEVICE'] = device3zpu
        
        if 'IMAGE' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo3zpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo3zpu-trying-by-write3image4zpu: " + str(imageinfo3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo3zpu = get3imageinfo4zpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo3zpu-got-by-write3image4zpu: " + str(imageinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo3zpu-failed-by-write3image4zpu: " + str(imageinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '25Z-'          
                      
            task4imagewrite3zpu ['IMAGE'] = imageinfo3zpu         
                    
        if 'QUEUE' in keys3task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY ZPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3zpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue3zpu-trying-by-write3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3zpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue3zpu-got-by-write3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue3zpu-failed-by-write3image4zpu: " + str(queue3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '26Z-'
                
            task4imagewrite3zpu ['QUEUE'] = queue3zpu

        if 'PIPE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY ZPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3zpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe3zpu-trying-by-write3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3zpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe3zpu-got-by-write3image4zpu: " + str(pipe3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe3zpu-failed-by-write3image4zpu: " + str(pipe3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '27Z-'
                
            task4imagewrite3zpu ['PIPE'] = pipe3zpu 
            
        if 'COMMANDQUEUE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3zpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo3zpu-trying-by-write3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3zpu = get2commandqueueinfo4zpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo3zpu-got-by-write3image4zpu: " + str(commandqueueinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo3zpu-failed-by-write3image4zpu: " + str(commandqueueinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '28Z-'
        
            task4imagewrite3zpu ['COMMANDQUEUE'] = commandqueueinfo3zpu
            
            
        if 'BUFFER' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY ZPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3zpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo3zpu-trying-by-write3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3zpu = get2bufferinfo4zpu()                                                           # SYSTEM CREATES BUFFER FOR ZPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo3zpu-got-by-write3image4zpu: " + str(bufferinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo3zpu-failed-by-write3image4zpu: " + str(bufferinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '29Z-'
        
            task4imagewrite3zpu ['BUFFER'] = bufferinfo3zpu
             
        if 'EVENT' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3zpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo3zpu-trying-by-write3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3zpu = get2eventinfo4zpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo3zpu-got-by-write3image4zpu: " + str(eventinfo3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo3zpu-failed-by-write3image4zpu: " + str(eventinfo3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '30Z-'
        
            task4imagewrite3zpu ['EVENT'] = eventinfo3zpu 
        
             
        if 'EVENTWAITLIST' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3zpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist3zpu-trying-by-write3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3zpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist3zpu-set-by-write3image4zpu: " + str(eventwaitlist3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist3zpu-failed-by-write3image4zpu: " + str(eventwaitlist3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '31Z-'
        
            task4imagewrite3zpu ['EVENTWAITLIST'] = eventwaitlist3zpu                     

                 
        if 'EVENTWAITLISTQUANTITY' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3zpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity3zpu-trying-by-write3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3zpu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3zpu = eventwaitlistquantity3zpu + 1                                                                     
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity3zpu-set-by-write3image4zpu: " + str(eventwaitlistquantity3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity3zpu-failed-by-write3image4zpu: " + str(eventwaitlistquantity3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '32Z-'
        
            task4imagewrite3zpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3zpu       
                                                 
                 
        if 'BLOCKING' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking3zpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking3zpu-trying-by-write3image4zpu: " + str(blocking3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking3zpu = 'CL_FALSE'                                                                       # the write command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking3zpu-set-by-write3image4zpu to: " + str(blocking3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking3zpu-failed-by-write3image4zpu: " + str(blocking3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '33Z-'
        
            task4imagewrite3zpu ['BLOCKING'] = blocking3zpu             
                 
        if 'ORIGIN' in keys3task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin3zpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin3zpu-trying-by-write3image4zpu: " + str(origin3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin3zpu['X'] = 0 
                origin3zpu['Y'] = 0  
                origin3zpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin3zpu-set-by-write3image4zpu to: " + str(origin3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin3zpu-failed-by-write3image4zpu: " + str(origin3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '34Z-'
        
            task4imagewrite3zpu ['ORIGIN'] = origin3zpu
                
                 
        if 'REGION' in keys3task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3zpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region3zpu-trying-by-write3image4zpu: " + str(region3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region3zpu['X'] = 330                                                                               # WIDTH  IN PIXELS TO WRITE 
                region3zpu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO WRITE
                region3zpu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region3zpu-set-by-write3image4zpu to: " + str(region3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region3zpu-failed-by-write3image4zpu: " + str(region3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '35Z-'
        
            task4imagewrite3zpu ['REGION'] = region3zpu
            
                
                 
        if 'ROWPITCH' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch3zpu = 0
            size3element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 1                                                                                  # ONE DIRECTIONS IN 3D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch3zpu-trying-by-write3image4zpu: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3zpu = int(width3zpu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch3zpu-set-by-write3image4zpu to: " + str(rowpitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code3ack  = code3ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch3zpu-failed-by-write3image4zpu: " + str(rowpitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '36Z-'
        
            task4imagewrite3zpu ['ROWPITCH'] = rowpitch3zpu     
                 
        if 'POINTER' in keys3task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer3zpu = None
            size3zpu = int(width3zpu * height3zpu)                                                                  # CALCULATED MEMORY
            if size3zpu == 0:
                size3zpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer3zpu-trying-by-write3image4zpu: " + str(pointer3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena3zpu = Arena4Xi(size3zpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena3zpu-got-by-write3image4zpu: " + str(arena3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size3arena = arena3zpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size3arena-got-by-write3buffer4zpu: " + str(size3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer3arena = arena3zpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer3arena-got-by-write3buffer4zpu: " + str(buffer3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code3ack  = code3ack + '37D-' 
                
                if size3arena < size3zpu:
                    size3zpu = size3arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size3zpu-set-by-write3buffer4zpu to: " + str(size3zpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code3ack  = code3ack + '37E-'
                else:
                    pass
                    
                pointer3zpu = buffer3arena
                message2print = "{(Step 37F/70) set" + "; pointer3zpu-set-by-write3buffer4zpu to: " + str(pointer3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code3ack  = code3ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer3zpu-failed-by-write3image4zpu: " + str(pointer3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '37Z-'
        
            task4imagewrite3zpu ['POINTER'] = pointer3zpu                            
            task4imagewrite3zpu ['SIZE']    = size3zpu                                                               # AUTOMATIC BY ZPUXI
        
             
        if 'SLICEPITCH' in keys3task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 3D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch3zpu = 0                                                                                   # SLICEPITCH IS ZERO FOR 3D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch3zpu-trying-by-write3image4zpu: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch3zpu = int(rowpitch3zpu * height3zpu)                                                      
                message2print = "{(Step 38B/70) set" + "; slicepitch3zpu-set-by-write3image4zpu to: " + str(slicepitch3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch3zpu-failed-by-write3image4zpu: " + str(slicepitch3zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '38Z-'
            
            task4imagewrite3zpu ['SLICEPITCH'] = slicepitch3zpu                  

        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imagewrite3zpu.keys()
        message2print = "{(Step 39/70) set" + "; keys3task-set-by-write3image4zpu to: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA3CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION3CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 07:14:55 10/10/11 Pacific Daylight Time}

        
             
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagewrite3zpu-yielded-by-write3image4zpu: " + str(task4imagewrite3zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control3writeimage4zpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels3zpu = False
    channels3zpu = ()
    message2print = "{(Step 40A/70) trying" + "channels3zpu-trying-by-write3image4zpu: " + str(channels3zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel3zpu, publisherchannel3zpu = task4imagewrite3zpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel3zpu-got-by-write3image4zpu: " + str(consumerchannel3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 636> 

        message2print = "{(Step 40C/70) got" + "; publisherchannel3zpu-got-by-write3image4zpu: " + str(publisherchannel3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 640>
                     
        ack4channels3zpu = True
        
        control3writeimage4zpu['CONSUMERCHANNEL']  = consumerchannel3zpu
        
        
        control3writeimage4zpu['PUBLISHERCHANNEL'] = publisherchannel3zpu
        
        code3ack  = code3ack + '40D-' 
        
    except:
        ack4channels3zpu = False
        code3nack  = code3nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue3zpu = False
    queue3zpu = None
    message2print = "{(Step 41A/70) trying" + "queue3zpu-trying-by-write3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue3zpu = task4imagewrite3zpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue3zpu-got-by-write3image4zpu: " + str(queue3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004777978> 
        
              
        ack4queue3zpu = True
        control3writeimage4zpu['QUEUE']  = queue3zpu        
        code3ack  = code3ack + '41C-'
    except:
        ack4queue3zpu = False  
        code3nack  = code3nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy3image4zpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy3image4zpu-trying-by-write3image4zpu: " + str(dummy3image4zpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy3image4zpu = DummyProcess4Xi(target=manoeuvre3readimage4zpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue3zpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel3zpu, publisherchannel3zpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy3image4zpu-created-by-write3image4zpu: " + str(dummy3image4zpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code3ack  = code3ack + '69B-'
        
    # except:
        # code3nack  = code3nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code3ack-set-by-write3image4zpu: " + str(code3ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code3nack-set-by-write3image4zpu: " + str(code3nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #   
        
    message2print = "{(Step 70C/70) returning" + "; control3writeimage4zpu-returning-by-write3image4zpu: " + str(control3writeimage4zpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004777978>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 640>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 636>}; time: 07:14:55 10/10/11 Pacific Daylight Time}

    return control3writeimage4zpu
                    
#</function4xi> 


 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release3image4zpu" usage = "
#
#   task4release3image  = {} 
#   task4release3image  ['COUNT3REFERENCE'] = 1 
#   response4release3image = {}
#   
#   print('(Step 12A/33) OF ZPU3IMAGE4XI5TEST - TRYING TO RELEASE IMAGE OF ZPU DEVICE ', time4xi())   
#   try:                               
#       response4release3image = release3image4zpu(task4release3image)                                                                                             
#       print('(Step 12B/33) OF ZPU3IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4release3image)                           
#   except:
#       print('(Step 12Z1/33) OF ZPU3IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF ZPU DEVICE ', time4xi())  
#   
#                 
#">
# -------------------------------------------------------------------------------------------------   
def release3image4zpu(task4release2zpu=None):

    '''
    releases zpu3image4xi
    '''
    global zpu3image4xi  
    global code3ack
    global code3nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release3image4zpu"
    method2print = "release3image4zpu" 

    message2print = "{(Step 1A/33) started" + "; release3image4zpu-started: " + str(release3image4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release3image4zpu at 0x00000000031B7D48> 
    
    message2print = "{(Step 1B/33) passed" + "; task4release2zpu-to-release3image4zpu: " + str(task4release2zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT3REFERENCE': 1}

             
    code3ack  = ' '
    code3nack = ' '

    try:
        if zpu3image4xi is None:
            zpu3image4xi = create3image4zpu()                                                                       # INITIALIZE IMAGE                 
    except:
        try:
            zpu3image4xi = create3image4zpu()
        except:
            zpu3image4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; zpu3image4xi-yielded-by-release3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release3image4zpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2zpu is None:
        task4release2zpu = {}
        task4release2zpu['COUNT3REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        zpu3image4xi['COUNT3REFERENCE'] = zpu3image4xi['COUNT3REFERENCE'] - task4release2zpu['COUNT3REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release3image4zpu: " + str(zpu3image4xi['COUNT3REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if zpu3image4xi['COUNT3REFERENCE'] < 0:
            zpu3image4xi['COUNT3REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release3image4zpu: " + " ** " + str(zpu3image4xi['COUNT3REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # 
              
            code3nack  = code3nack + '1B-'
        else:
            code3ack  = code3ack + '1B-'
    except:
        try:
            zpu3image4xi['COUNT3REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release3image4zpu: " + " ** " + str(zpu3image4xi['COUNT3REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # warning; count2reference-warning-by-release3image4zpu:  ** 0 **
            
            code3nack  = code3nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; zpu3image4xi-yielded-by-release3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    zpu3image4xi ['ACK2RETURN']  = code3ack
    zpu3image4xi ['NACK2RETURN'] = code3nack  
    
    message2print = "{(Step 70/70) returning" + "; zpu3image4xi-returning-by-release3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # ACK2RETURN  :   
    # BUFFER  :  {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4zpu-failed ; time: 06:24:23 10/10/ ...
    # CODE3ERROR  :  0
    # COMMANDQUEUE  :  {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4zpu-failed ; time: 06: ...
    # CONTEXT  :  {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 06:24:23 10/10/11 Pacific Dayl ...
    # COUNT3REFERENCE  :  0
    # DATA3CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create3image4zpu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create3image4zpu', 'METHOD': 'create3image4zpu'}
    # DEPTH  :  900
    # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHL ...
    # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4zpu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time ...
    # FLAGS  :  0
    # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # FUNCTION3CALLBACK  :  <function trace3image4zpu at 0x00000000031B1848>
    # HEIGHT  :  1000
    # NACK2RETURN  :   8Z1-
    # PIPE  :  (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>)
    # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
    # POINTER  :  <mmap.mmap object at 0x00000000041E03F8>
    # QUEUE  :  <queue4xi.Queue4Xi object at 0x00000000041E5EB8>
    # ROWPITCH  :  901
    # SLICEPITCH  :  902
    # WIDTH  :  1100
    
    return zpu3image4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain3image4zpu" usage = "
#
#   task4retain3image  = {}
#   task4retain3image  ['DIMENSION'] = 3 
#   task4retain3image  ['COUNT3REFERENCE'] = 1 
#   response4retain3Dimage = {}
#   print('(Step 13C/700) OF ZPU3IMAGE4XI5TEST - TRYING TO RETAIN 3D IMAGE OF ZPU DEVICE ', time4xi())  
#   try:                               
#       response4retain3Dimage = retain3image4zpu(task4retain3image)                                                                                                 
#       print('(Step 13D/700) OF ZPU3IMAGE4XI5TEST - RESPONSE4RETAIN3DIMAGE : ', response4retain3Dimage)                          
#       # 
#   except:
#       print('(Step 13Z2/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO RETAIN 3D IMAGE OF ZPU DEVICE ', time4xi())      
#
#">
# -------------------------------------------------------------------------------------------------   
def retain3image4zpu(task4retain3zpu=None):

    '''
    retains zpu3image4xi
    '''
    global zpu3image4xi 
     
    global code3ack
    global code3nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain3image4zpu"
    method2print = "retain3image4zpu"

    message2print = "{(Step 1A/33) started" + "; retain3image4zpu-started: " + str(retain3image4zpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain3image4zpu at 0x00000000031B7DC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4retain3zpu-to-retain3image4zpu: " + str(task4retain3zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT3REFERENCE': 1, 'DIMENSION': 3} 
    
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain3zpu is None:
        task4retain3zpu = {}
        task4retain3zpu['DIMENSION'] = 3
        task4retain3zpu['COUNT3REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain3zpu-set-by-retain3image4zpu to: " + str(task4retain3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass
        
    if 'DIMENSION' in task4retain3zpu.keys():
        if task4retain3zpu['DIMENSION'] == 3:
            pass
        else:
            task4retain3zpu['DIMENSION'] = 3 
            message2print = "{(Step 2B/33) set" + "; dimension-set-by-retain3image4zpu to: " + str(task4retain3zpu['DIMENSION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                       
    else:
        task4retain3zpu['DIMENSION'] = 3
    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT3REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT3REFERENCE' in task4retain3zpu.keys():
        pass
    else:
        task4retain3zpu['COUNT3REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain3image4zpu to: " + str(task4retain3zpu['COUNT3REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain3zpu['COUNT3REFERENCE'] < 1:
        task4retain3zpu['COUNT3REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain3image4zpu to: " + str(task4retain3zpu['COUNT3REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE 3D IMAGE IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if zpu3image4xi is None:
            zpu3image4xi = create3image4zpu()
            message2print = "{(Step 4A/70) created" + "; zpu3image4xi-created-by-retain3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            zpu3image4xi = create3image4zpu()
            message2print = "{(Step 4B/70) created" + "; zpu3image4xi-created-by-retain3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            zpu3image4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; zpu3image4xi-yielded-by-retain3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        zpu3image4xi['COUNT3REFERENCE'] = zpu3image4xi['COUNT3REFERENCE'] + task4retain3zpu['COUNT3REFERENCE']
        message2print = "{(Step 6B/70) calculated" + "; count2reference-calculated-by-retain3image4zpu: " + str(zpu3image4xi['COUNT3REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1 

        if zpu3image4xi['COUNT3REFERENCE'] <= 0:
            zpu3image4xi['COUNT3REFERENCE'] = 1
            message2print = "{(Step 6Z2/70) warning" + "; count2reference-warning-by-retain3image4zpu: " + " ** " + str(zpu3image4xi['COUNT3REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #   
            code3nack  = code3nack + '6Z2-'
        else:
            code3ack  = code3ack + '6D-'
                                
    except:
        try:
            zpu3image4xi['COUNT3REFERENCE'] = 1
        
            message2print = "{(Step 7Z3/70) warning" + "; count2reference-warning-by-retain3image4zpu: " + " ** " + str(zpu3image4xi['COUNT3REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code3nack  = code3nack + '7Z3-'
        
        except:
            message2print = "{(Step 7Z4/70) yielded" + "; zpu3image4xi-yielded-by-retain3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '7Z4-'                
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    zpu3image4xi ['ACK2RETURN']  = code3ack
    zpu3image4xi ['NACK2RETURN'] = code3nack     
    message2print = "{(Step 70B70) returning" + "; zpu3image4xi-returning-by-retain3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}, 'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041E5EB8>, 'WIDTH': 1100, 'DEPTH': 900, 'CONTEXT' ...
    #  'COUNT3REFERENCE': 1, 'EVENT': {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4zpu-failed ; time: 06:24:23 10/10/11 ...
    
    return zpu3image4xi         
    

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get3imageinfo4zpu" usage = "
#
#   task4getinfo3image = {}
#   task4getinfo3image ['DIMENSION'] = 3
#   task4getinfo3image ['WIDTH'] = 0
#   task4getinfo3image ['HEIGHT'] = 0
#   task4getinfo3image ['DEPTH'] = 0
#   response4getinfo3image = {}
#   print('(Step 12C/33) OF ZPU3IMAGE4XI5TEST - TRYING TO GET SELECTED 3D IMAGE INFO OF ZPU DEVICE ', time4xi())    
#   try:                               
#       response4getinfo3image = get3imageinfo4zpu (task4getinfo3image)                                                                                             
#       print('(Step 13D/33) OF ZPU3IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4getinfo3image)                           
#       #
#   except:
#       print('(Step 12Z2/33) OF ZPU3IMAGE4XI5TEST *** FAILED TO GET SELECTED 3D IMAGE INFO OF ZPU DEVICE ', time4xi()) 
#            
#">
# -------------------------------------------------------------------------------------------------   
def get3imageinfo4zpu(task4getinfo3image=None):

    '''
    gets zpu3image4xi
    '''
    
    global zpu3image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get3imageinfo4zpu"
    method2print = "get3imageinfo4zpu"

    message2print = "{(Step 0A/33) started" + "; get3imageinfo4zpu-started: " + str(get3imageinfo4zpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get3imageinfo4zpu at 0x000000000318FE48>                                                      
    
    message2print = "{(Step 0A/33) passed" + "; task4getinfo3image-passed-to-get3imageinfo4zpu: " + str(task4getinfo3image) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'WIDTH': 0, 'DEPTH': 0, 'DIMENSION': 3, 'HEIGHT': 0}
    
    image3zpu = {}
    try: 
        if zpu3image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE 3D IMAGE FOR ZPU           
            # -------------------------------------------------------------------------------------           
            image3zpu = create3image4zpu()
            message2print = "{(Step 1B3/33) initialized" + "; image3zpu-initialized-by-get3imageinfo4zpu to: " + str(image3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                           
        else:
            message2print = "{(Step 1B4/33) using" + "; zpu3image4xi-using-by-get3imageinfo4zpu: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'PIPE': (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>), 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-1 ...
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF THE IMAGE INFO OF ZPU          
        # -------------------------------------------------------------------------------------     
        dimension3xi = 0
        
        if task4getinfo3image is None:
            task4getinfo3image = {}
            task4getinfo3image ['DIMENSION'] = 3 
            task4getinfo3image ['DOMAIN'] = 'ALL'
            image3zpu = zpu3image4xi
            message2print = "{(Step 1C/33) using" + "; image3zpu-got-by-get3imageinfo4zpu: " + str(image3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        else:
            pass    

        keys3task4imageinfo = task4getinfo3image.keys()
        if 'DIMENSION' in keys3task4imageinfo:
            dimension3xi = task4getinfo3image['DIMENSION']
            if dimension3xi == 3:
                pass
            else:
                dimension3xi = 3    
        else:
            pass
            
        if 'DOMAIN' in keys3task4imageinfo:
            domain3xi = task4getinfo3image['DOMAIN']
            if domain3xi == 'ALL':
                pass
            else:
                domain3xi = 'ALL'

            keys3domain = zpu3image4xi.keys()
            for key3domain in keys3domain:
                task4getinfo3image[key3domain] = zpu3image4xi[key3domain]
                    
            message2print = "{(Step 1C/33) using" + "; task4getinfo3image-using-by-get3imageinfo4zpu: " + str(task4getinfo3image)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
        else:
            pass            
            
        keys3task = task4getinfo3image.keys()
        for key3task in keys3task:
            try:
                message2print = "{(Step 3D2/33) using" + "; key3task-using-by-get2image4zpu: " + str(key3task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # WIDTH; DEPTH; DIMENSION; HEIGHT 
                image3zpu [key3task] = zpu3image4xi[key3task]
                
                message2print = "{(Step 3D2/33) set" + "; info-of-image3zpu-set-by-get3imageinfo4zpu to: " + str(image3zpu [key3task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 800; 800; 800 
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key3task-yielded-by-get2image4zpu: " + str(key3task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # DIMENSION - OK 
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo3image-using-by-get2image4zpu: " + str(task4getinfo3image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; zpu3image4xi-using-by-get2image4zpu: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                           
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image3zpu = create3image4zpu()
            message2print = "{(Step 1F/33) initialized" + "; image3zpu-initialized-by-get3imageinfo4zpu to: " + str(image3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if task4getinfo3image is None:
                image3zpu = zpu3image4xi
                message2print = "{(Step 1G/33) got" + "; image3zpu-got-by-get3imageinfo4zpu: " + str(image3zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            else:
                keys3task = task4getinfo3image.keys()
                for key3task in keys3task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key3task-using-by-get2image4zpu: " + str(key3task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        image3zpu [key3task] = zpu3image4xi[key3task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-image3zpu-set-by-get3imageinfo4zpu to: " + str(image3zpu [key3task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key3task-yielded-by-get2image4zpu: " + str(key3task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                            
        except:
            image3zpu = {}
            message2print = "{(Step 1Z3/70) yielded" + "; image3zpu-yielded-by-get2image4zpu: " + str(image3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; image3zpu-returning-by-get3imageinfo4zpu: " + str(image3zpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    #SOME: {'WIDTH': 800, 'DEPTH': 800, 'HEIGHT': 800}
    #ALL:  {'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4zpu-failed ; time ...
    
    return image3zpu
            
#</function4xi> 
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get3supportedimageformats4zpu" usage = "
#
#    print('(Step 3A/33) OF ZPU3IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR ZPU', time4xi())  
#    try:
#        imageformats = get3supportedimageformats4zpu()                                                                       
#        print('(Step 3B/33) OF ZPU3IMAGE4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR ZPU: ', imageformats) 
#    except:
#        print('(Step 3Z/33) OF ZPU3IMAGE4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR ZPU', time4xi())
#            
#">
# -------------------------------------------------------------------------------------------------   
def get3supportedimageformats4zpu(task4supportedimageformats3zpu=None):  

    '''
    gets supported image formats for zpu
    '''
    
    global zpu3image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get3supportedimageformats4zpu"
    method2print = "get3supportedimageformats4zpu"

    message2print = "{(Step 0A/33) started" + "; get3supportedimageformats4zpu-started: " + str(get3supportedimageformats4zpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    # <function get3supportedimageformats4zpu at 0x000000000318FEC8>                                                          
    
    message2print = "{(Step 0B/33) passed" + "; task4supportedimageformats3zpu-passed-to-get3supportedimageformats4zpu: " + str(task4supportedimageformats3zpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'DIMENSION': 3} 
    
    
    if task4supportedimageformats3zpu is None:
        task4supportedimageformats3zpu = {} 
        task4supportedimageformats3zpu['DIMENSION'] = 3                                                             # 3D image is the default
    
    dimension3xi = 3
    try:
        keys2supportedimageformats = task4supportedimageformats3zpu.keys()    
        if 'DIMENSION' in keys2supportedimageformats:
            dimension3xi = task4supportedimageformats3zpu['DIMENSION']
            if ((dimension3xi <=0) or (dimension3xi > 3)): 
                dimension3xi = 3
        else:
            dimension3xi = 3
    except:
        pass        
    
    message2print = "{(Step 1A/33) using" + "; dimension3xi-using-by-dimension3xi: " + str(dimension3xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 
                            
    supportedimageformats3zpu = {} 
    try: 
        if zpu3image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image3zpu = create3image4zpu()                                                                          # create zpu3image4xi
            message2print = "{(Step 1C/33) initialized" + "; image3zpu-initialized-by-get3supportedimageformats4zpu to: " + str(image3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
        else:
            pass
                        
        # -------------------------------------------------------------------------------------
        # SUPPORTED IMAGE FORMATS FOR ZPU       
        # -------------------------------------------------------------------------------------     
        supportedimageformats3zpu = zpu3image4xi['FORMAT']
        message2print = "{(Step 1E/33) got" + "; supportedimageformats3zpu-got-by-get3supportedimageformats4zpu: " + str(supportedimageformats3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}; time: 05:18:11 10/10/11 Pacific Daylight Time}
            
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image3zpu = create3image4zpu()
            message2print = "{(Step 1F2/33) initialized" + "; image3zpu-initialized-by-get3supportedimageformats4zpu to: " + str(image3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
            
            # -------------------------------------------------------------------------------------
            # SUPPORTED IMAGE FORMATS FOR ZPU       
            # -------------------------------------------------------------------------------------     
            supportedimageformats3zpu = zpu3image4xi['FORMAT']
            message2print = "{(Step 1H/33) got" + "; supportedimageformats3zpu-got-by-get3supportedimageformats4zpu: " + str(supportedimageformats3zpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            #  
                                          
        except:
            message2print = "{(Step 1Z/70) yielded" + "; supportedimageformats3zpu-yielded-by-get2image4zpu: " + str(supportedimageformats3zpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; supportedimageformats3zpu-returning-by-get3supportedimageformats4zpu: " + str(supportedimageformats3zpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}; time: 05:18:11 10/10/11 Pacific Daylight Time}
    
    return supportedimageformats3zpu
            
#</function4xi> 

 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "wait3image4zpu" usage = "
#   print('=' *33)          
#   print('(Step 16A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO WAIT 3D IMAGE FOR ZPU ', time4xi())   
#   print('=' *33)      
#   try:
#       response4wait2zpu = wait3image4zpu()
#       keys3response = response4wait2zpu.keys()
#       print('(Step 16B/700) OF ZPU3IMAGE4XI5TEST - WAITED 3D IMAGE FOR ZPU ', time4xi()) 
#       for key3response in sorted(keys3response):
#           print(key3response, ' : ', response4wait2zpu[key3response])
#           # 
#   except:
#       print('(Step 16Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO WAIT 3D IMAGE FOR ZPU: ', wait2zpu)            
#">
# -------------------------------------------------------------------------------------------------   
def wait3image4zpu(task4wait3image=None):

    '''
    waits zpu3image4xi
    '''
    
    global zpu3image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "wait3image4zpu"
    method2print = "wait3image4zpu"

    message2print = "{(Step 1A1/33) started" + "; wait3image4zpu-started: " + str(wait3image4zpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function wait3image4zpu at 0x00000000031B7F48>  
    
    message2print = "{(Step 1A2/33) passed" + "; task4wait3image-passed-by-wait3image4zpu to: " + str(task4wait3image)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # None 
    
    try: 
        if zpu3image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE     
            # -------------------------------------------------------------------------------------           
            zpu3image4xi = create3image4zpu()
            
            message2print = "{(Step 1B/33) created" + "; zpu3image4xi-created-by-wait3image4zpu: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                                                
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue3image4zpu = None
            try:
                queue3image4zpu = zpu3image4xi['QUEUE']
                message2print = "{(Step 1C/33) got" + "; queue3image4zpu-got-by-wait3image4zpu to: " + str(queue3image4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z1/70) yielded" + "; zpu2xi-yielded-by-wait3image4zpu: " + str(zpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            size3image4zpu = 0
            try:                    
                size3image4zpu = queue3image4zpu.qsize()
                message2print = "{(Step 1D/33) got" + "; size3image4zpu-got-by-wait3image4zpu to: " + str(size3image4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
                               
                
            except:     
                message2print = "{(Step 1Z2/70) yielded" + "; zpu2xi-yielded-by-wait3image4zpu: " + str(zpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
             
            empty3image4zpu = True       
            try:                    
                empty3image4zpu= queue3image4zpu.empty()
                message2print = "{(Step 1E/33) got" + "; empty3image4zpu-got-by-wait3image4zpu to: " + str(empty3image4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z3/70) yielded" + "; empty3image4zpu-yielded-by-wait3image4zpu: " + str(empty3image4zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
             
            full3image4zpu = False       
            try:                                
                full3image4zpu = queue3image4zpu.full()
                message2print = "{(Step 1F/33) got" + "; full3image4zpu-got-by-wait3image4zpu to: " + str(full3image4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z4/70) yielded" + "; full3image4zpu-yielded-by-wait3image4zpu: " + str(full3image4zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                            
            message2print = "{(Step 1G/33) finished" + "; zpu3image4xi-finished-by-wait3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                            
        else:
            # -------------------------------------------------------------------------------------
            # USE THE EXISTING IMAGE FOR ZPU          
            # -------------------------------------------------------------------------------------     
            
            message2print = "{(Step 1H/33) used" + "; zpu3image4xi-used-by-wait3image4zpu: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'},
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041E5EB8>, 'WIDTH': 1100, 'DEPTH': 900, 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{( ...
            
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue3image4zpu = None
            try:
                queue3image4zpu = zpu3image4xi['QUEUE']
                message2print = "{(Step 1I/33) got" + "; queue3image4zpu-got-by-wait3image4zpu to: " + str(queue3image4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <queue4xi.Queue4Xi object at 0x00000000041E5EB8>
            except:     
                message2print = "{(Step 1Z5/70) yielded" + "; zpu2xi-yielded-by-wait3image4zpu: " + str(zpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait3image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                message2print = "{(Step 1J1/33) passed" + "; task4wait3image-passed-by-wait3image4zpu to: " + str(task4wait3image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                queue3task = {}    
                try:
                    queue3task = task4wait3image ['QUEUE']
                    message2print = "{(Step 1J2/33) set" + "; queue3task-set-by-wait3image4zpu to: " + str(queue3task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:     
                    message2print = "{(Step 1Z61/70) yielded" + "; queue3task-yielded-by-wait3image4zpu: " + str(queue3task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                try:                        
                    if queue3task == queue3image4zpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1K/33) accepted" + "; queue3task-accepted-by-wait3image4zpu: " + str(queue3task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        try:
                            message2print = "{(Step 1L/33) warned" + "; queue3task-warned-by-wait3image4zpu: " + " ** " + str(queue3task)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # 
                        
                            message2print = "{(Step 1M/33) warned" + "; queue3image4zpu-warned-by-wait3image4zpu: " + " ** " + str(queue3image4zpu)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                            # 
                        except:
                            pass
                                                     
                except:     
                    message2print = "{(Step 1Z62/70) yielded" + "; queue3task-yielded-by-wait3image4zpu: " + str(queue3task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            size3queue4zpu = 0        
            try:                    
                size3queue4zpu = queue3image4zpu.qsize()
                message2print = "{(Step 1N1/33) got" + "; size3queue4zpu-got-by-wait3image4zpu to: " + str(size3queue4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 2147483646 
                
                zpu3image4xi ['SIZE2QUEUE'] = size3queue4zpu
                message2print = "{(Step 1N2/33) set" + "; size2queue4image-got-by-wait3image4zpu to: " + str(zpu3image4xi ['SIZE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 2147483646
            except:     
                message2print = "{(Step 1Z7/70) yielded" + "; size3queue4zpu-yielded-by-wait3image4zpu: " + str(size3queue4zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            empty3queue4zpu = False
            try:                    
                empty3queue4zpu = queue3image4zpu.empty()
                message2print = "{(Step 1O1/33) got" + "; empty3queue4zpu-got-by-wait3image4zpu to: " + str(empty3queue4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # True
                
                zpu3image4xi ['EMPTY2QUEUE'] = empty3queue4zpu
                message2print = "{(Step 1O2/33) set" + "; empty2queue4image-got-by-wait3image4zpu to: " + str(zpu3image4xi ['EMPTY2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # True               
            except:     
                message2print = "{(Step 1Z8/70) yielded" + "; empty3queue4zpu-yielded-by-wait3image4zpu: " + str(empty3queue4zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            full3queue4zpu = False
            try:                                
                full3queue4zpu = queue3image4zpu.full()
                message2print = "{(Step 1P1/33) got" + "; full3queue4zpu-got-by-wait3image4zpu to: " + str(full3queue4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # False
                 
                zpu3image4xi ['FULL2QUEUE'] = full3queue4zpu
                message2print = "{(Step 1P2/33) set" + "; full2queue4image-got-by-wait3image4zpu to: " + str(zpu3image4xi ['FULL2QUEUE'])  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                # False                         
            except:     
                message2print = "{(Step 1Z9/70) yielded" + "; full3queue4zpu-yielded-by-wait3image4zpu: " + str(full3queue4zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            state3queue4zpu = None
            try:                                
                state3queue4zpu = queue3image4zpu.__getstate__()
                message2print = "{(Step 1P3/33) got" + "; state3queue4zpu-got-by-wait3image4zpu to: " + str(state3queue4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #  (2147483647, <read-only PipeConnection, handle 500>, <write-only PipeConnection, handle 504>, <Lock(owner=None)>, None, <BoundedSemaphore4Xi(value=1, maxvalue=2147483647)>, 2200); time: 06:24:23 10/10/11 Pacific Daylight Time}
                 
                zpu3image4xi ['STATE2QUEUE'] = state3queue4zpu
                message2print = "{(Step 1P4/33) set" + "; state2queue4image-set-by-wait3image4zpu to: " + str(zpu3image4xi ['STATE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # (2147483647, <read-only PipeConnection, handle 500>, <write-only PipeConnection, handle 504>, <Lock(owner=None)>, None, <BoundedSemaphore4Xi(value=1, maxvalue=2147483647)>, 2200); time: 06:24:23 10/10/11 Pacific Daylight Time}
   
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; state3queue4zpu-yielded-by-wait3image4zpu: " + str(state3queue4zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                                                            
            message2print = "{(Step 1Q/33) finished" + "; zpu3image4xi-finished-by-wait3image4zpu: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'SIZE2QUEUE': 2147483646, 'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}, 'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041E5EB8>, 'WIDTH': 1100, 'DEPTH': 900, 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4zpu', 'METHOD': 'create2context4zpu'}, 'FUNCTION2CALLBACK': <function trace2context4zpu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4zpu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2commandqueue4zpu', 'METHOD': 'create2commandqueue4zpu'}, 'FUNCTION2CALLBACK': <function trace2commandqueue4zpu at 0x00000000031517C8>, 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-', 'INFO2PRIVATE': None, 'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004387E10>, 'NACK2RETURN': ' ', 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_ZPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'PROPERTIES': None, 'PIPE': (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)}, 'STATE2QUEUE': (2147483647, <read-only PipeConnection, handle 500>, <write-only PipeConnection, handle 504>, <Lock(owner=None)>, None, <BoundedSemaphore4Xi(value=1, maxvalue=2147483647)>, 2200), 'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'EMPTY2QUEUE': True, 'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4apu', 'METHOD': 'create2buffer4apu'}, 'FUNCTION2CALLBACK': <function trace2buffer4apu at 0x0000000004AFFE48>, 'POINTER2BUFFER': <mmap.mmap object at 0x00000000041E0500>, 'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_ZPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'NACK2RETURN': ' ', 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6A-7A-8A-8E-9A-10A-11B-12B-13B-14B-15B-16B-17B-18B-19B-', 'SIZE': 278528}, 'ACK2RETURN': ' 6D-', 'HEIGHT': 1000, 'FULL2QUEUE': False, 'FLAGS': 0, 'ROWPITCH': 901, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_ZPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'CODE3ERROR': 0, 'POINTER': <mmap.mmap object at 0x00000000041E03F8>, 'CALLBACK': <function trace3image4apu at 0x00000000031B1848>, 'NACK2RETURN': ' ', 'FUNCTION3CALLBACK': <function trace3image4apu at 0x00000000031B1848>, 'SLICEPITCH': 902, 'DATA3CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create3image4apu', 'METHOD': 'create3image4apu'}, 'COUNT3REFERENCE': 1, 'EVENT': {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2event4apu', 'METHOD': 'create2event4apu'}, 'FUNCTION2CALLBACK': <function trace2event4apu at 0x000000000315DCC8>, 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-', 'QUEUE': <queue4xi.Queue4Xi object at 0x000000000465C630>, 'NACK2RETURN': ' ', 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2commandqueue4apu', 'METHOD': 'create2commandqueue4apu'}, 'FUNCTION2CALLBACK': <function trace2commandqueue4apu at 0x00000000031517C8>, 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-', 'INFO2PRIVATE': None, 'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C8A0B8>, 'NACK2RETURN': ' ', 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_ZPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'PROPERTIES': None, 'PIPE': (<read-write PipeConnection, handle 488>, <read-write PipeConnection, handle 492>)}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_ZPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'PIPE': (<read-write PipeConnection, handle 456>, <read-write PipeConnection, handle 460>)}}; time: 06:24:23 10/10/11 Pacific Daylight Time}

    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            zpu3image4zpu = create3image4zpu()
            message2print = "{(Step 1R/33) got" + "; zpu3image4xi-got-by-wait3image4zpu to: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue3image4zpu = None
            try:
                queue3image4zpu = zpu3image4xi['QUEUE']
                message2print = "{(Step 1S/33) got" + "; queue3image4zpu-got-by-wait3image4zpu to: " + str(queue3image4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; queue3image4zpu-yielded-by-wait3image4zpu: " + str(queue3image4zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait3image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE QUEUE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                queue3task = None    
                try:
                    queue3task = task4wait3image ['QUEUE']
                    message2print = "{(Step 1T/33) passed" + "; queue3task-passed-by-wait3image4zpu to: " + str(queue3task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    if queue3task == queue3image4zpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1U/33) accepted" + "; queue3task-accepted-by-wait3image4zpu: " + str(queue3task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        message2print = "{(Step 1X/33) warned" + "; queue3task-warned-by-wait3image4zpu: " + " ** " + str(queue3task)  + " ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        
                        message2print = "{(Step 1Y1/33) warned" + "; queue3image4zpu-warned-by-wait3image4zpu: " + " ** " + str(queue3image4zpu)  + " ** " + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print)    
                        #                      
                except:     
                    message2print = "{(Step 1Z11/70) yielded" + "; queue3task-yielded-by-wait3image4zpu: " + str(queue3task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            
            size3image4zpu = 0                             
            try:                    
                size3image4zpu = queue3image4zpu.qsize()
                message2print = "{(Step 1Y2/33) got" + "; size3image4zpu-got-by-wait3image4zpu to: " + str(size3image4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z12/70) yielded" + "; size3image4zpu-yielded-by-wait3image4zpu: " + str(size3image4zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            empty3image4zpu = True          
            try:                    
                empty3image4zpu = queue3image4zpu.empty()
                message2print = "{(Step 1Y3/33) got" + "; empty3image4zpu-got-by-wait3image4zpu to: " + str(empty3image4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z13/70) yielded" + "; zpu2xi-yielded-by-wait3image4zpu: " + str(zpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            full3image4zpu = False          
            try:                                
                full3image4zpu= queue3image4zpu.full()
                message2print = "{(Step 1Y4/33) got" + "; full3image4zpu-got-by-wait3image4zpu to: " + str(full3image4zpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z14/70) yielded" + "; full3image4zpu-yielded-by-wait3image4zpu: " + str(full3image4zpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                                        
            message2print = "{(Step 1Y5/33) finished" + "; zpu3image4xi-finished-by-wait3image4zpu: " + str(zpu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
        except:
            zpu3image4zpu= {}
            message2print = "{(Step 1Z15/70) yielded" + "; zpu3image4xi-yielded-by-wait3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; zpu3image4xi-returning-by-wait3image4zpu: " + str(zpu3image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'SIZE2QUEUE': 2147483646, 'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}, 'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041E5EB8>, 'WIDTH': 1100, 'DEPTH': 900, 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4zpu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4zpu', 'METHOD': 'create2context4zpu'}, 'FUNCTION2CALLBACK': <function trace2context4zpu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4zpu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2commandqueue4zpu', 'METHOD': 'create2commandqueue4zpu'}, 'FUNCTION2CALLBACK': <function trace2commandqueue4zpu at 0x00000000031517C8>, 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-', 'INFO2PRIVATE': None, 'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004387E10>, 'NACK2RETURN': ' ', 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_ZPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'PROPERTIES': None, 'PIPE': (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)}, 'STATE2QUEUE': (2147483647, <read-only PipeConnection, handle 500>, <write-only PipeConnection, handle 504>, <Lock(owner=None)>, None, <BoundedSemaphore4Xi(value=1, maxvalue=2147483647)>, 2200), 'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'EMPTY2QUEUE': True, 'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4apu', 'METHOD': 'create2buffer4apu'}, 'FUNCTION2CALLBACK': <function trace2buffer4apu at 0x0000000004AFFE48>, 'POINTER2BUFFER': <mmap.mmap object at 0x00000000041E0500>, 'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_ZPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'NACK2RETURN': ' ', 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6A-7A-8A-8E-9A-10A-11B-12B-13B-14B-15B-16B-17B-18B-19B-', 'SIZE': 278528}, 'ACK2RETURN': ' 6D-', 'HEIGHT': 1000, 'FULL2QUEUE': False, 'FLAGS': 0, 'ROWPITCH': 901, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_ZPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'CODE3ERROR': 0, 'POINTER': <mmap.mmap object at 0x00000000041E03F8>, 'CALLBACK': <function trace3image4apu at 0x00000000031B1848>, 'NACK2RETURN': ' ', 'FUNCTION3CALLBACK': <function trace3image4apu at 0x00000000031B1848>, 'SLICEPITCH': 902, 'DATA3CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create3image4apu', 'METHOD': 'create3image4apu'}, 'COUNT3REFERENCE': 1, 'EVENT': {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2event4apu', 'METHOD': 'create2event4apu'}, 'FUNCTION2CALLBACK': <function trace2event4apu at 0x000000000315DCC8>, 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-', 'QUEUE': <queue4xi.Queue4Xi object at 0x000000000465C630>, 'NACK2RETURN': ' ', 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2commandqueue4apu', 'METHOD': 'create2commandqueue4apu'}, 'FUNCTION2CALLBACK': <function trace2commandqueue4apu at 0x00000000031517C8>, 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-', 'INFO2PRIVATE': None, 'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C8A0B8>, 'NACK2RETURN': ' ', 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_ZPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'PROPERTIES': None, 'PIPE': (<read-write PipeConnection, handle 488>, <read-write PipeConnection, handle 492>)}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_ZPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'PIPE': (<read-write PipeConnection, handle 456>, <read-write PipeConnection, handle 460>)}}; time: 06:24:23 10/10/11 Pacific Daylight Time}
    
    return zpu3image4xi
    
   
#</function4xi> 
                                               
# =================================================================================================
#<class4xi class = "ZPU3Image4Xi" base = "object" child = "None"
#   synopsis = "offers  methods for 3D ZPU images" usage = "3D Image4Xi of ZPU"
#>
# =================================================================================================
class ZPU3Image4Xi(object): 
    ''' 
    ZPU3Image4Xi offers  methods for 3D ZPU images
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable3zpu = cpu_corecount4xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'create3image',                    # ACK 5.31   p76    
            'copy3image',                      # ACK 5.31   p88
            'copy3imagetobuffer',              # ACK 5.3.4  p90
            'copy3buffertoimage',              # ACK 5.3.4  p92
            'read3image',                      # ACK 5.3.3  p84
            'write3image',                     # ACK 5.3.3  p84            
            'map3image',                       # ACK 5.3.5  p94            
            'get3imageinfo',                   # ACK 5.3.6  p97
            'get3supportedimageformats',       # ACK 5.3.2  p82 nb: a separate method to get format whereas using get3imageinfo you can access any element by key such as 'FORMAT'   
            'retain3image',                    # ACK 5.4.1  p99 nb: for compability to OpenCL - you can get any element of dictionary by get3imageinfo
            'release3image',                   # ACK 5.4.1  p99
            'trace3image',                     # ACK modernized 
            'set3imagecallback',               # ACK modernized
            'set3image',                       # ACK modernized nb: with memorymap features you can process 3D images as files!                    
            'wait3image',                      # ACK modernized nb: checks Queue object and thereby generates delay   
            'sense',                           # ACK modernized  
            'reason',                          # ACK modernized  
            'ace')                             # ACK modernized           
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-ZPU3Image4Xi" usage = "
    #   print('(Step 5A/33) OF ZPU3IMAGE4XI5TEST - CREATING 3D IMAGE OBJECT FOR ZPU ', time4xi())     
    #   try:
    #       imageobject3zpu = ZPU3Image4Xi()
    #       print('(Step 5B/33) OF ZPU3IMAGE4XI5TEST - CREATED 3D IMAGE OBJECT FOR ZPU: ', imageobject3zpu)                                  
    #   except:
    #       print('(Step 5Z/33) OF ZPU3IMAGE4XI5TEST *** FAILED TO CREATE 3D IMAGE OBJECT FOR ZPU')
    #">
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4imagecreate3zpu = None):
        '''
        _init__ method initializes ZPU3Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "ZPU3Image4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-ZPU3Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 

        message2print = "{(Step 1B/33) borning" + "; zpu3image3xi-of-ZPU3Image4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-zpu3image3xi-of-ZPU3Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable3zpu-sharing-by-zpu3image3xi-of-ZPU3Image4Xi: " + str(ZPU3Image4Xi.classvariable3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #           
                                
        message2print = "{(Step 1E/33) passed" + "; task4imagecreate3zpu-passed-to-init-of-ZPU3Image4Xi: " + str(task4imagecreate3zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #                       
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) passing" + "; task4imagecreate3zpu-passing-to-create3image4zpu-by-init-of-ZPU3Image4Xi: " + str(task4imagecreate3zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        zpu3image3xi = create3image4zpu(task4imagecreate3zpu)
        
        message2print = "{(Step 2B/33) performed" + "; create3image4zpu-performed-by-init-of-ZPU3Image4Xi: " + str(create3image4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; zpu3image3xi-of-ZPU3Image4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-zpu3image3xi-of-ZPU3Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable3zpu-shared-by-zpu3image3xi-of-ZPU3Image4Xi: " + str(ZPU3Image4Xi.classvariable3zpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #          
       
        message2print = "{(Step 3D/33) returning" + "; init-of-ZPU3Image4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create3image-of-ZPU3Image4Xi" usage = "
    #   print('(Step 23A/33) OF ZPU3IMAGE4XI5TEST - TRYING TO CREATE ZPU IMAGE', time4xi()) 
    #   try:
    #       xi3zpu = ZPU3Image4Xi()                            
    #       image3zpu = xi3zpu.create3image()                                  
    #       print('(Step 23B/33) OF ZPU3IMAGE4XI5TEST - CREATED ZPU IMAGE: ', image3zpu)
    #   except:
    #       print('(Step 23Z/33) OF ZPU3IMAGE4XI5TEST *** FAILED TO CREATE ZPU IMAGE ***', time4xi())                  
    #                     
    #">   
    # ---------------------------------------------------------------------------------------------   
    def create3image(self, task4image2create = None):
        '''
        create3image method creates 3D image objects of ZPU3Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "create3image"
                     
        message2print = "{(Step 1A/33) started" + "; create3image-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4image2create-passed-to-create-of-ZPU3Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4image2create-passing-to-create3image4zpu-by-create3image-of-ZPU3Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #    
  
        zpu3image3xi = create3image4zpu(task4image2create)

        message2print = "{(Step 2B/33) performed" + "; create3image4zpu-performed-by-create3image-of-ZPU3Image4Xi: " + str(create3image4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; zpu3image3xi-returning-by-create3image-of-ZPU3Image4Xi: " + str(zpu3image3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # 

        return zpu3image3xi
                                
    #                        
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set3image-of-ZPU3Image4Xi" usage = "
    #   print('(Step 1A/33) OF ZPU3IMAGE4XI - TRYING TO SET 3D IMAGE FOR ZPU')
    #   try:
    #       cc3xi = ZPU3Image4Xi()                            
    #       image3zpu = cc3xi.set3image()                                  
    #       print('(Step 1B/33) OF ZPU4XI5TEST - SET 3D IMAGE FOR ZPU: ', image3zpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF ZPU4XI5TEST *** FAILED TO SET 3D IMAGE FOR ZPU ***')  
    #">
    # ---------------------------------------------------------------------------------------------   
    def set3image(self, task4set3image = None):
        '''
        sets 3D image ZPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "set3image"
                     
        message2print = "{(Step 1A/33) started" + "; set3image-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4set3image-passed-to-set3image-of-ZPU3Image4Xi: " + str(task4set3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set3image-passing-to-set3image4zpu-by-set3image-of-ZPU3Image4Xi: " + str(task4set3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        zpu3image3xi = set3image4zpu(task4set3image)

        message2print = "{(Step 2B/33) performed" + "; set3image4zpu-performed-by-set3image-of-ZPU3Image4Xi: " + str(set3image4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; zpu3image3xi-returning-by-set3image-of-ZPU3Image4Xi: " + str(zpu3image3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #  
                
        return zpu3image3xi
                                
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get3imageinfo-of-ZPU3Image4Xi" usage = "
    #   print('(Step 1A/33) OF ZPU3IMAGE4XI - TRYING TO GET ZPU 3D IMAGE')
    #   try:
    #       cc3xi = ZPU3Image4Xi()                            
    #       image3zpu = cc3xi.get3imageinfo()                                  
    #       print('(Step 1B/33) OF ZPU4XI5TEST - GOT ZPU 3D IMAGE: ', image3zpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF ZPU4XI5TEST *** FAILED TO GET ZPU 3D IMAGE ***')   
    # ">
    # ---------------------------------------------------------------------------------------------   
    def get3imageinfo(self, task4getinfo3image=None):
        '''
        get3imageinfo method gets 3D objects of ZPU3Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "get3imageinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get3imageinfo-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        zpu3image3xi = get3imageinfo4zpu(task4getinfo3image)
        
        message2print = "{(Step 2A/33) returning" + "; zpu3image3xi-returning-by-get3imageinfo-of-ZPU3Image4Xi: " + str(zpu3image3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
        return zpu3image3xi
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get3supportedimageformats-of-ZPU3Image4Xi" usage = "
    #   print('(Step 6B1/33) OF ZPU3IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR ZPU')
    #   try:
    #       cc3xi = ZPU3Image4Xi()                            
    #       supportedimageformats3zpu = cc3xi.get3supportedimageformats()                                  
    #       print('(Step 6B2/33) OF ZPU4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR ZPU: ', supportedimageformats3zpu)
    #                                                   
    #   except:
    #       print('(Step 2Z/33) OF ZPU4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR ZPU***')     
    # ">
    # ---------------------------------------------------------------------------------------------   
    def get3supportedimageformats(self):
        '''
        get3supportedimageformats method gets supported imageformats by objects of ZPU3Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "get3supportedimageformats"
                     
        message2print = "{(Step 1A/33) started" + "; get3supportedimageformats-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        supportedimageformats3zpu = get3supportedimageformats4zpu()
        
        message2print = "{(Step 2A/33) returning" + "; supportedimageformats3zpu-returning-by-get3imageinfo-of-ZPU3Image4Xi: " + str(supportedimageformats3zpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
        return supportedimageformats3zpu
                            
    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "read3image-of-ZPU3Image4Xi" usage = "
    #   print('(Step 127A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi()) 
    #   response4read3zpu = {}
    #   try:
    #       response4read3zpu = xi3zpu.read3image()
    #       print('(Step 127B/700) OF ZPU3IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH READ IMAGE OBJECT, RESPONSE4READ2ZPU: ', response4read3zpu) 
    #   except:
    #       print('(Step 127Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi())
    #   ... continue as with read3image4zpu function
    #">
    # ---------------------------------------------------------------------------------------------   
    def read3image(self, task4read3image = None):
        '''
        reads ZPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "read3image"
                     
        message2print = "{(Step 1A/33) started" + "; read3image-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4read3image-passed-to-read3image-of-ZPU3Image4Xi: " + str(task4read3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4read3image-passing-to-read3image4zpu-by-read3image-of-ZPU3Image4Xi: " + str(task4read3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        response4readimage3zpu = read3image4zpu(task4read3image)

        message2print = "{(Step 2B/33) performed" + "; read3image4zpu-performed-by-read3image-of-ZPU3Image4Xi: " + str(read3image4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4readimage3zpu-returning-by-read3image-of-ZPU3Image4Xi: " + str(response4readimage3zpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
  
        return response4readimage3zpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map3image-of-ZPU3Image4Xi" usage = "
    #   print('(Step 127A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi()) 
    #   response4map3zpu = {}
    #   try:
    #       response4map3zpu = xi3zpu.map3image()
    #       print('(Step 127B/700) OF ZPU3IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH MAP IMAGE OBJECT, RESPONSE4READ2ZPU: ', response4map3zpu) 
    #   except:
    #       print('(Step 127Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi())
    #   ... continue as with map3image4zpu function
    #">
    # ---------------------------------------------------------------------------------------------   
    def map3image(self, task4map3image = None):
        '''
        maps ZPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "map3image"
                     
        message2print = "{(Step 1A/33) started" + "; map3image-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4map3image-passed-to-map3image-of-ZPU3Image4Xi: " + str(task4map3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4map3image-passing-to-map3image4zpu-by-map3image-of-ZPU3Image4Xi: " + str(task4map3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        response4mapimage3zpu = map3image4zpu(task4map3image)

        message2print = "{(Step 2B/33) performed" + "; map3image4zpu-performed-by-map3image-of-ZPU3Image4Xi: " + str(map3image4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4mapimage3zpu-returning-by-map3image-of-ZPU3Image4Xi: " + str(response4mapimage3zpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        return response4mapimage3zpu
                                                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release3image-of-ZPU3Image4Xi" usage = "
    #   print('(Step 1A/33) OF ZPU3IMAGE4XI - TRYING TO RELEASE 3D IMAGE FOR ZPU')
    #   try:
    #       cc3xi = ZPU3Image4Xi()                            
    #       image3zpu = cc3xi.release3image()                                  
    #       print('(Step 1B/33) OF ZPU4XI5TEST - RELEASED 3D IMAGE FOR ZPU: ', image3zpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF ZPU4XI5TEST *** FAILED TO RELEASE 3D IMAGE FOR ZPU ***')  
    #">
    # ---------------------------------------------------------------------------------------------   
    def release3image(self, task4release3image = None):
        '''
        releases ZPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "release3image"
                     
        message2print = "{(Step 1A/33) started" + "; release3image-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release3image-passed-to-release3image-of-ZPU3Image4Xi: " + str(task4release3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release3image-passing-to-release3image4zpu-by-release3image-of-ZPU3Image4Xi: " + str(task4release3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        zpu3image3xi = release3image4zpu(task4release3image)

        message2print = "{(Step 2B/33) performed" + "; release3image4zpu-performed-by-release3image-of-ZPU3Image4Xi: " + str(release3image4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; zpu3image3xi-returning-by-release3image-of-ZPU3Image4Xi: " + str(zpu3image3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                          
                
        return zpu3image3xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain3image-of-ZPU3Image4Xi" usage = "
    #   print('(Step 1A/33) OF ZPU3IMAGE4XI - TRYING TO RETAIN 3D IMAGE FOR ZPU')
    #   try:
    #       cc3xi = ZPU3Image4Xi()                            
    #       image3zpu = cc3xi.retain3image()                                  
    #       print('(Step 1B/33) OF ZPU4XI5TEST - RETAINED 3D IMAGE FOR ZPU: ', image3zpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF ZPU4XI5TEST *** FAILED TO RETAIN 3D IMAGE FOR ZPU ***')  
    #">
    # ---------------------------------------------------------------------------------------------   
    def retain3image(self, task4retain3image = None):
        '''
        retains ZPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "retain3image"
                     
        message2print = "{(Step 1A/33) started" + "; retain3image-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain3image-of-ZPU3Image4Xi: " + str(task4retain3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain3image4zpu-by-retain-of-ZPU3Image4Xi: " + str(task4retain3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        zpu3image3xi = retain3image4zpu(task4retain3image)

        message2print = "{(Step 2B/33) performed" + "; retain3image4zpu-performed-by-retain3image-of-ZPU3Image4Xi: " + str(retain3image4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; zpu3image3xi-returning-by-retain3image-of-ZPU3Image4Xi: " + str(zpu3image3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
              
                
        return zpu3image3xi
                                                           
    #</methodize4xi>   
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait3image-of-ZPU3Image4Xi" usage = "
    #   print('(Step 1A/33) OF ZPU3IMAGE4XI - TRYING TO WAIT ZPU 3D IMAGE')
    #   try:
    #       cc3xi = ZPU3Image4Xi()                            
    #       image3zpu = cc3xi.wait3image()                                  
    #       print('(Step 1B/33) OF ZPU4XI5TEST - WAITED ZPU 3D IMAGE: ', image3zpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF ZPU4XI5TEST *** FAILED TO WAIT ZPU 3D IMAGE ***')  
    #">
    # ---------------------------------------------------------------------------------------------   
    def wait3image(self, task4wait3image = None):
        '''
        waits ZPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "wait3image"
                     
        message2print = "{(Step 1A/33) started" + "; wait3image-of-ZPU3Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4wait3image-passed-to-wait3image-of-ZPU3Image4Xi: " + str(task4wait3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4wait-passing-to-wait3image4zpu-by-wait3image-of-ZPU3Image4Xi: " + str(task4wait3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        zpu3image3xi = wait3image4zpu(task4wait3image)

        message2print = "{(Step 2B/33) performed" + "; wait3image4zpu-performed-by-wait3image-of-ZPU3Image4Xi: " + str(wait3image4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; zpu3image3xi-returning-by-wait3image-of-ZPU3Image4Xi: " + str(zpu3image3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return zpu3image3xi
                                                           
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "write3image-of-ZPU3Image4Xi" usage = "
    #   print('(Step 127A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4write3zpu = {}
    #   try:
    #       response4write3zpu = xi3zpu.write3image()
    #       print('(Step 127B/700) OF ZPU3IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2ZPU: ', response4write3zpu) 
    #   except:
    #       print('(Step 127Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   ... continue as with write3image4zpu function
    #">
    # ---------------------------------------------------------------------------------------------   
    def write3image(self, task4write3image = None):
        '''
        writes ZPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "write3image"
                     
        message2print = "{(Step 1A/33) started" + "; write3image-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4write3image-passed-to-write3image-of-ZPU3Image4Xi: " + str(task4write3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4write3image-passing-to-write3image4zpu-by-write3image-of-ZPU3Image4Xi: " + str(task4write3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        response4writeimage3zpu = write3image4zpu(task4write3image)

        message2print = "{(Step 2B/33) performed" + "; write3image4zpu-performed-by-write3image-of-ZPU3Image4Xi: " + str(write3image4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4writeimage3zpu-returning-by-write3image-of-ZPU3Image4Xi: " + str(response4writeimage3zpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #    
                
        return response4writeimage3zpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy3image-of-ZPU3Image4Xi" usage = "
    #   print('(Step 127A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4copy3zpu = {}
    #   try:
    #       response4copy3zpu = xi3zpu.copy3image()
    #       print('(Step 127B/700) OF ZPU3IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2ZPU: ', response4copy3zpu) 
    #   except:
    #       print('(Step 127Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   ... continue as with copy3image4zpu function
    #">
    # ---------------------------------------------------------------------------------------------   
    def copy3image(self, task4copy3image = None):
        '''
        copies ZPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "copy3image"
                     
        message2print = "{(Step 1A/33) started" + "; copy3image-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy3image-passed-to-copy3image-of-ZPU3Image4Xi: " + str(task4copy3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #  
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy3image-passing-to-copy3image4zpu-by-copy3image-of-ZPU3Image4Xi: " + str(task4copy3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        response4copyimage3zpu = copy3image4zpu(task4copy3image)

        message2print = "{(Step 2B/33) performed" + "; copy3image4zpu-performed-by-copy3image-of-ZPU3Image4Xi: " + str(copy3image4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage3zpu-returning-by-copy3image-of-ZPU3Image4Xi: " + str(response4copyimage3zpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
        return response4copyimage3zpu
                                                           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy3imagetobuffer-of-ZPU3Image4Xi" usage = "
    #   print('(Step 127A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER OBJECT', time4xi()) 
    #   response4copyimagetobuffer = {}
    #   try:
    #       response4copyimagetobuffer = xi3zpu.copy3imagetobuffer()
    #       print('(Step 127B/700) OF ZPU3IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER  OBJECT, RESPONSE4WRITE2ZPU: ', response4copyimagetobuffer) 
    #   except:
    #       print('(Step 127Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER ', time4xi()) 
    #   ... continue as with copy3imagetobuffer4zpu function
    #">
    # ---------------------------------------------------------------------------------------------   
    def copy3imagetobuffer(self, task4copy3imagetobuffer = None):
        '''
        copies 3D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "copy3imagetobuffer"
                     
        message2print = "{(Step 1A/33) started" + "; copy3imagetobuffer-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy3imagetobuffer-passed-to-copy3imagetobuffer-of-ZPU3Image4Xi: " + str(task4copy3imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy3imagetobuffer-passing-to-copy3imagetobuffer4zpu-by-copy3imagetobuffer-of-ZPU3Image4Xi: " + str(task4copy3imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        response4copyimage3zpu = copy3imagetobuffer4zpu(task4copy3imagetobuffer)

        message2print = "{(Step 2B/33) performed" + "; copy3imagetobuffer4zpu-performed-by-copy3imagetobuffer-of-ZPU3Image4Xi: " + str(copy3imagetobuffer4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage3zpu-returning-by-copy3imagetobuffer-of-ZPU3Image4Xi: " + str(response4copyimage3zpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
          
        return response4copyimage3zpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy3buffertoimage-of-ZPU3Image4Xi" usage = "
    #   print('(Step 127A/700) OF ZPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE OBJECT', time4xi()) 
    #   response4copybuffertoimage = {}
    #   try:
    #       response4copybuffertoimage = xi3zpu.copy3buffertoimage()
    #       print('(Step 127B/700) OF ZPU3IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE  OBJECT, RESPONSE4WRITE2ZPU: ', response4copybuffertoimage) 
    #   except:
    #       print('(Step 127Z1/700) OF ZPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE ', time4xi()) 
    #   ... continue as with copy3buffertoimage4zpu function
    #">
    # ---------------------------------------------------------------------------------------------   
    def copy3buffertoimage(self, task4copy3buffertoimage = None):
        '''
        copies 3D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "copy3buffertoimage"
                     
        message2print = "{(Step 1A/33) started" + "; copy3buffertoimage-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy3buffertoimage-passed-to-copy3buffertoimage-of-ZPU3Image4Xi: " + str(task4copy3buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy3buffertoimage-passing-to-copy3buffertoimage4zpu-by-copy3buffertoimage-of-ZPU3Image4Xi: " + str(task4copy3buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        response4copyimage3zpu = copy3buffertoimage4zpu(task4copy3buffertoimage)

        message2print = "{(Step 2B/33) performed" + "; copy3buffertoimage4zpu-performed-by-copy3buffertoimage-of-ZPU3Image4Xi: " + str(copy3buffertoimage4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage3zpu-returning-by-copy3buffertoimage-of-ZPU3Image4Xi: " + str(response4copyimage3zpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 

                
        return response4copyimage3zpu
                                                           
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace3image-of-ZPU3Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 15A/25) OF ZPU3IMAGE4XI5TEST - TRYING TO TRACE ZPU: ', time4xi())      
    #   print('=' *33)      
    #   trace3zpu = None
    #   try:
    #       trace3zpu = trace3image4zpu() 
    #       print('(Step 15B/25) OF ZPU3IMAGE4XI5TEST - TRACE3ZPU: ', trace3zpu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace3image4zpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace3image4zpu', 'METHOD': 'trace3image4zpu'}
    #    except:
    #       print('(Step 15Z1/25) OF ZPU3IMAGE4XI5TEST *** FAILED TO TRACE ZPU: ', trace3zpu)     
    #">
    # ---------------------------------------------------------------------------------------------   
    def trace3image(self, task4trace3image = None):
        '''
        traces ZPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "trace3image"
                     
        message2print = "{(Step 1A/33) started" + "; trace3image-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4trace3image-passed-to-trace-of-ZPU3Image4Xi: " + str(task4trace3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #  
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace3image-passing-to-trace3image4zpu-by-trace3image-of-ZPU3Image4Xi: " + str(task4trace3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        trace3xi = trace3image4zpu(task4trace3image)

        message2print = "{(Step 2B/33) performed" + "; trace3image4zpu-performed-by-trace3image-of-ZPU3Image4Xi: " + str(trace3image4zpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace3xi-returning-by-trace3image-of-ZPU3Image4Xi: " + str(trace3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
                
        return trace3xi
                                                           
    #</methodize4xi>       


    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set3imagecallback-of-ZPU3Image4Xi" usage = "
    #   print('(Step 1A/33) OF ZPU3IMAGE4XI - TRYING TO SET 3D IMAGE FOR ZPU')
    #   try:
    #       cc3xi = ZPU3Image4Xi()                            
    #       image3zpu = cc3xi.set3imagecallback()                                  
    #       print('(Step 1B/33) OF ZPU4XI5TEST - SET 3D IMAGE FOR ZPU: ', image3zpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF ZPU4XI5TEST *** FAILED TO SET 3D IMAGE FOR ZPU ***')  
    #">
    # ---------------------------------------------------------------------------------------------   
    def set3imagecallback(self, task4set3imagecallback = None): 
        '''
        set3imagecallback of ZPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "ZPU3Image4Xi"
        method2print = "set3imagecallback"
                     
        message2print = "{(Step 1A/33) started" + "; set3imagecallback-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4set3imagecallback-passed-to-set3imagecallback-of-ZPU3Image4Xi: " + str(task4set3imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #   
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set3imagecallback-passing-to-set3imagecallback -by-set3imagecallback-of-ZPU3Image4Xi: " + str(task4set3imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        callback3zpu4image = {} 
        try:
            callback3zpu4image = set3imagecallback4zpu (task4set3imagecallback)

            message2print = "{(Step 2B/33) callback3zpu4image" + "; callback3zpu4image-set-by-set3imagecallback-of-ZPU3Image4Xi to: " + str(callback3zpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # 
            
        except:
            message2print = "{(Step 4A/33) failed" + "; callback3zpu4image-failed-by-set3imagecallback-of-ZPU3Image4Xi: " + str(callback3zpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback3zpu4image-returning-by-set3imagecallback-of-ZPU3Image4Xi: " + str(callback3zpu4image) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
                                             
        return callback3zpu4image
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-ZPU3Image4Xi" usage = "
    #   try:
    #       print(xi3zpu.sense())
    #       print('(Step 24B/33) OF ZPU3IMAGE4XI5TEST - SENSED ', time4xi()) 
    #   except:
    #       print('(Step 24Z/33) OF ZPU3IMAGE4XI5TEST *** FAILED TO SENSE ', time4xi())  
    # ">
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses ZPU3Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'ZPU3Image4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; ZPU3Image4Xi-sensing-by-sense-of-ZPU3Image4Xi: " + str(ZPU3Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #    
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-ZPU3Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable3zpu-of-ZPU3Image4Xi'] = ZPU3Image4Xi.classvariable3zpu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-ZPU3Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; ZPU3Image4Xi-sensed-by-sense-of-ZPU3Image4Xi: " + str(ZPU3Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # 
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-ZPU3Image4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        #             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-ZPU3Image4Xi" usage = "
    #   print('(Step 25A/33) OF ZPU3IMAGE4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(xi3zpu.reason())
    #       print('(Step 25B/33) OF ZPU3IMAGE4XI5TEST - REASONED ', time4xi())
    #   except:
    #       print('(Step 25Z/33) OF ZPU3IMAGE4XI5TEST *** FAILED TO REASON ', time4xi())   
    # ">
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons ZPU3Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'ZPU3Image4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; ZPU3Image4Xi-reasoning-by-reason-of-ZPU3Image4Xi: " + str(ZPU3Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-ZPU3Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-ZPU3Image4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-ZPU3Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; ZPU3Image4Xi-reasoned-by-reason-of-ZPU3Image4Xi: " + str(ZPU3Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-ZPU3Image4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        #               
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-ZPU3Image4Xi" usage = "
    #   print('(Step 26A/33) OF ZPU3IMAGE4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(xi3zpu.ace())
    #       print('(Step 26B/33) OF ZPU3IMAGE4XI5TEST - ACED ', time4xi())                                                 
    #   except:
    #       print('(Step 26Z/33) OF ZPU3IMAGE4XI5TEST *** FAILED TO ACE ', time4xi())                                       
    #    
    # ">
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces ZPU3Image4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'ZPU3Image4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-ZPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; ZPU3Image4Xi-acing-by-ace-of-ZPU3Image4Xi: " + str(ZPU3Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #       
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-ZPU3Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-ZPU3Image4Xi'] = ZPU3Image4Xi.classvariable3zpu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-ZPU3Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; ZPU3Image4Xi-aced-by-ace-of-ZPU3Image4Xi: " + str(ZPU3Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-ZPU3Image4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   zpu3image4xi5test.txt
#   sense-zpu3image4xi5test.xml
#   reason-zpu3image4xi5test.xml
#   ace-zpu3image4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of zpu3image4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
